DROP TABLE IF EXISTS CommunityTranslation;
CREATE TABLE CommunityTranslation (
	translationId INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
	word_DE TINYTEXT NOT NULL,
	article_DE TINYTEXT,
	other_DE TEXT,
	word_NO TINYTEXT NOT NULL,
	article_NO TINYTEXT,
	other_NO TEXT,
	grade INTEGER,
	category TINYTEXT,
        -- additional parts
        forumUser INTEGER UNSIGNED NOT NULL,
        comment TEXT,
        flag ENUM('enabled','deleted','accepted','rejected') NOT NULL,
        timestamp TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
        address TINYTEXT,
        originalTranslationId INTEGER UNSIGNED,
	PRIMARY KEY (translationId),
        FOREIGN KEY (originalTranslationId) REFERENCES CommunityTranslation(translationId)
);

-- select * from CommunityTranslation group by originalTranslationId having timestamp = max(timestamp) order by timestamp limit 10;

-- below follow copied parts of dictionary.sql
DROP TABLE IF EXISTS CommunityIndexWord_DE;
CREATE TABLE CommunityIndexWord_DE (
	indexWordId INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
	indexWord TINYBLOB NOT NULL,
	translationId INTEGER UNSIGNED NOT NULL REFERENCES Translation(translationId),
	PRIMARY KEY (indexWordId),
	INDEX (indexWord(64))
);

DROP TABLE IF EXISTS CommunityIndexWord_NO;
CREATE TABLE CommunityIndexWord_NO (
	indexWordId INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
	indexWord TINYBLOB NOT NULL,
	translationId INTEGER UNSIGNED NOT NULL REFERENCES Translation(translationId),
	PRIMARY KEY (indexWordId),
	INDEX (indexWord(64))
);

DROP TABLE IF EXISTS CommunityStemWord_DE;
CREATE TABLE CommunityStemWord_DE (
	stemWordId INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
	stemWord TINYBLOB NOT NULL,
	translationId INTEGER UNSIGNED NOT NULL REFERENCES Translation(translationId),
	PRIMARY KEY (stemWordId),
	INDEX (stemWord(64))
);

DROP TABLE IF EXISTS CommunityStemWord_NO;
CREATE TABLE CommunityStemWord_NO (
	stemWordId INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
	stemWord TINYBLOB NOT NULL,
	translationId INTEGER UNSIGNED NOT NULL REFERENCES Translation(translationId),
	PRIMARY KEY (stemWordId),
	INDEX (stemWord(64))
);

DROP TABLE IF EXISTS CommunityPhoneticWord_DE;
CREATE TABLE CommunityPhoneticWord_DE (
	phoneticWordId INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
	phoneticWord TINYBLOB NOT NULL,
	baseWord TINYBLOB NOT NULL,
	PRIMARY KEY (phoneticWordId),
	INDEX (phoneticWord(64))
);

DROP TABLE IF EXISTS CommunityPhoneticWord_NO;
CREATE TABLE CommunityPhoneticWord_NO (
	phoneticWordId INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
	phoneticWord TINYBLOB NOT NULL,
	baseWord TINYBLOB NOT NULL,
	PRIMARY KEY (phoneticWordId),
	INDEX (phoneticWord(64))
);
