# -- table for storing heinzelliste in original format
DROP TABLE  IF EXISTS HeinzelListStore;
CREATE TABLE HeinzelListStore (
    id INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
    heinzelliste MEDIUMBLOB,
    PRIMARY KEY (id)
);

# -- the german part of the dictionary
DROP TABLE IF EXISTS Translation;
CREATE TABLE Translation (
	translationId INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
	word_DE TINYTEXT NOT NULL,
	article_DE TINYTEXT,
	other_DE TEXT,
	word_NO TINYTEXT NOT NULL,
	article_NO TINYTEXT,
	other_NO TEXT,
	grade INTEGER,
	category TINYTEXT,
	PRIMARY KEY (translationId)
);


DROP TABLE IF EXISTS IndexWord_DE;
CREATE TABLE IndexWord_DE (
	indexWordId INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
	indexWord TINYBLOB NOT NULL,
	translationId INTEGER UNSIGNED NOT NULL REFERENCES Translation(translationId),
	PRIMARY KEY (indexWordId),
	INDEX (indexWord(64))
);

DROP TABLE IF EXISTS IndexWord_NO;
CREATE TABLE IndexWord_NO (
	indexWordId INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
	indexWord TINYBLOB NOT NULL,
	translationId INTEGER UNSIGNED NOT NULL REFERENCES Translation(translationId),
	PRIMARY KEY (indexWordId),
	INDEX (indexWord(64))
);

DROP TABLE IF EXISTS StemWord_DE;
CREATE TABLE StemWord_DE (
	stemWordId INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
	stemWord TINYBLOB NOT NULL,
	translationId INTEGER UNSIGNED NOT NULL REFERENCES Translation(translationId),
	PRIMARY KEY (stemWordId),
	INDEX (stemWord(64))
);

DROP TABLE IF EXISTS StemWord_NO;
CREATE TABLE StemWord_NO (
	stemWordId INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
	stemWord TINYBLOB NOT NULL,
	translationId INTEGER UNSIGNED NOT NULL REFERENCES Translation(translationId),
	PRIMARY KEY (stemWordId),
	INDEX (stemWord(64))
);

DROP TABLE IF EXISTS PhoneticWord_DE;
CREATE TABLE PhoneticWord_DE (
	phoneticWordId INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
	phoneticWord TINYBLOB NOT NULL,
	baseWord TINYBLOB NOT NULL,
	PRIMARY KEY (phoneticWordId),
	INDEX (phoneticWord(64))
);

DROP TABLE IF EXISTS PhoneticWord_NO;
CREATE TABLE PhoneticWord_NO (
	phoneticWordId INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
	phoneticWord TINYBLOB NOT NULL,
	baseWord TINYBLOB NOT NULL,
	PRIMARY KEY (phoneticWordId),
	INDEX (phoneticWord(64))
);


# -- counter / wordquestion
DROP TABLE IF EXISTS WordAccess;
CREATE TABLE WordAccess (
	word TINYTEXT,
	haveAnswer BOOL
);

DROP TABLE IF EXISTS DictionaryCounter;
CREATE TABLE DictionaryCounter (
	date DATE NOT NULL,
	amount INTEGER UNSIGNED NOT NULL,
	type ENUM('dict','trainer','mobile', 'suggest') NOT NULL,
	PRIMARY KEY (date, type)
);

# -- download repository
DROP TABLE IF EXISTS Download;
CREATE TABLE Download (
	download INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
	email TINYTEXT,
	requireUpdate BOOL NOT NULL DEFAULT 0,
	insertDate DATE,
	comment TEXT,
	PRIMARY KEY(download)
);

# -- links to Canoo
# -- article and word are foreign keys of Translation word_DE and article_DE
# -- but this is not enforced
# -- using varchar instead of tinytext because of bug? in mysql and indices
DROP TABLE IF EXISTS CanooLinks;
CREATE TABLE CanooLinks (
	word_DE VARCHAR(64) NOT NULL,
	article_DE VARCHAR(64),
	canooLink VARCHAR(255) NOT NULL,
	UNIQUE INDEX (word_DE, article_DE)
);

# -- bokmaal words from ordbanken
# -- word: as searched in heinzelnisse (ordbanken column 3) (binary since char aa == a in etc in mysql)
# -- article: ordbanken column 4
# -- searchItem: searchItem in ordbanken (ordbanken column 2)
DROP TABLE IF EXISTS BokmaalWord;
CREATE TABLE BokmaalWord (
        word TINYBLOB NOT NULL,
        article CHAR(6),
        searchTerm TINYBLOB NOT NULL,
        INDEX (word(128)),
        INDEX (searchTerm(128)),
        UNIQUE INDEX (word(128), searchTerm(128))
);

# -- nynorsk words from ordbanken
# -- word: as searched in heinzelnisse (ordbanken column 3) (binary since char aa == a in etc in mysql)
# -- article: ordbanken column 4
# -- searchItem: searchItem in ordbanken (ordbanken column 2)
DROP TABLE IF EXISTS NynorskWord;
CREATE TABLE NynorskWord (
        word TINYBLOB NOT NULL,
        article CHAR(6),
        searchTerm TINYBLOB NOT NULL,
        INDEX (word(128)),
        INDEX (searchTerm(128)),
        UNIQUE INDEX (word(128), searchTerm(128))
);