DROP TABLE IF EXISTS ForumQuestion;
CREATE TABLE ForumQuestion (
	forumQuestion INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
        forumUser INTEGER UNSIGNED, # -- no foreign key, user might get deleted
	text TEXT NOT NULL,
        originalText TEXT NOT NULL,
	timestamp TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
	address TINYTEXT,
	PRIMARY KEY (forumQuestion)
);

# -- keywords
DROP TABLE IF EXISTS ForumKeyword;
CREATE TABLE ForumKeyword (
        keyword VARCHAR(128) NOT NULL,
        forumQuestion INTEGER UNSIGNED NOT NULL REFERENCES ForumQuestion(forumQuestion)
);

DROP TABLE IF EXISTS ForumAnswer;
CREATE TABLE ForumAnswer (
	forumAnswer INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
        forumUser INTEGER UNSIGNED, # -- no foreign key, user might get deleted
	text TEXT NOT NULL,
        originalText TEXT NOT NULL,
	timestamp TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
	address TINYTEXT,
	forumQuestion INTEGER UNSIGNED NOT NULL REFERENCES ForumQuestion(forumQuestion),
	PRIMARY KEY (forumAnswer)
);

# -- user of forum
DROP TABLE IF EXISTS ForumUser;
CREATE TABLE ForumUser (
	forumUser INTEGER UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	displayName VARCHAR(32) NOT NULL UNIQUE KEY,
        admin TINYINT UNSIGNED NOT NULL DEFAULT 0, # -- 0 everybody, 1 forumAdmin, 2 superAdmin
	email VARCHAR(128) NOT NULL UNIQUE KEY,
	password CHAR(13) NOT NULL,
	followUp BOOL NOT NULL,
	preferredLanguage ENUM('de','no') NOT NULL,
    motherTongue CHAR(2),
	creationDate TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
	lastLogin DATETIME # -- won't get a timestamp
);

DROP TABLE IF EXISTS BlockedIps;
CREATE TABLE BlockedIps (
        address TINYTEXT NOT NULL,
        forumUser INTEGER UNSIGNED, # -- no foreign key, user might get deleted
	timestamp TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
	PRIMARY KEY (address(196))
);

DROP TABLE IF EXISTS TorBlockedIps;
CREATE TABLE TorBlockedIps (
        address TINYTEXT NOT NULL,
	PRIMARY KEY (address(196))
);


# -- ALTER TABLE ForumUser ADD COLUMN motherTongue CHAR(2);


ALTER TABLE ForumQuestion
ADD INDEX (timestamp);
ALTER TABLE ForumQuestion
ADD FULLTEXT (text);
ALTER TABLE ForumQuestion
ADD FULLTEXT (originalText);
ALTER TABLE ForumKeyword
ADD INDEX (keyword);
ALTER TABLE ForumKeyword
ADD INDEX (forumQuestion);
ALTER TABLE ForumAnswer
ADD INDEX (timestamp);
ALTER TABLE ForumAnswer
ADD INDEX (forumQuestion);
ALTER TABLE ForumAnswer
ADD FULLTEXT (text);
ALTER TABLE ForumAnswer
ADD FULLTEXT (originalText);
