/*
 * CanooBokmaalLinkDictUtils.java
 * 
 * Created on Oct 13, 2007, 7:56:20 PM
 * 
 * 
 * 
 */

package info.heinzelnisse.dictionary;

import info.heinzelnisse.dictionary.dicttables.TranslationDeNo;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Set;

/**
 *
 * @author hklein
 */
public class CanooBokmaalLinkDictUtils extends DictUtils {
    static CanooBokmaalLinkDictUtils instance;
    
    public static synchronized CanooBokmaalLinkDictUtils getInstance() {
        if (instance == null) {
            instance = new CanooBokmaalLinkDictUtils();
        }
        return instance;
    }

    @Override
    public String getTablePrefix() {
        return "";
    }

    /**
     * search database for all items with exactly searchitem
     * use WordHelper
     *
     * @return: sorted List of Translation
     */
    @Override
    public List<CanooBokmaalLinkTranslation> dbSearch(String searchItem, DictLang lang) {
        Set<Integer> uniqueIdSet = dbSearchIdSet(searchItem, lang);
        List<CanooBokmaalLinkTranslation> translations = new ArrayList<CanooBokmaalLinkTranslation>(getTranslations(uniqueIdSet, lang));
        Collections.sort(translations);
        return translations;
    }
    
    /**
     * search database for all items with searchitem approximately
     *
     * @return: sorted List of Translation
     */
    @Override
    public List<CanooBokmaalLinkTranslation> dbSearchStart(String searchItem, DictLang lang) {
        Set<Integer> uniqueIdSet = dbSearchStartIdSet(searchItem, lang);
        List<CanooBokmaalLinkTranslation> translations = new ArrayList<CanooBokmaalLinkTranslation>(getTranslations(uniqueIdSet, lang));
        Collections.sort(translations);
        return translations;
    }
    
            /** create a Collection of Translations from the database */
    @Override
    public Collection<CanooBokmaalLinkTranslation> getTranslations(Collection<Integer> ids, DictUtils.DictLang language) {
        Collection<TranslationDeNo> trans = DictTables.getTranslations(ids);
        Collection<CanooBokmaalLinkTranslation> transOut= new ArrayList<CanooBokmaalLinkTranslation>(trans.size());
        for (TranslationDeNo t : trans) {
            String canooLink = null;
            if (t.has_wiki) canooLink = t.word_DE;
            if (language == DictLang.DE) {
                transOut.add(new CanooBokmaalLinkTranslation(DictLang.DE, 0, t.word_DE, t.article_DE, t.other_DE, t.word_NO, t.article_NO, t.other_NO, "", "", canooLink, t.word_NO));
            } else {
                transOut.add(new CanooBokmaalLinkTranslation(DictLang.NO, 0, t.word_NO, t.article_NO, t.other_NO, t.word_DE, t.article_DE, t.other_DE, "", "", canooLink, t.word_NO));
            }
        }
        return transOut;
    }
    



}
