/*
 * CanooLinkDictUtils.java
 * 
 * Created on Oct 13, 2007, 7:56:20 PM
 * 
 * 
 * 
 */

package info.heinzelnisse.dictionary;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 *
 * @author hklein
 */
public class CanooLinkDictUtils extends DictUtils {
    static CanooLinkDictUtils instance;
    
    public static synchronized CanooLinkDictUtils getInstance() {
        if (instance == null) {
            instance = new CanooLinkDictUtils();
        }
        return instance;
    }

    public String getTablePrefix() {
        return "";
    }

    /**
     * search database for all items with exactly searchitem
     * use WordHelper
     *
     * @return: sorted List of Translation
     */
    public List<CanooLinkTranslation> dbSearch(String searchItem, DictLang lang) {
        Set<Integer> uniqueIdSet = dbSearchIdSet(searchItem, lang);
        List<CanooLinkTranslation> translations = new ArrayList<CanooLinkTranslation>(getTranslations(uniqueIdSet, lang));
        Collections.sort(translations);
        return translations;
    }
    
    /**
     * search database for all items with searchitem approximately
     *
     * @return: sorted List of Translation
     */
    public List<CanooLinkTranslation> dbSearchStart(String searchItem, DictLang lang) {
        Set<Integer> uniqueIdSet = dbSearchStartIdSet(searchItem, lang);
        List<CanooLinkTranslation> translations = new ArrayList<CanooLinkTranslation>(getTranslations(uniqueIdSet, lang));
        Collections.sort(translations);
        return translations;
    }
    
            /** create a Collection of Translations from the database */
    public Collection<CanooLinkTranslation> getTranslations(Collection<Integer> ids, DictUtils.DictLang language) {
        String lang = DictUtils.dbLangFlag(language);
        String tlang = DictUtils.dbTransLangFlag(language);
        
        Collection<CanooLinkTranslation> output = new ArrayList<CanooLinkTranslation>();
        if (ids.size() == 0) {
            return output;
        } else {
            Connection conn = null;
            PreparedStatement statement = null;
            ResultSet rs = null;
            try {
                conn = DictUtils.dbConnection();
                StringBuffer questionMarks = new StringBuffer();
                for (int i = 0; i < ids.size(); i++) {
                    questionMarks.append("?,");
                }
                questionMarks.deleteCharAt(questionMarks.length()-1); // remove last ,
                String query = "SELECT translationId, Translation.word_"+lang+" AS word, Translation.article_"+lang+" AS article, other_"+lang+" AS other, Translation.word_"+tlang+" AS t_word, Translation.article_"+tlang+" AS t_article, other_"+tlang+" AS t_other, category, grade, canooLink "+
                        "FROM Translation LEFT JOIN CanooLinks "+
                        "ON Translation.word_DE = CanooLinks.word_DE "+
                        "AND Translation.article_DE = CanooLinks.article_DE "+
                        "WHERE translationId IN ("+questionMarks.toString()+")";
                statement = conn.prepareStatement(query);
                int i = 1;
                for (int id : ids) {
                    statement.setInt(i, id);
                    i++;
                }
                rs = statement.executeQuery();
                while (rs.next()) {
                    output.add(new CanooLinkTranslation(language, rs.getInt("translationId"), rs.getString("word"), rs.getString("article"), rs.getString("other"), rs.getString("t_word"), rs.getString("t_article"), rs.getString("t_other"), rs.getString("category"), rs.getString("grade"), rs.getString("canooLink")));
                }
            } catch (SQLException se) {
                se.printStackTrace();
            } finally {
                try {rs.close();} catch (Exception e) {}
                try {statement.close();} catch (Exception e) {}
                try {conn.close();} catch (Exception e) {}
            }
            return output;
        }
    }
    



}
