/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package info.heinzelnisse.dictionary;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

/**
 *
 * @author hklein
 */
public class DictStatisticCache {
    private static final DictStatisticCache instance = new DictStatisticCache();
    private Map<DictUtils.DictType, Integer> usageCounter; // type => count
    private List<String> wordAccess[]; // list[0] = found, list[1] = not found
    
    
    private DictStatisticCache() {
        usageCounter = new HashMap<DictUtils.DictType, Integer>(10000); // type => count
        wordAccess = new List[] {new ArrayList<String>(10000), new ArrayList<String>(10000)};
    }
    
    /**
     * get the instance of the statistics singleton
     * @return instance
     */
    public static DictStatisticCache getInstance() {
        return instance;
    }
    
    /**
     * increment the cache: usage counter
     * This is thread-safe.
     * @param type
     */
    public void incrementUsageCounter(DictUtils.DictType type) {
        synchronized (usageCounter) {
            Integer amount = usageCounter.get(type);
            int number;
            if (amount != null) {
                number = amount;
            } else {
                number = 0;
            }
            number++;
            usageCounter.put(type, number);
        }
    }
    
    /**
     * increment the cache: wordAcces
     * This is thread-safe.
     * @param word
     * @param found
     */
    public void addWordAccess(String word, boolean  found) {
        synchronized (wordAccess) {
            if (found) {
                wordAccess[0].add(word);
            } else {
                wordAccess[1].add(word);
            }
            // don't let the wordAccess become to large
            // it will have usually have twice the size on database-writing
            if ((wordAccess[0].size() + wordAccess[1].size()) > 5000) {
                Logger.getLogger(DictStatisticCache.class.getName()).info("writing word-access on more than 5000 elements");
                Connection conn = null;
                try {
                    conn = CanooBokmaalLinkDictUtils.dbConnection();
                    CanooBokmaalLinkDictUtils.getInstance().addWordAccess(conn, wordAccess);
                    wordAccess[0].clear();
                    wordAccess[1].clear();
                } catch (SQLException e) {
                    e.printStackTrace();
                } finally {
                    try {conn.close();} catch (Exception e) {}
                }
                        
            }
        }
    }
    
    /**
     * retrieve a copy and clear the usageCounter cache
     * This is thread-safe
     * @return copy of original usageCounter
     */
    public Map<DictUtils.DictType, Integer> clearUsageCounter() {
        Map<DictUtils.DictType, Integer> retVal = new HashMap<DictUtils.DictType, Integer>();
        synchronized (usageCounter) {
            retVal.putAll(usageCounter);
            usageCounter.clear();
        }
        return retVal;
    }
    
    /**
     * retrieve a copy of wordAccess and clear the wordAccess cache
     * This is thread-safe
     * 
     * @return Array of lists, with list[0] = found elements, list[1] = not found elements
     */
    public List<String>[] clearWordAccess() {
        List<String> retVal[] = new List[]{new ArrayList<String>(), new ArrayList<String>()};
        synchronized (wordAccess) {
            retVal[0].addAll(wordAccess[0]);
            retVal[1].addAll(wordAccess[1]);
            wordAccess[0].clear();
            wordAccess[1].clear();
        }
        return retVal;                
    }
    
}
