/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package info.heinzelnisse.dictionary;

import info.heinzelnisse.dictionary.dicttables.WordArticle;
import info.heinzelnisse.dictionary.dicttables.TranslationDeNo;
import info.heinzelnisse.dictionary.dicttables.DictTablesStorer;
import info.heinzelnisse.dictionary.DictUtils.DictLang;
import info.heinzelnisse.multiphone.AbstractMultiphone;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.text.Collator;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author hklein
 */
public class DictTables {

    static private void load() {
        synchronized (DictTables.class) {
            if (!loaded) {
                translations.clear();
                indexWordDE.clear();
                indexWordNO.clear();
                stemWordDE.clear();
                stemWordNO.clear();
                phoneticWordDE.clear();
                phoneticWordNO.clear();

                OpenOfficeTableLoader ootl = new OpenOfficeTableLoader();
                DictTablesStorer dts = new DictTablesStorer(translations, indexWordDE, indexWordNO, stemWordDE, stemWordNO, phoneticWordDE, phoneticWordNO);
                ootl.addTranslationStore(dts);
                DictUtils.dbReadOOTable(ootl);
                Logger.getLogger(DictTables.class.getName()).info("heinzelliste.ods read into memory");
                //loadCannoLinks();
                loadDeWikiLinks();
                loaded = true;
            }
        }
    }

    public static void reset() {
        synchronized (DictTables.class) {
            loaded = false;
            load();
        }        
    }
    
    private static void loadDeWikiLinks() {
        try {
            Set<String> wikiList = new HashSet<>();
            BufferedReader wikiWords = new BufferedReader(new InputStreamReader(DictTables.class.getResourceAsStream("/deWikiArticles.txt"), "UTF-8"));
            String line;
            while ((line = wikiWords.readLine()) != null) {
                wikiList.add(line);
            }
            wikiWords.close();
            
            for (TranslationDeNo t : translations) {
                if (wikiList.contains(t.word_DE)) {
                    t.has_wiki = true;
                }
            }
            Logger.getLogger(DictTables.class.getName()).info("deWikiArticles read into memory");            
        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(DictTables.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(DictTables.class.getName()).log(Level.SEVERE, null, ex);
        } 
        
    }

    // no longer used
    private static void loadCannoLinks() {
        TreeMap<WordArticle, String> canooMap = new TreeMap<WordArticle, String>();
        
        try {
        
            BufferedReader canooLinks = new BufferedReader(new InputStreamReader(DictTables.class.getResourceAsStream("/canooLink.csv"), "UTF-8"));
            String line;
            while ((line = canooLinks.readLine()) != null) {
                String[] fields = line.split("\t");
                if (fields.length == 3) {
                    canooMap.put(new WordArticle(fields[0], fields[1]), fields[2]);
                }
            }
            canooLinks.close();
            
            for (TranslationDeNo t : translations) {
                WordArticle wa = new WordArticle(t.word_DE, t.article_DE);
                if (canooMap.containsKey(wa)) {
                    // disabled canooLink, use t.deWikiExists = true
                    //t.canooLink = canooMap.get(wa);
                }
            }
            Logger.getLogger(DictTables.class.getName()).info("canooLink.csv read into memory");            
        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(DictTables.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(DictTables.class.getName()).log(Level.SEVERE, null, ex);
        } 
        
    }
        
    static public Collection<TranslationDeNo> getTranslations(Collection<Integer> ids) {
        if (ids == null) return new ArrayList<TranslationDeNo>();
        ArrayList<TranslationDeNo> trans = new ArrayList<TranslationDeNo>(ids.size());
        synchronized (DictTables.class) {
            load();
            for (int id : ids) {
                trans.add(translations.get(id));
            }
        }
        return trans;
    }
    static private SortedMap<String, Set<Integer>> getIndexMap(DictLang lang) {
        return (lang == DictLang.DE) ? indexWordDE : indexWordNO;
    }
    static private Map<String, Set<Integer>> getStemMap(DictLang lang) {
        return (lang == DictLang.DE) ? stemWordDE : stemWordNO;
    }
    static private Map<String, Set<String>> getPhoneticMap(DictLang lang) {
        return (lang == DictLang.DE) ? phoneticWordDE : phoneticWordNO;
    }

    
    /**
     * 
     * @param searchTerm clean and lower-case
     * @param lang
     * @return 
     */
    static public Set<Integer> searchExactIds(String searchTerm, DictLang lang) {
        searchTerm = searchTerm.toLowerCase(Utils.langLocale(lang));
        
        synchronized (DictTables.class) {
            load();
            SortedMap<String, Set<Integer>> indexWord = getIndexMap(lang);
            return indexWord.get(searchTerm);
        }
    }
    /**
     * searches translation-ids starting with the searchTerm
     * or being stemmed with searchTerm
     * 
     * 
     * @param searchTerm clean word
     * @param lang
     * @return 
     */
    static public Set<Integer> searchStartIds(String searchTerm, DictLang lang) {
        searchTerm = searchTerm.toLowerCase(Utils.langLocale(lang));
        // find the first word to be excluded
        String nextTerm = searchTerm;
        char lastChar = (char) (1 + nextTerm.charAt(nextTerm.length()-1));
        nextTerm = nextTerm.substring(0, nextTerm.length()-1) + lastChar;
        Logger.getLogger(DictTables.class.getName()).log(Level.FINE, "search between:{0} - {1}", new String[]{searchTerm, nextTerm});
        // stem the lower-case searchTerm
        String stemmed = Utils.stem(searchTerm, lang);

        Set<Integer> retSet = new TreeSet<Integer>();
        synchronized (DictTables.class) {
            load();
            SortedMap<String, Set<Integer>> indexWord = getIndexMap(lang);
            SortedMap<String, Set<Integer>> subMap = indexWord.subMap(searchTerm, nextTerm);
            for (Set<Integer> ids : subMap.values()) {
                if (ids != null) retSet.addAll(ids);
            }
            if ((stemmed.length() != 0) && (!stemmed.equals(searchTerm))) {
                final Map<String, Set<Integer>> stemWord = getStemMap(lang);
                Set<Integer> stemSet = stemWord.get(stemmed);
                if (stemSet != null ) retSet.addAll(stemSet);
            }
        }
        
        return retSet;
    }
    
    /**
     * retrieve a list of words similar (typos, phonetics) to the searchItem
     * 
     * @param searchItem the word inserted
     * @param lang language of the searchItem
     * @return list of similar words, never null
     */
    static public List<String> searchSimilarWords(String searchItem, DictLang lang) {
        if (searchItem.length() == 0) {
            return new ArrayList<String>(0);
        }
        String lcSearchItem = searchItem.toLowerCase(Utils.langLocale(lang));
        String stemmed = Utils.stem(lcSearchItem, lang);
        AbstractMultiphone phoner = null;
        switch (lang) {
            case DE: phoner = new info.heinzelnisse.multiphone.De(); break;
            case NO: phoner = new info.heinzelnisse.multiphone.No(); break;
        }
        Collection<String> phoned = phoner.multiphone(stemmed);
        Set<String> uniqueStringSet = new HashSet<String>();
        synchronized (DictTables.class) {
            load();
            Map<String, Set<String>> phoneticMap = getPhoneticMap(lang);
            for (String item : phoned) {
                Set<String> phonSet = phoneticMap.get(item);
                if (phonSet != null) {
                    uniqueStringSet.addAll(phonSet);
                }
            }
        }
        
        List<String> similars = new ArrayList<String>(uniqueStringSet);
        Collections.sort(similars, Collator.getInstance(Utils.langLocale(lang)));
        return similars;
    }    

    
    
    //GuardedBy(class)
    static private boolean loaded = false;
    //GuardedBy(class)
    static private final List<TranslationDeNo> translations = new ArrayList<TranslationDeNo>();
    //GuardedBy(class)
    static private final SortedMap<String, Set<Integer>> indexWordDE = new TreeMap<String, Set<Integer>>();
    //GuardedBy(class)
    static private final SortedMap<String, Set<Integer>> indexWordNO = new TreeMap<String, Set<Integer>>();
    //GuardedBy(class)
    static private final Map<String, Set<Integer>> stemWordDE = new TreeMap<String, Set<Integer>>();
    //GuardedBy(class)
    static private final Map<String, Set<Integer>> stemWordNO = new TreeMap<String, Set<Integer>>();
    // phonetic maps to a baseword
    //GuardedBy(class)
    static private final Map<String, Set<String>> phoneticWordDE = new TreeMap<String, Set<String>>();
    //GuardedBy(class)
    static private final Map<String, Set<String>> phoneticWordNO = new TreeMap<String, Set<String>>();     
    
    
}
