/*
 * HeinzellisteTranslationStore.java
 *
 * Created on Sep 21, 2007, 9:30:52 AM
 *
 */

package info.heinzelnisse.dictionary;

import info.heinzelnisse.he.HeinzelTable;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.json.JSONWriter;

/**
 * write a new heinzelliste to the filename given in properties
 * @warning the filenames are noct locked, the users need to make sure that not
 *          two people upload at the same time!
 *
 * @author hklein
 */
public class HeinzellisteJSONTranslationStore implements TranslationStore {

    private StringBuffer heinzelListe;

    public HeinzellisteJSONTranslationStore() {
        heinzelListe = new StringBuffer();
    }
    
    @Override
    public void addTranslation(String word_DE, String article_DE, String other_DE, Collection<String> optional_DE, String word_NO, String article_NO, String other_NO, Collection<String> optional_NO, String category, String grade) {
        String optionals_DE = join(HeinzelTable.OPTIONAL_SEPARATOR, optional_DE.toArray(new String[0]));
        String optionals_NO = join(HeinzelTable.OPTIONAL_SEPARATOR, optional_NO.toArray(new String[0]));
        joinTo(heinzelListe, HeinzelTable.SEPARATOR, clean(word_NO), clean(article_NO), optionals_NO, clean(other_NO), clean(word_DE), clean(article_DE), optionals_DE, clean(other_DE), category, grade);
        heinzelListe.append('\n');
        return;
    }

    private String clean(String str) {
        return str.replaceAll("[\\[\\]]", "");
    }

    private String join(String separator, String... strs) {
        StringBuffer sb = new StringBuffer();
        joinTo(sb, separator, strs);
        return sb.toString();
    }


    private void joinTo(StringBuffer sb, String separator, String... strs) {
        for (String s : strs) {
            sb.append(s);
            sb.append(separator);
        }
        // remove the last separator
        sb.deleteCharAt(sb.length() - 1);
        return;
    }

    private StringBuffer transliterate(CharSequence in, String from, String to) {
        String fromChars[] = from.split("");
        String toChars[] = to.split("");

        Map<String, String> mapTable = new HashMap<String, String>();

        for(int i = 0; i < fromChars.length; i++) {
            String c = toChars[i];
            mapTable.put(fromChars[i], c);
        }

        Pattern p = Pattern.compile("([" + from + "])");
        Matcher m = p.matcher(in);
        StringBuffer sb = new StringBuffer();
        while (m.find()) {
            m.appendReplacement(sb, mapTable.get(m.group(1)));
        }
        return sb;
    }

    @Override
    public boolean finish() {
        try {
            String jsonFileName = java.util.ResourceBundle.getBundle("info/heinzelnisse/dictionary/properties").getString("JSONHeinzellisteLocation");
            File jsonFile = new File(jsonFileName + ".new");
            OutputStream json = new FileOutputStream(jsonFile);
//        BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new BitObfuscatorOutputStream(json),HeinzelTable.CHARSET));
            BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(json,HeinzelTable.CHARSET));
            if (writer != null) {
                writer.append("/*\n"
                        + " * (C) Copyright 2001-2012, Heiko Klein/Julia Emmerich\n"
                        + " * License: This work is licensed under the Creative Commons\n"
                        + " *   Attribution-NonCommercial-ShareAlike 3.0 Unported License.\n"
                        + " *   To view a copy of this license,\n"
                        + " *   visit http://creativecommons.org/licenses/by-nc-sa/3.0/.\n"
                        + " */\n");
                writer.append("HeinzelSearch.Config.isNice = true;\n");
                writer.append("HeinzelSearch.Config.hndb = ");
                //heinzelListe = transliterate(heinzelListe,"ADEIMNOUXYZadeimnouxyz", "nMiEdAzYxUoNmIeDaZyXuO");
                new JSONWriter(writer).array().value(heinzelListe).endArray();
                writer.append("[0];\n");
                writer.close();
                File finalFile = new File(jsonFileName);
                if (finalFile.exists()) {
                    if (!finalFile.delete()) {
                        // don't care on errors
                    }
                }
                if (!jsonFile.renameTo(finalFile)) {
                    Logger.getLogger(HeinzellisteJSONTranslationStore.class.getName()).log(Level.SEVERE, "cannot rename filename from: {0} to {1}", new Object[]{jsonFile.getAbsolutePath(), finalFile.getAbsolutePath()});
                }
            }
        } catch (Exception ex) {
            Logger.getLogger(HeinzellisteJSONTranslationStore.class.getName()).log(Level.SEVERE, null, ex);
            return false;
        }
        return true;
    }
}