/*
 * HeinzellisteTranslationStore.java
 *
 * Created on Sep 21, 2007, 9:30:52 AM
 *
 */

package info.heinzelnisse.dictionary;

import info.heinzelnisse.he.BitObfuscatorOutputStream;
import info.heinzelnisse.he.HeinzelTable;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Collection;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * write a new heinzelliste to the filename given in properties
 * @warning the filenames are noct locked, the users need to make sure that not
 *          two people upload at the same time!
 *
 * @author hklein
 */
public class HeinzellisteTranslationStore implements TranslationStore {

    private BufferedWriter writer = null;
    private File zipFile = null;
    private String zipFileName = "";

    public HeinzellisteTranslationStore() {
        zipFileName = java.util.ResourceBundle.getBundle("info/heinzelnisse/dictionary/properties").getString("ZipFileLocation");
        String txtFileName = java.util.ResourceBundle.getBundle("info/heinzelnisse/dictionary/properties").getString("TextFile");
        try {
            zipFile = new File(zipFileName + ".new");
            ZipOutputStream zip = new ZipOutputStream(new FileOutputStream(zipFile));
            zip.putNextEntry(new ZipEntry(txtFileName));
            writer = new BufferedWriter(new OutputStreamWriter(new BitObfuscatorOutputStream(zip),HeinzelTable.CHARSET));
            writer.write(join(HeinzelTable.SEPARATOR, "# ord","kjønn","opsjonal","annet","Wort","Geschlecht","Optional","Sonstiges","Category","Grade"));
            writer.newLine();
        } catch (IOException ex) {
            Logger.getLogger(HeinzellisteTranslationStore.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    
    @Override
    public void addTranslation(String word_DE, String article_DE, String other_DE, Collection<String> optional_DE, String word_NO, String article_NO, String other_NO, Collection<String> optional_NO, String category, String grade) {
        String optionals_DE = join(HeinzelTable.OPTIONAL_SEPARATOR, optional_DE.toArray(new String[0]));
        String optionals_NO = join(HeinzelTable.OPTIONAL_SEPARATOR, optional_NO.toArray(new String[0]));
        String trans = join(HeinzelTable.SEPARATOR, clean(word_NO), clean(article_NO), optionals_NO, clean(other_NO), clean(word_DE), clean(article_DE), optionals_DE, clean(other_DE), category, grade);
        try {
            writer.write(trans);
            writer.newLine();
        } catch (IOException ex) {
            Logger.getLogger(HeinzellisteTranslationStore.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private String clean(String str) {
        return str.replaceAll("[\\[\\]]", "");
    }

    private String join(String separator, String... strs) {
        StringBuffer sb = new StringBuffer("");
        for (String s : strs) {
            sb.append(s);
            sb.append(separator);
        }
        // remove the last separator
        sb.deleteCharAt(sb.length() - 1);
        return sb.toString();
    }

    @Override
    public boolean finish() {
        if (writer != null) {
            try {
                writer.close();
                File finalFile = new File(zipFileName);
                if (finalFile.exists()) {
                    if (!finalFile.delete()) {
                        // don't care on errors
                    }
                }
                if (!zipFile.renameTo(finalFile)) {
                    Logger.getLogger(HeinzellisteTranslationStore.class.getName()).log(Level.SEVERE, "cannot rename filename from: "+zipFile.getAbsolutePath()+" to "+finalFile.getAbsolutePath());
                }
            } catch (IOException ex) {
                Logger.getLogger(HeinzellisteTranslationStore.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        return true;
    }
}