/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package info.heinzelnisse.dictionary.dicttables;

import info.heinzelnisse.dictionary.TranslationStore;
import info.heinzelnisse.dictionary.Utils;
import info.heinzelnisse.multiphone.AbstractMultiphone;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;

/**
 *
 * @author hklein
 */
public class DictTablesStorer implements TranslationStore {

    private List<TranslationDeNo> translations;
    private SortedMap<String, Set<Integer>> indexWordDE;
    private SortedMap<String, Set<Integer>> indexWordNO;
    private Map<String, Set<Integer>> stemWordDE;
    private Map<String, Set<Integer>> stemWordNO;
    private Map<String, Set<String>> phoneticWordDE;
    private Map<String, Set<String>> phoneticWordNO;

    
    public DictTablesStorer(List<TranslationDeNo> translations, SortedMap<String, Set<Integer>> indexWordDE, SortedMap<String, Set<Integer>> indexWordNO, Map<String, Set<Integer>> stemWordDE, Map<String, Set<Integer>> stemWordNO, Map<String, Set<String>> phoneticWordDE, Map<String, Set<String>> phoneticWordNO) {
        this.translations = translations;
        this.indexWordDE = indexWordDE;
        this.indexWordNO = indexWordNO;
        this.stemWordDE = stemWordDE;
        this.stemWordNO = stemWordNO;
        this.phoneticWordDE = phoneticWordDE;
        this.phoneticWordNO = phoneticWordNO;
    }

    @Override
    public void addTranslation(String word_DE, String article_DE, String other_DE, Collection<String> optional_DE, String word_NO, String article_NO, String other_NO, Collection<String> optional_NO, String category, String grade) {
        // make sure to run within a write-lock
        word_DE = word_DE.trim().intern();
        article_DE = article_DE.trim().intern();
        other_DE = other_DE.trim().intern();
        word_NO = word_NO.trim().intern();
        article_NO = article_NO.trim().intern();
        other_NO = other_NO.trim().intern();
        category = category.trim().intern();
        grade = grade.trim().intern();

        int translationId = translations.size();
        TranslationDeNo trans = new TranslationDeNo(word_DE, article_DE, other_DE, word_NO, article_NO, other_NO);
        translations.add(trans);
        // add to indexWords DE
        Set<String> optionalSet_DE = new HashSet<String>();
        for (String str : optional_DE) {
            optionalSet_DE.add(str.toLowerCase().trim().intern());
        }
        optionalSet_DE.add(word_DE.toLowerCase());
        for (String word : optionalSet_DE) {
            for (String str : Utils.multiplyUmlautWords(word)) {
                str = str.intern();
                if (!str.matches("\\s*")) {
                    addToMap(indexWordDE, str, translationId);
                    addStemPhonetics_DE(translationId, word_DE, str);
                }
            }
        }
        // add to indexWords NO
        Set<String> optionalSet_NO = new HashSet<String>();
        for (String str : optional_NO) {
            optionalSet_NO.add(str.toLowerCase().trim().intern());
        }
        optionalSet_NO.add(word_NO.toLowerCase());
        for (String word : optionalSet_NO) {
            for (String str : Utils.multiplyUmlautWords(word)) {
                if (!str.matches("\\s*")) {
                    addToMap(indexWordNO, str, translationId);
                    addStemPhonetics_NO(translationId, word_NO, str);
                }
            }
        }
    }

    @Override
    public boolean finish() {
        return true;
    }

    private void addStemPhonetics_DE(int id, String word, String str) {
        org.tartarus.snowball.SnowballProgram stemmer = new org.tartarus.snowball.ext.germanStemmer();
        stemmer.setCurrent(str);
        stemmer.stem();
        String stemmed = stemmer.getCurrent();
        addToMap(stemWordDE, word, id);
        AbstractMultiphone multiphone = new info.heinzelnisse.multiphone.De();
        for (String mps : multiphone.multiphone(stemmed)) {
            addToMap(phoneticWordDE, mps, word);
        }
    }

    private void addStemPhonetics_NO(int id, String word, String str) {
        org.tartarus.snowball.SnowballProgram stemmer = new org.tartarus.snowball.ext.norwegianStemmer();
        stemmer.setCurrent(str);
        stemmer.stem();
        String stemmed = stemmer.getCurrent();
        addToMap(stemWordDE, word, id);
        AbstractMultiphone multiphone = new info.heinzelnisse.multiphone.No();
        for (String mps : multiphone.multiphone(stemmed)) {
            addToMap(phoneticWordNO, mps, word);
        }
    }

    private void addToMap(Map<String, Set<Integer>> map, String word, int translationId) {
        Set<Integer> idSet = map.get(word);
        if (idSet == null) {
            idSet = new HashSet<Integer>();
            map.put(word, idSet);
        }
        idSet.add(translationId);
    }

    private void addToMap(Map<String, Set<String>> map, String mps, String word) {
        Set<String> wordSet = map.get(mps);
        if (wordSet == null) {
            wordSet = new HashSet<String>();
            map.put(mps, wordSet);
        }
        wordSet.add(word);
    }
    
}
