/*
 * Answer.java
 *
 * Created on November 9, 2005, 10:22 AM
 *
 */

package info.heinzelnisse.forum;

import info.heinzelnisse.dictionary.DictUtils;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 *
 * @author Heiko Klein
 */
public class Answer {
    private int id = 0;
    private String text;
    private String originalText;
    private Date timestamp;
    private String address;
    private int questionId = 0;
    private int userId = 0;
    private User user;
    
    /** Creates a new instance of Answer */
    public Answer(int id, String text, String originalText, Date timestamp, String address, int questionId, int userId) {
        this.id = id;
        this.text = text;
        this.originalText = originalText;
        this.timestamp = timestamp;
        this.address = address;
        this.questionId = questionId;
        this.userId = userId;
    }
    
    /**
     * create a persistent answer
     *
     * @param userId use 0 for no user
     */
    static public Answer create(String text, String originalText, String keywords, Date timestamp, String address, int questionId, int userId) {
        Connection conn = null;
        PreparedStatement statement = null;
        ResultSet rs = null;
        
        Answer answer = null;
        try {
            conn = DictUtils.dbConnection();
            String query =
                    "INSERT INTO ForumAnswer (text, originalText, timestamp, address, forumQuestion, forumUser) "+
                    "VALUES (?,?,?,?,?,?) ";
            statement = conn.prepareStatement(query, PreparedStatement.RETURN_GENERATED_KEYS);
            statement.setString(1, text);
            statement.setString(2, originalText);
            statement.setTimestamp(3, new Timestamp(timestamp.getTime()));
            statement.setString(4, address);
            statement.setInt(5, questionId);
            if (userId != 0) {
                statement.setInt(6, userId);
            } else {
                //statement.setNull(6, java.sql.Types.INTEGER);
                throw new SQLException("user required for new answer");
            }
            statement.execute();
            rs = statement.getGeneratedKeys();
            if (rs.next()) {
                answer = new Answer(rs.getInt(1), text, originalText, timestamp, address, questionId, userId);
            } else {
                throw new SQLException("couldn't create answer");
            }
        } catch (SQLException se) {
            se.printStackTrace();
        } finally {
            try {rs.close();} catch (Exception e) {e.printStackTrace();}
            try {statement.close();} catch (Exception e) {e.printStackTrace();}
            try {conn.close();} catch (Exception e) {e.printStackTrace();}
        }
        if (answer != null) {
            if (keywords != null && (!keywords.trim().equals(""))) {
                Question.addKeywords(questionId, keywords);
            }
        }
        return answer;
    }
    
    public static List<Answer> getAnswers(Question q) {
        if (q == null) {
            return null;
        }
        List<Answer> answers = new ArrayList<Answer>();
        Connection conn = null;
        PreparedStatement statement = null;
        ResultSet rs = null;
        try {
            conn = DictUtils.dbConnection();
            String query =
                    "SELECT forumAnswer, text, originalText, timestamp, address, forumQuestion, forumUser "+
                    "FROM ForumAnswer WHERE forumQuestion = ? "+
                    "ORDER BY timestamp ASC ";
            statement = conn.prepareStatement(query);
            statement.setInt(1,q.getId());
            rs = statement.executeQuery();
            while (rs.next()) {
                answers.add(new Answer(rs.getInt("forumAnswer"), rs.getString("text"), rs.getString("originalText"), rs.getTimestamp("timestamp"), rs.getString("address"), q.getId(), rs.getInt("forumUser")));
            }
        } catch (SQLException se) {
            se.printStackTrace();
        } finally {
            try {rs.close();} catch (Exception e) {e.printStackTrace();}
            try {statement.close();} catch (Exception e) {e.printStackTrace();}
            try {conn.close();} catch (Exception e) {e.printStackTrace();}
        }
        return answers;
    }
    
    public static void removeFromDb(int id) {
        Connection conn = null;
        PreparedStatement statement = null;
        try {
            conn = DictUtils.dbConnection();
            String query =
                    "DELETE FROM ForumAnswer "+
                    "WHERE forumAnswer = ? ";
            statement = conn.prepareStatement(query);
            statement.setInt(1,id);
            statement.execute();
        } catch (SQLException se) {
            se.printStackTrace();
        } finally {
            try {statement.close();} catch (Exception e) {e.printStackTrace();}
            try {conn.close();} catch (Exception e) {e.printStackTrace();}
        }
    }
    
    public void removeFromDb() {
        removeFromDb(getId());
    }
    
    public int getId() {
        return id;
    }
    
    
    /**
     * temporary set text (no write to db!)
     */
    public void setText(String text) {
        this.text = text;
    }
    
    public String getText() {
        return text;
    }
    
    /**
     * temporary set text (no write to db!)
     */
    public void setOriginalText(String originalText) {
        this.text = originalText;
    }
    
    public String getOriginalText() {
        return originalText;
    }
    
    public Date getTimestamp() {
        return timestamp;
    }
    
    public String getAddress() {
        return address;
    }
    
    public int getQuestionId() {
        return questionId;
    }
    
    public int getUserId() {
        return userId;
    }
    
    public User getUser() {
        if (user == null) {
            user = new User(userId);
        }
        return user;
    }
}
