/*
 * User.java
 *
 * Created on November 9, 2005, 10:22 AM
 *
 */

package info.heinzelnisse.forum;

import info.heinzelnisse.dictionary.DictUtils;
import java.io.Serializable;
import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.logging.Logger;
import org.vps.crypt.Crypt;

/**
 * Object-Relation mapping of the user class
 * <em>don't forget to call update on the user</em>
 *
 * @author Heiko Klein
 */
// TODO: NotThreadSafe, but maybe it should?
public class User implements Serializable {
    private static final long serialVersionUID = -4202812824783928945L;
    public enum Lang {de, no};
    static final public int INVALID = 0;
    private int id = INVALID;
    private String displayName;
    private String email;
    private String password;
    private boolean followUp;
    private Lang preferredLanguage = Lang.de;
    private String motherTongue;
    private Date creationDate;
    private Date lastLogin;
    private byte admin = 0; // cannot be changed from java, only via sql!
    
    private User originalUser; // for changes
    
    static Logger logger = Logger.getLogger(User.class.getName());
    
    /** Creates a new instance of User */
    public User() {
    }
    
    public User(int id) {
        if (id != 0) {
            Connection conn = null;
            try {
                conn = DictUtils.dbConnection();
                sqlSetUserById(conn, id);
            } catch (SQLException se) {
                se.printStackTrace();
            } finally {
                try {if (conn!=null) conn.close();} catch (SQLException ex) {ex.printStackTrace();}
            }
        }
    }
    
    /**
     *
     */
    public User(String displayName) {
        Connection conn = null;
        PreparedStatement statement = null;
        ResultSet rs = null;
        
        try {
            conn = DictUtils.dbConnection();
            String query =
                    "SELECT forumUser, displayName, password, email, followUp, preferredLanguage, motherTongue, creationDate, lastLogin, admin "+
                    "FROM ForumUser "+
                    "WHERE displayName = ?";
            statement = conn.prepareStatement(query);
            statement.setString(1,displayName);
            rs = statement.executeQuery();
            if (rs.next()) {
                this.id = rs.getInt("forumUser");
                this.displayName = rs.getString("displayName");
                this.password = rs.getString("password");
                this.email = rs.getString("email");
                this.followUp = rs.getBoolean("followUp");
                this.preferredLanguage = Lang.valueOf(rs.getString("preferredLanguage"));
                this.motherTongue = rs.getString("motherTongue");
                this.creationDate = rs.getDate("creationDate");
                this.lastLogin = rs.getDate("lastLogin");
                this.admin = rs.getByte("admin");
                // we're now up to date
                this.originalUser = (User) this.clone();
            } else {
                // empty user, might be okay
            }
        } catch (SQLException se) {
            se.printStackTrace();
        } finally {
            try {rs.close();} catch (Exception e) {e.printStackTrace();}
            try {statement.close();} catch (Exception e) {e.printStackTrace();}
            try {conn.close();} catch (Exception e) {e.printStackTrace();}
        }
    }
    
    /**
     * The real login with password. The user.lastLogin will be updated automatically.
     */
    public User(String displayNameOrEmail, String password) {
        User nameUser = new User(displayNameOrEmail);
        String crypt = Crypt.crypt("he", password);
        if (crypt.equals(nameUser.getPassword())) {
            setByUser(nameUser);
        } else {
            nameUser = User.findByEmail(displayNameOrEmail);
            if (crypt.equals(nameUser.getPassword())) {
                setByUser(nameUser);
            }
        }
        lastLogin = new Date((new java.util.Date()).getTime());
        update();
    }
    
    protected User(User user) {
        setByUser(user);
    }
    
    public boolean isValid() {
        if (id > 0) {
            return true;
        } else {
            return false;
        }
    }
    
    public boolean isAdmin() {
        if (admin > 0) {
            return true;
        } else {
            return false;
        }
    }
    
    public boolean isSuperAdmin() {
        if (admin > 1) {
            return true;
        } else {
            return false;
        }
    }
    
    
    private void setByUser(User user) {
        this.id = user.getId();
        this.displayName = user.getDisplayName();
        this.email = user.getEmail();
        this.password = user.getPassword();
        this.followUp = user.isFollowUp();
        this.preferredLanguage = user.getPreferredLanguage();
        this.motherTongue = user.getMotherTongue();
        this.creationDate = (Date) user.getCreationDate().clone();
        this.admin = user.admin;
        if (user.getLastLogin() == null) {
            this.lastLogin = null;
        } else {
            this.lastLogin = (Date) user.getLastLogin().clone();
        }
    }
    
    @Override
    protected Object clone() {
        return new User(this);
    }
    
    private static boolean objectEqual(Object o1, Object o2) {
        return o1 == o2 || (o1 != null && o1.equals(o2));
    }
    
    private static int objectHashCode(Object o) {
        if (o == null) return 0;
        return o.hashCode();
    }
    
    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        User user = (User) obj;
        if (    (id == user.getId()) &&
                (objectEqual(displayName,user.getDisplayName())) &&
                (objectEqual(email,user.getEmail())) &&
                (objectEqual(password,user.getPassword())) &&
                (followUp == user.isFollowUp()) &&
                (objectEqual(preferredLanguage,user.getPreferredLanguage())) &&
                (objectEqual(motherTongue, user.getMotherTongue())) &&
                (objectEqual(creationDate,user.getCreationDate())) &&
                (objectEqual(lastLogin,user.getLastLogin())) &&
                (objectEqual(admin, user.admin))
                ) {
            return true;
        }
        return false;
    }
    
    @Override
    public int hashCode() {
        return objectHashCode(id) +
                objectHashCode(displayName) +
                objectHashCode(email) +
                objectHashCode(password) +
                objectHashCode(followUp) +
                objectHashCode(preferredLanguage) +
                objectHashCode(motherTongue) +
                objectHashCode(creationDate) +
                objectHashCode(lastLogin) +
                objectHashCode(admin);
    }

    
    static public User findByEmail(String email) {
        Connection conn = null;
        PreparedStatement statement = null;
        ResultSet rs = null;
        User user = new User();
        try {
            conn = DictUtils.dbConnection();
            String query =
                    "SELECT forumUser "+
                    "FROM ForumUser "+
                    "WHERE email = ? ";
            statement = conn.prepareStatement(query);
            statement.setString(1,email.toLowerCase());
            rs = statement.executeQuery();
            if (rs.next()) {
                user = new User(rs.getInt("forumUser"));
            } else {
                // empty user, might be okay
            }
        } catch (SQLException se) {
            se.printStackTrace();
        } finally {
            try {rs.close();} catch (Exception e) {e.printStackTrace();}
            try {statement.close();} catch (Exception e) {e.printStackTrace();}
            try {conn.close();} catch (Exception e) {e.printStackTrace();}
        }
        return user;
    }
    
    public int getId() {
        return id;
    }
    
    public String getDisplayName() {
        return displayName;
    }
    
    public void setDisplayName(String displayName) {
        this.displayName = displayName;
    }
    
    public String getEmail() {
        return email;
    }
    
    public void setEmail(String email) {
        this.email = email;
    }
    
    public String getPassword() {
        return password;
    }
    
    /**
     * @param the clear text password
     */
    public void setPassword(String password) {
        this.password = Crypt.crypt("he",password);
    }
    
    public boolean isFollowUp() {
        return followUp;
    }
    
    public void setFollowUp(boolean followUp) {
        this.followUp = followUp;
    }
    
    public Lang getPreferredLanguage() {
        return preferredLanguage;
    }
    
    public void setPreferredLanguage(Lang preferredLanguage) {
        this.preferredLanguage = preferredLanguage;
    }

    public void setMotherTongue(String motherTongue) {
        this.motherTongue = motherTongue;
    }

    public String getMotherTongue() {
        return motherTongue;
    }

    public Date getCreationDate() {
        return creationDate;
    }
    
    public Date getLastLogin() {
        return lastLogin;
    }
    
    public void setLastLogin(Date lastLogin) {
        this.lastLogin = lastLogin;
    }
    
    
    public void update() {
        Connection conn = null;
        try {
            if (id == 0) {
                if (displayName != null) { // only update existing users
                    conn = DictUtils.dbConnection();
                    int newId = sqlCreateUser(conn);
                    sqlSetUserById(conn, newId);
                }
            } else if (!this.equals(originalUser)) {
                conn = DictUtils.dbConnection();
                sqlUpdateUser(conn);
            }
        } catch (SQLException se) {
            se.printStackTrace();
        } finally {
            try {if (conn!=null) conn.close();} catch (SQLException ex) {ex.printStackTrace();}
        }
    }
    
    /**
     * Create the user in the db
     * @return the auto-generated primary key
     */
    private int sqlCreateUser(Connection conn) throws SQLException {
        PreparedStatement statement = null;
        ResultSet rs = null;
        int key = 0;
        try {
            String query = "INSERT INTO ForumUser (displayName, email, password, followUp, preferredLanguage, motherTongue) VALUES(?,?,?,?,?,?)";
            statement = conn.prepareStatement(query, PreparedStatement.RETURN_GENERATED_KEYS);
            statement.setString(1, displayName);
            statement.setString(2, email);
            statement.setString(3, password);
            statement.setBoolean(4, followUp);
            statement.setString(5, preferredLanguage.toString());
            statement.setString(6, motherTongue);
            statement.execute();
            rs = statement.getGeneratedKeys();
            if (rs.next()) {
                key = rs.getInt(1);
            } else {
                throw new SQLException("Couldn't fetch auto generated key");
            }
        } catch (SQLException sex) {
            throw sex;
        } finally {
            try {rs.close();} catch (Exception e) {e.printStackTrace();}
            try {statement.close();} catch (Exception e) {e.printStackTrace();}
        }
        return key;
    }
    
    private void sqlUpdateUser(Connection conn) throws SQLException {
        PreparedStatement statement = null;
        try {
            String query =
                    "UPDATE ForumUser "+
                    "SET displayName = ?, email = ?, password = ?, followUp = ?, preferredLanguage = ?, motherTongue = ?, lastLogin = ? "+
                    "WHERE forumUser = ? ";
            statement = conn.prepareStatement(query, PreparedStatement.RETURN_GENERATED_KEYS);
            statement.setString(1, displayName);
            statement.setString(2, email);
            statement.setString(3, password);
            statement.setBoolean(4, followUp);
            statement.setString(5, preferredLanguage.toString());
            statement.setString(6, motherTongue);
            statement.setDate(7, lastLogin);
            
            statement.setInt(8, id);
            if (statement.executeUpdate() != 1) {
                throw new SQLException("Couldn't find forumUser "+id+" for update");
            }
        } catch (SQLException sex) {
            throw sex;
        } finally {
            try {statement.close();} catch (Exception e) {e.printStackTrace();}
        }
    }
    
    
    private void sqlSetUserById(Connection conn, int id) throws SQLException {
        PreparedStatement statement = null;
        ResultSet rs = null;
        try {
            String query =
                    "SELECT forumUser, displayName, email, password, followUp, preferredLanguage, motherTongue, creationDate, lastLogin, admin "+
                    "FROM ForumUser "+
                    "WHERE forumUser = ? ";
            statement = conn.prepareStatement(query);
            statement.setInt(1,id);
            rs = statement.executeQuery();
            if (rs.next()) {
                this.id = rs.getInt("forumUser");
                this.displayName = rs.getString("displayName");
                this.email = rs.getString("email");
                this.password = rs.getString("password");
                this.followUp = rs.getBoolean("followUp");
                this.preferredLanguage = Lang.valueOf(rs.getString("preferredLanguage"));
                this.motherTongue = rs.getString("motherTongue");
                this.creationDate = rs.getDate("creationDate");
                this.lastLogin = rs.getDate("lastLogin");
                this.admin = rs.getByte("admin");
                // we're now up to date
                this.originalUser = (User) clone();
            } else {
                throw new SQLException("Cannot find user with id: " + id);
            }
        } catch (SQLException sex) {
            throw sex;
        } finally {
            try {rs.close();} catch (Exception e) {e.printStackTrace();}
            try {statement.close();} catch (Exception e) {e.printStackTrace();}
        }
    }
    
    public void setFollowUp(String followUp) {
        if (followUp == null) followUp = "null";
        if (followUp.toLowerCase().equals("yes")) {
            setFollowUp(true);
        } else if (followUp.toLowerCase().equals("no")) {
            setFollowUp(false);
        } else {
            logger.warning("unkown followUp: " + followUp);
        }
    }
    
    public void setPreferredLanguage(String prefLang) {
        prefLang = prefLang != null ? prefLang.toLowerCase() : null;
        if ("de".equals(prefLang)) {
            setPreferredLanguage(Lang.de);
        } else if ("no".equals(prefLang)) {
            setPreferredLanguage(Lang.no);
        } else {
            logger.warning("unknown prefLang: " + prefLang);
        }
    }
    
}

