/*
 * AbstractMultiphone.java
 *
 * Created on October 18, 2005, 11:27 AM
 *
 */

package info.heinzelnisse.multiphone;

import java.util.ArrayList;
import java.util.Collection;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 *
 * @author Heiko Klein
 */
public abstract class AbstractMultiphone {
    
    /** Creates a new instance of AbstractMultiphone */
    public AbstractMultiphone() {
    }
    public Collection<String> multiphone(String word) {
        String newWord = preSplit(word);
        Collection<String> wordParts = doSplit(newWord);
        Collection<Collection<String>> wordParts2 = processBits(wordParts);
        Collection<String> joinedWords = doJoin(wordParts2);
        Collection<String> finalWords = postJoin(joinedWords);
        return finalWords;
    }
    
    abstract protected String vowels();
    
    abstract protected String preSplit(String word);
    
    /**a word consist of 2 levels of arrays
     * 1. level splits vowels and consonants
     * 2. level splits several meanings of this vowel or consonant
     * i.e. Alphabet will be [[a],[lf],[a], [b], [e], [t] ]
     * i.e. 
     */
    protected Collection<String> doSplit(String word) {
        ArrayList<String> splitted = new ArrayList<String>();
        Pattern pattern = Pattern.compile("(["+vowels()+"]*)([^"+vowels()+"]*)");
        Matcher matcher = pattern.matcher(word);
        while (matcher.find()) {
            splitted.add(matcher.group(1));
            splitted.add(matcher.group(2));
        }
        return splitted;
    }
    
    abstract protected Collection<Collection<String>> processBits(Collection<String> wordParts);
    
    protected Collection<String> doJoin(Collection<Collection<String>> partCollection) {
        ArrayList<String> results = new ArrayList<String>();
        results.add(""); // starting with empty String
        for (Collection<String> wordParts : partCollection) {
            ArrayList<String> newResults = new ArrayList<String>();
            int resultSize = 0;
            for (String part : wordParts) {
                for (String result : results) {
                    resultSize += result.length() + part.length();
                    if (resultSize > 10240) {
                        // giving up
                        resultSize /= 1024;
                        Logger.getLogger(AbstractMultiphone.class.getName()).warning("multiphoning uses more than " + resultSize+ "kB, stopping doJoin");
                        return new ArrayList<String>();
                    }                    
                    newResults.add(result + part);
                }
            }
            results = newResults;
        }
        return results;
    }
    
    abstract protected Collection<String> postJoin(Collection<String> words);
    
}
