/*
 * De.java
 *
 * Created on October 18, 2005, 12:25 PM
 *
 */

package info.heinzelnisse.multiphone;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Locale;

/**
 *
 * @author Heiko Klein
 */
public class De extends AbstractMultiphone {
    Locale loc = new Locale("de");
    
    /** Creates a new instance of De */
    public De() {
    }
    public String vowels() {
        return "aeiouyæøå";
    }
    protected String preSplit(String word) {
        word = word.toLowerCase(loc);
        word = word.replaceAll("ß", "s");
        word = word.replaceAll("qu","q"); // q is always alone
        word = word.replaceAll("sch","ch"); // ch == sch
        word = word.replaceAll("sc[^h]","ch"); // usual typo for sch
        word = word.replaceAll("sh","ch"); // usual typo for sch
        word = word.replaceAll("ck","k");
        word = word.replaceAll("ie","i");
        word = word.replaceAll("ph","f");
        word = word.replaceAll("pf","f");
        word = word.replaceAll("(\\w)\\1", "$1"); // removing double characters
        word = word.replaceAll("(\\w{2})\\1", "$1"); // removing double pairs as stst in "selbstständig"
        
        return word;
    }
    
    protected Collection<Collection<String>> processBits(Collection<String> wordParts) {
        int partNo = 0;
        ArrayList<Collection<String>> result = new ArrayList<Collection<String>>();
        for (String part : wordParts) {
            partNo++;
            ArrayList<String> splits = new ArrayList<String>();
            
            if (part.matches("v")) {
                // v sounds like w or f
                splits.add(part.replaceAll("v", "f"));
                splits.add(part.replaceAll("v", "w"));
            } else if (part.matches("y")) {
                // y sounds like ü(= u,i), i, j
                splits.add(part.replaceAll("y", "i"));
                splits.add(part.replaceAll("y", "j"));
                splits.add(part.replaceAll("y", "u"));
            } else if (part.equals("ü") || part.equals("ue")) {
                // ü sounds like u,i
                splits.add("u");
                splits.add("i");
            } else if (part.equals("ä") || part.equals("ae")) {
                // ä sounds like a,e
                splits.add("a");
                splits.add("e");
            } else if (part.equals("ö") || part.equals("oe")) {
                // ö sounds like o,e
                splits.add("o");
                splits.add("e");
            } else if (part.equals("ai")) {
                splits.add("ei");
            } else if (part.equals("oi")) {
                splits.add("eu");
            } else if (part.equals("c")) {
                splits.add("z");
            } else if(partNo > 1 && part.matches("^h")) {
                // ignore silent h after vowel(lengthening the vowel
                splits.add(part.replaceAll("^h",""));
            } else {
                splits.add(part);
            }
            result.add(splits);
        }
        return result;
    }
    
    protected Collection<String> postJoin(Collection<String> words) {
        // nothing to do here
        return words;
    }
    
}
