/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package info.heinzelnisse.trainer;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import javax.xml.namespace.QName;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 *
 * @author hklein
 */
public class OpenOfficeTrainerStore {
    private List<ArrayList<ArrayList<StringBuilder>>> tables;
    private List<String> tableNames;

    public OpenOfficeTrainerStore(URL url) throws IOException {
        readZipContent(url.openStream());
    }

    private void readZipContent(InputStream stream) throws IOException {
        ZipInputStream zis = new ZipInputStream(stream);
        ZipEntry ze = zis.getNextEntry();
        while (ze != null) {
            if (ze.getName().equals("content.xml")) {
                readContent(zis);
                break;
            }
            zis.closeEntry();
            ze = zis.getNextEntry();
        }
    }
    private void readContent(InputStream is) {
        try {
            XMLReader xr = XMLReaderFactory.createXMLReader();
            xr.setFeature("http://xml.org/sax/features/validation", false);

            SAXReadXMLHandler fxh = new SAXReadXMLHandler();

            xr.setContentHandler(fxh);
            xr.setErrorHandler(fxh);
            xr.parse(new InputSource(is));
            tableNames = fxh.getTableNames();
            tables = fxh.getTables();
        } catch (SAXException sx) {
            sx.printStackTrace();
        } catch (IOException iox) {
            iox.printStackTrace();
        }

    }

    /**
     * @return the tables
     */
    public List<ArrayList<ArrayList<StringBuilder>>> getTables() {
        return tables;
    }

    /**
     * @return the tableNames
     */
    public List<String> getTableNames() {
        return tableNames;
    }

    private class SAXReadXMLHandler extends org.xml.sax.helpers.DefaultHandler {

        final String nameSpaceURI = "urn:oasis:names:tc:opendocument:xmlns:table:1.0";
        final QName qTableName = new QName(nameSpaceURI, "table");
        final QName qRowName = new QName(nameSpaceURI, "table-row");
        final QName qCellName = new QName(nameSpaceURI, "table-cell");
        final String qCollsRepeated = "table:number-columns-repeated";
        final QName qCellTextName = new QName("urn:oasis:names:tc:opendocument:xmlns:text:1.0", "p");

        boolean inCellText = false;
        StringBuilder cellText = null;
        ArrayList<String> tableNames = new ArrayList<String>();
        ArrayList<ArrayList<ArrayList<StringBuilder>>> tables = new ArrayList<ArrayList<ArrayList<StringBuilder>>>();

        ArrayList<ArrayList<StringBuilder>> currentTable = null;
        ArrayList<StringBuilder> currentRow = null;

        List<String> getTableNames() {
            return tableNames;
        }
        List<ArrayList<ArrayList<StringBuilder>>> getTables() {
            return tables;
        }

        @Override
        public void startElement(String uri, String name, String qName, Attributes atts) {
            QName qqName = new QName(uri,name);
            if (isTable(qqName)) {
                tableNames.add(atts.getValue(nameSpaceURI, "name"));
                currentTable = new ArrayList<ArrayList<StringBuilder>>();
                tables.add(currentTable);
            } else if (currentTable != null && isRow(qqName)) {
                currentRow = new ArrayList<StringBuilder>();
                currentTable.add(currentRow);
            } else if (currentTable != null && currentRow != null && isCell(qqName)) {
                cellText = new StringBuilder();
                currentRow.add(cellText);
                String repeat = atts.getValue(qCollsRepeated);
                if (repeat != null && !repeat.equals("")) {
                    int iRepeat = Integer.parseInt(repeat);
                    for (int i = 1; i < iRepeat; i++) { // first one already added
                        currentRow.add(cellText);
                    }
                }
            } else if (cellText != null && isCellText(qqName)) {
                inCellText = true;
            }
        }

        @Override
        public void endElement(String uri, String name, String qName) {
            QName qqName = new QName(uri,name);
//            System.err.println(qqName);
            if (isTable(qqName)) {
                currentTable = null;
            } else if (isRow(qqName)) {
                currentRow = null;
            } else if (isCell(qqName)) {
                assert(currentRow != null);
                cellText = null;
            } else if (isCellText(qqName)) {
                inCellText = false;
            }
        }

        @Override
        public void characters(char[] ch, int start, int length) throws SAXException {
            if (inCellText) {
                assert(cellText != null);
                cellText.append(ch, start, length);
            }
        }
        public boolean isTable(QName qname) {
            return qname.equals(qTableName);
        }
        public boolean isRow(QName qname) {
            return qname.equals(qRowName);
        }
        public boolean isCell(QName qname) {
            return qname.equals(qCellName);
        }
        public boolean isCellText(QName qname) {
            return qname.equals(qCellTextName);
        }

    }

}
