/*
 */

package info.heinzelnisse.utils;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 *
 * @author Heiko Klein
 */
public class Formatter {
    List<LineFormatter> lineFormatters = new ArrayList<LineFormatter>();
    List<FormatterTag> tags = new ArrayList<FormatterTag>();
    static final Pattern pSlash = Pattern.compile("/");
    static final Pattern pAsterix = Pattern.compile("\\*");
    static final Pattern pUnderscore = Pattern.compile("_");
    static final Pattern pAmpersand = Pattern.compile("&");
    static final Pattern pGt = Pattern.compile(">");
    static final Pattern pLt = Pattern.compile("<");
    
    /** Creates a new instance of Formatter */
    public Formatter(InputStream tagsStream) {
        try {
            XMLReader xr = XMLReaderFactory.createXMLReader();
            xr.setFeature("http://xml.org/sax/features/validation", false);
            
            FormatterXMLHandler fxh = new FormatterXMLHandler();
            
            xr.setContentHandler(fxh);
            xr.setErrorHandler(fxh);
            xr.parse(new InputSource(tagsStream));
        } catch (SAXException sx) {
            sx.printStackTrace();
        } catch (IOException iox) {
            iox.printStackTrace();
        }
    }
        
    public String formatLists(String text) {
        // TODO, maybe join with formatTables
        return text;
    }
    
    private String formatByLineFormatters(String text, List<LineFormatter> lineFormatters) {
        for (LineFormatter lf : lineFormatters) {
            LineFormatter llf;
            try {
                // lineFormatters have a state, so local copy required
                llf = lf.clone();
            } catch (CloneNotSupportedException ex) {
                Logger.getLogger(Formatter.class.getName()).log(Level.SEVERE, null, ex);
                return text;
            }
            StringBuffer out = new StringBuffer();
            for (String line : text.split("\\n")) {
                out.append(llf.formatLine(line));
            }
            out.append(llf.finish());
            text = out.toString();
        }
        return text;
    }
    
    public String format(String text) {
        // make sure the text finishes with a counted empty line
        text = text + "\n \n";
        text = escapeHTML(text);
        text = formatByLineFormatters(text, lineFormatters);
        for (FormatterTag tag : tags) {
            Matcher matcher = tag.getWikiPattern().matcher(text);
            StringBuffer sb = new StringBuffer();
            while (matcher.find()) {
                String currentMatch = matcher.group(0);
                Matcher subMatcher = tag.getWikiPattern().matcher(currentMatch);
                currentMatch = subMatcher.replaceAll(tag.getHtmlTag());
                if (tag.doEscape()) {
                    currentMatch = escapeWikiCode(currentMatch);
                }
                matcher.appendReplacement(sb, currentMatch);
            }
            matcher.appendTail(sb);
            text = sb.toString();
        }
        return text;
    }
    
    
    static public String escapeWikiCode(String text) {
        if (text == null) return null;
        text = pSlash.matcher(text).replaceAll("//");
        text = pAsterix.matcher(text).replaceAll("**");
        text = pUnderscore.matcher(text).replaceAll("__");
        return text;
    }
    static public String escapeHTML(String text) {
        if (text == null) return null;
        text = pAmpersand.matcher(text).replaceAll("&amp;");
        text = pGt.matcher(text).replaceAll("&gt;");
        return pLt.matcher(text).replaceAll("&lt;");
    }
    
    @Override
    protected Formatter clone() throws CloneNotSupportedException {
        throw new CloneNotSupportedException();
    }
    
    private class FormatterXMLHandler extends org.xml.sax.helpers.DefaultHandler {
        @Override
        public void startElement(String uri, String name, String qName, Attributes atts) {
            if ("tag".equals(qName)) {
                String attName = atts.getValue("name");
                String wikiMatch = atts.getValue("wikiMatch");
                String htmlTag = atts.getValue("htmlTag");
                String escape = atts.getValue("escape");
                tags.add(new FormatterTag(attName, htmlTag, wikiMatch, escape));
            }
            if ("lineformatter".equals(qName)) {
                String vname = atts.getValue("name");
                if ("block".equals(vname)) {
                    lineFormatters.add(new BlockLineFormatter());
                } else if ("table".equals(vname)) {
                    lineFormatters.add(new TableLineFormatter());
                } else if ("list".equals(vname)) {
                    lineFormatters.add(new ListLineFormatter());
                } else {
                    Logger.getAnonymousLogger().warning("cannot find lineformatter: "+vname);
                }
            }
        }
        
    }
    static private class FormatterTag {
        private String name;
        private String htmlTag;
        private Pattern wikiPattern;
        private boolean escape = false;
        
        public FormatterTag(String name, String htmlTag, String wikiMatch, String escape) {
            this.name = name;
            this.htmlTag = htmlTag;
            wikiPattern = Pattern.compile(wikiMatch);
            if ("true".equals(escape)) {
                this.escape = true;
            }
        }
        
        public String getName() {
            return name;
        }
        
        public String getHtmlTag() {
            return htmlTag;
        }
        
        public Pattern getWikiPattern() {
            return wikiPattern;
        }
        public boolean doEscape() {
            return escape;
        }
    }
    
}
