/*
 * ListLineFormatter.java
 *
 * Created on May 14, 2006, 2:17 PM
 *
 */

package info.heinzelnisse.utils;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 *
 * @author Heiko Klein
 */
class ListLineFormatter implements LineFormatter, Cloneable {
    List<String> itemList;
    static final Pattern ptrn = Pattern.compile("^([\\#0]+)\\s*(.*)");
    
    /** Creates a new instance of ListLineFormatter */
    public ListLineFormatter() {
        itemList = new ArrayList<String>();
    }
    
    @Override
    public String formatLine(String line) {
        StringBuffer out = new StringBuffer();
        Matcher m = ptrn.matcher(line);
        if (m.find()) {
            String match = m.group(1);
            int currentOrder = match.length();
            String lastItem = "";
            if (itemList.size() != 0) {
                lastItem = itemList.get(itemList.size() - 1);
            }
            int last = lastItem.length();
            if (currentOrder == last) {
                out.append(" <//li>\n<li> ");
            } else if (currentOrder > last) {
                itemList.add(match);
                if (!lastItem.equals("")) {
                    out.append(" ");
                }
                out.append(startTag(match));
                out.append("<li> ");
            } else {
                finishUntil(match, out);
                out.append("<//li>\n<li> ");
            }
            out.append(m.group(2));
        } else {
            if (line.trim().equals("")) {
                // empty line, return
                return finish() + "\n";
            } else {
                if (itemList.size() > 0) {
                    // continuing line inside list
                    return  line + " ";
                } else {
                    // outside list simply print line
                    return line + "\n";
                }
            }
        }
        return out.toString();
    }
    
    private void finishUntil(String match, StringBuffer out) {
        boolean first = true;
        while (itemList.size() > 0) {
            String last = itemList.remove(itemList.size()-1);
            if (last.length() > match.length()) {
                if (first) {
                    out.append(" ");
                    first = false;
                }
                out.append("<//li>\n");
                out.append(endTag(last));
            } else {
                itemList.add(last);
                break;
            }
        }
    }

    private String startTag(String m) {
        if (m.startsWith("#")) {
            return "<ul>\n";
        } else {
            return "<ol>\n";
        }
    }
    
    private String endTag(String m) {
        if (m.startsWith("#")) {
            return "<//ul>\n";
        } else {
            return "<//ol>\n";
        }
    }
    
    @Override
    public String finish() {
        StringBuffer out = new StringBuffer();
        finishUntil("", out);
        return out.toString();
    }

    @Override
    public ListLineFormatter clone() throws CloneNotSupportedException {
        ListLineFormatter llf = (ListLineFormatter) super.clone();
        // standard clone isn't deep, create a new arrayList
        llf.itemList = new ArrayList<String>();
        return llf;
    }
    
}
