/*
 * DiffManager.java
 *
 * Created on May 28, 2006, 2:16 PM
 *
 */

package info.heinzelnisse.wiki;

import com.nothome.delta.Delta;
import com.nothome.delta.DeltaException;
import com.nothome.delta.DiffWriter;
import com.nothome.delta.GDiffPatcher;
import com.nothome.delta.GDiffWriter;
import com.nothome.delta.PatchException;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;

/**
 *
 * @author Heiko Klein
 */
public class DiffManager {
    static final String CHARSET = "UTF-8";
    public static String patch(String source, byte[] delta) throws PatchException {
        ByteArrayInputStream deltaIS = new ByteArrayInputStream(delta);
        ByteArrayOutputStream patchedOS = new ByteArrayOutputStream();
        try {
            new GDiffPatcher(source.getBytes(CHARSET), deltaIS, patchedOS);
            deltaIS.close();
            patchedOS.close();
        } catch (IOException ioe) {
            ioe.printStackTrace(); // should never happen in memory
        }
        String ret = "";
        try {
            ret = patchedOS.toString(CHARSET);
        } catch (UnsupportedEncodingException uee) {
            uee.printStackTrace();
        }
        return ret;
    }
    
    public static byte[] delta(String source, String target) throws DeltaCreationException {
        ByteArrayOutputStream delta = new ByteArrayOutputStream();
        if (source == null) {
            source = "";
        }
        if (target == null) {
            target = "";
        }
        DiffWriter output = null;
        try {
            output = new GDiffWriter(new DataOutputStream(new BufferedOutputStream(delta)));
            byte[] targetBytes = target.getBytes(CHARSET);
            Delta.computeDelta(source.getBytes(CHARSET), new ByteArrayInputStream(targetBytes), (int) targetBytes.length, output);
        } catch (IOException ex) {
            throw new DeltaCreationException("IO error");
        } catch (DeltaException e) {
            throw new DeltaCreationException("Delta error");
        } finally {
            if (output != null) {
                try {
                    output.close();
                } catch (IOException ex) {
                    throw new DeltaCreationException("IO error");
                }
            }
        }
        return delta.toByteArray();
    }
}
