/*
 * KwikiConverter.java
 *
 * Created on May 14, 2006, 6:10 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package info.heinzelnisse.wiki;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.InputStreamReader;
import java.io.Serializable;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 *
 * @author Heiko Klein
 */
public class KwikiConverter {
    
    /** Creates a new instance of KwikiConverter */
    public KwikiConverter(String directory) {
        System.err.println("running in: " + directory);
        try {
            File dir = new File(directory);
            if (!dir.isDirectory()) {
                throw (new Exception(directory + "not a directory"));
            }
            List<File> rcsFiles = Arrays.asList(dir.listFiles(new RCSConvertedFileFilter()));
            Collections.sort(rcsFiles, new VersionExtensionComparator());
            for (File f : rcsFiles) {
                if (f.length() < 30) {
                    continue;
                }
                if (f.getName().startsWith("Kwiki")) {
                    continue; // ignore all Kwiki pages
                }
                String filename = f.getAbsolutePath();
                InputStreamReader isr = new InputStreamReader(new FileInputStream(f), "UTF-8");
                BufferedReader br = new BufferedReader(isr);
                StringBuffer sb = new StringBuffer();
                String line = br.readLine();
                while (line != null) {
                    // replace bullet at beginning of line with #
                    line = line.replaceAll("^\\*([^*]*)$", "#$1");
                    line = line.replaceAll("^([^/]*)/([^/]*)$", "$1//$2");
                    line = line.replaceAll("\\|([^/|])*/([^/|])\\|", "|$1//$2|");
                    sb.append(line + "\n");
                    line = br.readLine();
                }
                br.close();
                String fileName = f.getName();
                System.err.println(fileName);
                fileName = fileName.replaceAll("(.*)\\.\\d+","$1");
                // user 1 is heinzelnisse (or should be)
                Page.createOrUpdate(fileName, sb.toString(), 0, new Date(), null);
                Page page = Page.getPage(fileName);
//                if (!page.getContent().equals(sb.toString())) {
//                    System.err.println("error in update");
//                    System.exit(1);
//                }
//                String converted = fmt.format(sb.toString());
//                File to = new File(filename + ".html");
//                OutputStreamWriter osw = new OutputStreamWriter(new FileOutputStream(to), "UTF-8");
//                osw.write("<html><head> <meta http-equiv=\"content-type\" content=\"text/html; charset=utf-8\"></head><body>\n");
//                osw.write(converted);
//                osw.write("</body></html>\n");
//                osw.close();
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    /**
     * @param args the command line arguments
     *
     * requires a directory with the rcs-decoded files as file.1, file.2 ...
     * (see rcs2txt.pl)
     * Loads data to the database
     */
    public static void main(String[] args) {
        new KwikiConverter("/home/hklein/Programme/Heinzelnisse/Backup");
    }
    
    static private class NoHTMLFile implements FilenameFilter {
        @Override
        public boolean accept(File dir, String name) {
            if (!name.matches(".*\\.(?i)html?")) {
                return true;
            }
            return false;
        }
    }
    static private class RCSConvertedFileFilter implements FilenameFilter {
        @Override
        public boolean accept(File dir, String name) {
            if (name.matches(".*\\.\\d+")) {
                return true;
            }
            return false;
        }
    }
    static private class VersionExtensionComparator implements Comparator<File>, Serializable {
        Pattern pattern = Pattern.compile(".*\\.(\\d+)");
        @Override
        public int compare(File o1, File o2) {
            Matcher m1 = pattern.matcher(o1.getName());
            Matcher m2 = pattern.matcher(o2.getName());
            if (m1.matches() && m2.matches()) {
                Integer i1 = new Integer(m1.group(1));
                Integer i2 = new Integer(m2.group(1));
                return i1.compareTo(i2);
            } else {
                System.err.println("Not a version filename: " + o1.getName() + " " + o2.getAbsoluteFile());
                return o1.compareTo(o2);
            }
        }
        
    }
}
