/*
 * ListLineFormatter.java
 *
 * Created on May 14, 2006, 2:17 PM
 *
 */

package info.heinzelnisse.wiki;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 *
 * @author Heiko Klein
 */
public class ListLineFormatter implements LineFormatter {
    List<ListItem> inList;
    Pattern numberPtrn = Pattern.compile("^(0+)\\s*(.*)");
    Pattern bulletPtrn = Pattern.compile("^(\\#+)\\s*(.*)");
    
    /** Creates a new instance of ListLineFormatter */
    public ListLineFormatter() {
        inList = new ArrayList<ListItem>();
    }
    
    public String formatLine(String line) {
        StringBuffer out = new StringBuffer();
        Matcher m = numberPtrn.matcher(line);
        if (m.find()) {
            // number
            int currentOrder = m.group(1).length();
            int last = lastNumberOrder();
            NumberItem ni = new NumberItem(currentOrder);
            if (currentOrder > last) {
                inList.add(ni);
                out.append(ni.startTag());
            } else {
                out.append(finishUntil(ni));
            }
            out.append("<li> ");
            out.append(m.group(2));
            out.append(" <//li>\n");
        } else {
            m = bulletPtrn.matcher(line);
            if (m.find()) {
                int currentOrder = m.group(1).length();
                int last = lastBulletOrder();
                BulletItem bi = new BulletItem(currentOrder);
                if (currentOrder > last) {
                    inList.add(bi);
                    out.append(bi.startTag());
                } else {
                    out.append(finishUntil(bi));
                }
                out.append("<li> ");
                out.append(m.group(2));
                out.append(" <//li>\n");
                
            } else {
                if (line.trim().equals("")) {
                    return finish() + "\n";
                } else {
                    return  line + "\n";
                }
            }
        }
        return out.toString();
    }
    
    private String finishUntil(ListItem li) {
        StringBuffer out = new StringBuffer();
        ArrayList<ListItem> newList = new ArrayList<ListItem>();
        Collections.reverse(inList);
        boolean isHigherOrder = true;
        for (ListItem ili : inList) {
            if (isHigherOrder == true) {
                if (ili.getClass().equals(li.getClass())) {
                    if (ili.getOrder() <= li.getOrder()) {
                        isHigherOrder = false;
                        newList.add(li);
                        continue;
                    }
                }
                out.append(ili.endTag());
            } else {
                newList.add(ili);
            }
        }
        
        Collections.reverse(newList);
        inList = newList;
        return out.toString();
    }
    
    public String finish() {
        StringBuffer out = new StringBuffer();
        Collections.reverse(inList);
        for (ListItem li : inList) {
            out.append(li.endTag());
        }
        inList.clear();
        return out.toString();
    }
    
    private int lastBulletOrder() {
        int last = -1;
        for (ListItem li : inList) {
            if (li instanceof BulletItem) {
                last = li.getOrder();
            }
        }
        return last;
    }
    private int lastNumberOrder() {
        int last = -1;
        for (ListItem li : inList) {
            if (li instanceof NumberItem) {
                last = li.getOrder();
            }
        }
        return last;
    }
    
    
    
    abstract protected class ListItem {
        int order  = 0;
        public ListItem(int order) {
            this.order = order;
        }
        public int getOrder() {
            return order;
        }
        abstract public boolean equals(Object o);
        abstract public String startTag();
        abstract public String endTag();
    }
    private class BulletItem extends ListLineFormatter.ListItem {
        public BulletItem(int order) {
            super(order);
        }
        public String startTag() {
            return "<ul>\n";
        }
        public String endTag() {
            return "<//ul>\n";
        }
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o != null) {
                if (o instanceof BulletItem) {
                    return ((ListItem) o).order == order;
                }
            }
            return false;
        }
    }
    private class NumberItem extends ListItem {
        public NumberItem(int order) {
            super(order);
        }
        public String startTag() {
            return "<ol>\n";
        }
        public String endTag() {
            return "<//ol>\n";
        }
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o != null) {
                if (o instanceof NumberItem) {
                    return ((ListItem) o).order == order;
                }
            }
            return false;
        }
    }
}
