/*
 * Page.java
 *
 * Created on May 28, 2006, 1:04 PM
 *
 */

package info.heinzelnisse.wiki;

import com.nothome.delta.PatchException;
import info.heinzelnisse.dictionary.DictUtils;
import info.heinzelnisse.forum.User;
import java.sql.Blob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 *
 * @author Heiko Klein
 */
public class Page {
    private int id = 0;
    private String name;
    private String content;
    private int version;
    private int userId = 0;
    private boolean admin = false;
    private User user = null;
    private Date timestamp;
    private String address;
    
    /**
     * Creates a new instance of Page
     *
     * admin can currently only be changed via db, not via java
     */
    public Page(int id, String name, String content, boolean admin, int version, int userId, Date timestamp, String address) {
        this.id = id;
        this.name = name;
        this.content = content;
        this.admin = admin;
        this.version = version;
        this.userId = userId;
        this.timestamp = timestamp;
        this.address = address;
    }
    private static Page connCreatePage(Connection conn, String name, String content, int userId, Date timestamp, String address) throws SQLException {
        Page page = null;
        PreparedStatement statement = null;
        ResultSet rs = null;
        try {
            int version = 1;
            String query =
                    "INSERT INTO WikiCurrentPage (name, content, version, forumUser, timestamp, address) "+
                    "VALUES (?,?,?,?,?,?) ";
            statement = conn.prepareStatement(query, PreparedStatement.RETURN_GENERATED_KEYS);
            statement.setString(1, name);
            statement.setString(2, content);
            statement.setInt(3, version);
            statement.setInt(4,  userId);
            statement.setTimestamp(5, new Timestamp(timestamp.getTime()));
            statement.setString(6, address);
            statement.execute();
            rs = statement.getGeneratedKeys();
            if (rs.first()) {
                // reading from database in case characters etc have changed
                page = connFetchPage(conn, name);
                // page = new Page(rs.getInt(1), name, content, version, userId, timestamp, address);
            } else {
                throw new SQLException("cannot generate new page");
            }
        } catch (SQLException sex) {
            throw sex;
        } finally {
            try {rs.close();} catch (Exception e) {e.printStackTrace();}
            try {statement.close();} catch (Exception e) {e.printStackTrace();}
        }
        return page;
    }
    
    public static Page createOrUpdate(String name, String newContent,  int userId, Date timestamp, String address) {
        Connection conn = null;
        PreparedStatement statement = null;
        Statement stat = null;
        ResultSet rs = null;
        Page page = null;
        try {
            conn = DictUtils.dbConnection();
            stat = conn.createStatement();
            stat.execute("LOCK TABLES WikiCurrentPage WRITE, WikiDelta WRITE");
            
            page = connFetchPage(conn, name);
            if (page == null) {
                return connCreatePage(conn, name, newContent, userId, timestamp, address);
            } else if (newContent.equals(page.content)) {
                return page;
            }
            // else
            
            // going backwards
            int newVersion = page.getVersion()+1;
            String oldContent = page.getContent();
            
            String pageQuery =
                    "UPDATE WikiCurrentPage SET content = ?, version = ?, forumUser = ?, timestamp = ?, address = ? " +
                    "WHERE wikiCurrentPage = ?";
            statement = conn.prepareStatement(pageQuery);
            statement.setString(1, newContent);
            statement.setInt(2, newVersion);
            statement.setInt(3, userId);
            statement.setTimestamp(4, new Timestamp(timestamp.getTime()));
            statement.setString(5, address);
            statement.setInt(6, page.getId());
            statement.execute();
            statement.close();
            
            // get the content as written to the database (character conversion might change things)
            statement = conn.prepareStatement("SELECT content FROM WikiCurrentPage WHERE wikiCurrentPage = ?");
            statement.setInt(1, page.getId());
            rs = statement.executeQuery();
            rs.first();
            newContent = rs.getString("content");
            rs.close();
            statement.close();
            
            byte[] delta = DiffManager.delta(newContent, oldContent);
            
            String deltaQuery =
                    "INSERT INTO WikiDelta (wikiCurrentPage, delta, version, forumUser, timestamp, address) "+
                    "VALUES (?,?,?,?,?,?) ";
            statement = conn.prepareStatement(deltaQuery);
            statement.setInt(1, page.getId());
            statement.setBytes(2, delta);
            statement.setInt(3, page.getVersion());
            statement.setInt(4,  page.getUserId());
            statement.setTimestamp(5, new Timestamp(page.getTimestamp().getTime()));
            statement.setString(6, page.getAddress());
            statement.execute();
            
            page = connFetchPage(conn, page.getName());
        } catch (SQLException se) {
            se.printStackTrace();
        } catch (DeltaCreationException dce) {
            dce.printStackTrace();
        } finally {
            try {if (rs != null) rs.close();} catch (Exception e) {e.printStackTrace();}
            try {if (statement != null) statement.close();} catch (Exception e) {e.printStackTrace();}
            try {stat.execute("UNLOCK TABLES");} catch (Exception e) {e.printStackTrace();}
            try {stat.close();} catch (Exception e) {e.printStackTrace();}
            try {conn.close();} catch (Exception e) {e.printStackTrace();}
        }
        return page;
    }
    
    public static Page getPage(String name) {
        Connection conn = null;
        Page page = null;
        try {
            conn = DictUtils.dbConnection();
            page = connFetchPage(conn, name);
        } catch (SQLException se) {
            se.printStackTrace();
        } finally {
            try {
                if (conn!=null) {
                    conn.close();
                }
            } catch (SQLException ex) {ex.printStackTrace();}
        }
        return page;
    }
    
    public static List<String> searchPages(String item) {
        List<String> results = new ArrayList<String>();
        Set<String> lookupSet = new HashSet<String>();
        if (item == null) return results;
        item = item.trim();
        if (item.equals("")) return results;
        
        String searchName = "SELECT name FROM WikiCurrentPage WHERE NAME LIKE ?";
        String searchContent = "SELECT name, MATCH(content) AGAINST(?) AS rank  from WikiCurrentPage where MATCH(content) AGAINST (?) ORDER BY rank DESC";
        Connection conn = null;
        PreparedStatement statement = null;
        ResultSet rs = null;
        PreparedStatement statement2 = null;
        ResultSet rs2 = null;
        Page page = null;
        try {
            conn = DictUtils.dbConnection();
            statement = conn.prepareStatement(searchName);
            statement.setString(1, '%' + item + '%');
            rs = statement.executeQuery();
            while (rs.next()) {
                String name = rs.getString("name");
                lookupSet.add(name);
                results.add(name);
            }
            rs.close();
            statement.close();
            
            statement2 = conn.prepareStatement(searchContent);
            statement2.setString(1, item);
            statement2.setString(2, item);
            rs2 = statement2.executeQuery();
            while (rs2.next()) {
                String name = rs2.getString("name");
                if (! lookupSet.contains(name)) {
                    results.add(name);
                }
            }
        } catch (SQLException se) {
            se.printStackTrace();
        } finally {
            try {rs.close();} catch (Exception e) {e.printStackTrace();}
            try {statement.close();} catch (Exception e) {e.printStackTrace();}
            try {rs2.close();} catch (Exception e) {e.printStackTrace();}
            try {statement2.close();} catch (Exception e) {e.printStackTrace();}
            try {conn.close();} catch (Exception e) {e.printStackTrace();}
        }
        return results;
    }
    /**
     * get a list of pages changed between earliestDate and latestDate
     * @return list of pages ordered by timestamp
     */
    public static List<Page> getChangedPages(Date earliestDate, Date latestDate) {
        Connection conn = null;
        PreparedStatement statement = null;
        ResultSet rs = null;
        List<Page> pages = new ArrayList<Page>();
        try {
            conn = DictUtils.dbConnection();
            String query =
                    "SELECT wikiCurrentPage, name, content, version, forumUser, timestamp, address, admin " +
                    "FROM WikiCurrentPage WHERE timestamp > ? AND timestamp < ?ORDER BY timestamp DESC";
            
            statement = conn.prepareStatement(query);
            statement.setTimestamp(1, new Timestamp(earliestDate.getTime()));
            statement.setTimestamp(2, new Timestamp(latestDate.getTime()));
            rs = statement.executeQuery();
            while (rs.next()) {
                pages.add(new Page(rs.getInt("wikiCurrentPage"), rs.getString("name"), rs.getString("content"), rs.getBoolean("admin"), rs.getInt("version"), rs.getInt("forumUser"), new Date(rs.getTimestamp("timestamp").getTime()), rs.getString("address")));
            }
        } catch (SQLException se) {
            se.printStackTrace();
        } finally {
            try {rs.close();} catch (Exception e) {e.printStackTrace();}
            try {statement.close();} catch (Exception e) {e.printStackTrace();}
            try {conn.close();} catch (Exception e) {e.printStackTrace();}
        }
        return pages;
    }
    
    public static Page getOldVersion(String name, int version) {
        Connection conn = null;
        Statement stat = null;
        Page page = null;
        try {
            conn = DictUtils.dbConnection();
            stat = conn.createStatement();
            stat.execute("LOCK TABLES WikiCurrentPage WRITE, WikiDelta WRITE");
            
            page = connFetchPage(conn, name);
            if (page == null) {
                return page;
            }
            int lastVersion = page.getVersion();
            String lastContent = page.getContent();
            while (lastVersion > version) {
                lastVersion--;
                byte[] delta = connFetchDelta(conn, page.getId(), lastVersion);
                if (delta == null) {
                    // no such version, does not happen normally
                    lastVersion++;
                    break;
                }
                lastContent = DiffManager.patch(lastContent, delta);
            }
            page.content = lastContent;
            page.version = lastVersion;
            page.connUpdateCredentials(conn);
        } catch (PatchException pe) {
            pe.printStackTrace();
        } catch (SQLException se) {
            se.printStackTrace();
        } finally {
            try {stat.execute("UNLOCK TABLES");} catch (Exception e) {e.printStackTrace();}
            try {stat.close();} catch (Exception e) {e.printStackTrace();}
            try {conn.close();} catch (Exception e) {e.printStackTrace();}
        }
        return page;
    }
    
    private static Page connFetchPage(Connection conn, String name) throws SQLException {
        Page page = null;
        PreparedStatement statement = null;
        ResultSet rs = null;
        
        try {
            String query =
                    "SELECT wikiCurrentPage, content, admin, version, forumUser, timestamp, address " +
                    "FROM WikiCurrentPage WHERE name = ?";
            statement = conn.prepareStatement(query);
            statement.setString(1, name);
            rs = statement.executeQuery();
            if (rs.first()) {
                page = new Page(rs.getInt("wikiCurrentPage"), name, rs.getString("content"), rs.getBoolean("admin"), rs.getInt("version"), rs.getInt("forumUser"), new Date(rs.getTimestamp("timestamp").getTime()), rs.getString("address"));
            }
        } catch (SQLException sex) {
            throw sex;
        } finally {
            try {rs.close();} catch (Exception e) {e.printStackTrace();}
            try {statement.close();} catch (Exception e) {e.printStackTrace();}
        }
        return page;
    }
    
    private static byte[] connFetchDelta(Connection conn, int pageId, int version) throws SQLException {
        byte[] delta = null;
        PreparedStatement statement = null;
        ResultSet rs = null;
        try {
            String query =
                    "SELECT delta " +
                    "  FROM WikiDelta " +
                    " WHERE wikiCurrentPage = ? AND version = ?";
            statement = conn.prepareStatement(query);
            statement.setInt(1, pageId);
            statement.setInt(2, version);
            rs = statement.executeQuery();
            if (rs.first()) {
                Blob blob = rs.getBlob("delta");
                delta = blob.getBytes(1, (int) blob.length());
            }
        } catch (SQLException sex) {
            throw sex;
        } finally {
            try {rs.close();} catch (Exception e) {e.printStackTrace();}
            try {statement.close();} catch (Exception e) {e.printStackTrace();}
        }
        return delta;
    }
    /**
     * update the current pages credentials from the delta-credentials
     */
    private void connUpdateCredentials(Connection conn) throws SQLException {
        PreparedStatement statement = null;
        ResultSet rs = null;
        try {
            String query =
                    "SELECT forumUser, timestamp, address " +
                    "  FROM WikiDelta " +
                    " WHERE wikiCurrentPage = ? AND version = ?";
            statement = conn.prepareStatement(query);
            statement.setInt(1, id);
            statement.setInt(2, version);
            rs = statement.executeQuery();
            while (rs.next()) {
                userId = rs.getInt("forumUser");
                user = null;
                timestamp = rs.getTimestamp("timestamp");
                address = rs.getString("address");
            }
        }  catch (SQLException sex) {
            throw sex;
        } finally {
            try {rs.close();} catch (Exception e) {e.printStackTrace();}
            try {statement.close();} catch (Exception e) {e.printStackTrace();}
        }
    }
    
    
    public int getId() {
        return id;
    }
    
    public String getName() {
        return name;
    }
    
    public String getContent() {
        return content;
    }
    
    public int getVersion() {
        return version;
    }
    
    public int getUserId() {
        return userId;
    }
    
    public User getUser() {
        if (user == null) {
            user = new User(userId);
        }
        return user;
    }
    
    public Date getTimestamp() {
        return timestamp;
    }
    
    public String getAddress() {
        return address;
    }
    
    public static List<PageVersionBean> getVersions(String pageName) {
        List<PageVersionBean> versions = new ArrayList<PageVersionBean>();
        Connection conn = null;
        PreparedStatement statement = null;
        Statement stat = null;
        ResultSet rs = null;
        Page page = null;
        try {
            conn = DictUtils.dbConnection();
            stat = conn.createStatement();
            stat.execute("LOCK TABLES WikiCurrentPage READ, WikiDelta READ, ForumUser READ");
            
            page = connFetchPage(conn, pageName);
            if (page == null) {
                return versions;
            }
            PageVersionBean pvb = new PageVersionBean();
            pvb.setVersion(page.getVersion());
            if (page.getUser() != null) {
                pvb.setUserName(page.getUser().getDisplayName());
            }
            pvb.setTimestamp(page.getTimestamp());
            pvb.setAddress(page.getAddress());
            versions.add(pvb);
            
            statement = conn.prepareStatement("SELECT version, displayName, timestamp, address from WikiDelta LEFT JOIN ForumUser ON WikiDelta.forumUser = ForumUser.forumUser WHERE wikiCurrentPage = ? ORDER BY version DESC");
            statement.setInt(1,page.getId());
            rs = statement.executeQuery();
            while (rs.next()) {
                pvb = new PageVersionBean();
                pvb.setVersion(rs.getInt("version"));
                pvb.setUserName(rs.getString("displayName"));
                pvb.setTimestamp(rs.getTimestamp("timestamp"));
                pvb.setAddress(rs.getString("address"));
                versions.add(pvb);
            }
            
        } catch (SQLException se) {
            se.printStackTrace();
        } finally {
            try {if (rs != null) rs.close();} catch (Exception e) {e.printStackTrace();}
            try {if (statement != null) statement.close();} catch (Exception e) {e.printStackTrace();}
            try {stat.execute("UNLOCK TABLES");} catch (Exception e) {e.printStackTrace();}
            try {stat.close();} catch (Exception e) {e.printStackTrace();}
            try {conn.close();} catch (Exception e) {e.printStackTrace();}
        }
        return versions;
    }

    public boolean isAdmin() {
        return admin;
    }

    public void setAdmin(boolean admin) {
        this.admin = admin;
    }
    
}
