/*
 * CronServlet.java
 *
 * Created on January 9, 2007, 8:34 PM
 */

package info.heinzelnisse.www;

import info.heinzelnisse.dictionary.CanooBokmaalLinkDictUtils;
import info.heinzelnisse.dictionary.DictStatisticCache;
import info.heinzelnisse.dictionary.DictUtils;
import java.io.*;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import java.util.logging.Logger;

import javax.servlet.*;
import javax.servlet.http.*;

/**
 *
 * @author Heiko Klein
 * @version
 */
public class CronServlet extends HttpServlet {
    private final ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);
    private static Logger logger = Logger.getLogger(CronServlet.class.getName());
    // tasks to be run when shutting down
    private final List<Runnable> onShutdownTasks = new ArrayList<Runnable>();

    @Override
    public void init() throws ServletException {
        
        // clearWordAccess Task, every 24 hours
        final Runnable clearWordAccess = new Runnable() {
            @Override
            public void run() { 
                int count = CanooBokmaalLinkDictUtils.getInstance().clearWordAccessLarger(200000);
                logger.info("clearWordAccessLarger(200000) found "+ count + " items");
            }
        };
        //final ScheduledFuture<?> clearWordAccessHandle = 
        scheduler.scheduleAtFixedRate(clearWordAccess, 1, 24, TimeUnit.HOURS);

        // move cached statistics to database, every 5 minutes
        final Runnable stats2Database = new Runnable() {
            @Override
            public void run() {
                Connection conn = null;
                try {
                    conn = DictUtils.dbConnection();
                    DictStatisticCache dsc = DictStatisticCache.getInstance();
                    Map<DictUtils.DictType, Integer> usageCounter = dsc.clearUsageCounter();
                    CanooBokmaalLinkDictUtils.getInstance().incrementUsageCounter(conn, usageCounter);
                    List<String>[] wordAccess = dsc.clearWordAccess();
                    CanooBokmaalLinkDictUtils.getInstance().addWordAccess(conn, wordAccess);
                    logger.fine("writing statistics to discs");
                } catch (SQLException se) {
                    se.printStackTrace();
                } finally {
                    try {conn.close();} catch (Exception e) {e.printStackTrace();}
                }
            }
        };
        onShutdownTasks.add(stats2Database); 
        // final ScheduledFuture<?> stats2DatabaseHandle = 
        scheduler.scheduleAtFixedRate(stats2Database, 5, 5, TimeUnit.MINUTES);
        
        
        
 //        scheduler.schedule(new Runnable() {
//                public void run() { clearWordAccessHandle.cancel(true); }
//            }, 60 * 60, SECONDS);
    }
    
    @Override
    public void destroy() {
        scheduler.shutdown();
        for (Runnable r : onShutdownTasks) {
            r.run();
        }
    }
    
    /** Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
     * @param request servlet request
     * @param response servlet response
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        response.setContentType("text/html;charset=UTF-8");
        PrintWriter out = response.getWriter();
        /* TODO output your page here
        out.println("<html>");
        out.println("<head>");
        out.println("<title>Servlet CronServlet</title>");
        out.println("</head>");
        out.println("<body>");
        out.println("<h1>Servlet CronServlet at " + request.getContextPath () + "</h1>");
        out.println("</body>");
        out.println("</html>");
         */
        out.close();
    }
    
    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /** Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        processRequest(request, response);
    }
    
    /** Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        processRequest(request, response);
    }
    
    /** Returns a short description of the servlet.
     */
    @Override
    public String getServletInfo() {
        return "Dummy Servlet for scheduled tasks";
    }
    // </editor-fold>
}
