/*
 * Dict.java
 *
 * Created on January 8, 2006, 2:49 PM
 */

package info.heinzelnisse.www;

import info.heinzelnisse.dictionary.DictStatisticCache;
import info.heinzelnisse.dictionary.DictUtils;
import info.heinzelnisse.wiki.Page;
import info.heinzelnisse.www.wiki.PageBean;
import java.util.logging.Logger;
import javax.servlet.http.*;

/**
 * * Created on January 8, 2006, 2:49 PM
 *
 * @author Heiko Klein
 * @version
 */
public class Dict extends BaseServlet {

    private static Logger logger = Logger.getLogger(Dict.class.getName());

    /** Body of the page
     * @param request servlet request
     * @param response servlet response
     */
    @Override
    protected void processRequestBody(HttpServletRequest request, ResponseCache responseCache) {
        SearchSettingsBean ssb = (SearchSettingsBean) request.getSession().getAttribute("dictSearchSettings");
        if (ssb == null) {
            logger.fine("initialize new searchsettingsbean");
            ssb = getDefaultSearchSettingsBean();
            request.getSession().setAttribute("dictSearchSettings", ssb);
        }
        ssb.addRequestParameters(request);
        if (ssb.equals(getDefaultSearchSettingsBean())) {
            // don't store default values, can be recreate otherwise
            request.getSession().removeAttribute("dictSearchSettings");
        }
        SearchBean sb = new SearchBean();
        sb.setSsb(ssb);
        
        request.setAttribute("searchSettings", sb);
        // enter statistics
        if (ssb.getSearchItem() != null && (!ssb.getSearchItem().equals(""))) {
            logger.fine("adding statistics");
            DictStatisticCache dsc = DictStatisticCache.getInstance();
            dsc.incrementUsageCounter(DictUtils.DictType.dict);
            if ((sb.getDeTranslations().size() + sb.getNoTranslations().size()) > 0) {
                dsc.addWordAccess(ssb.getSearchItem(), true);
            } else {
                dsc.addWordAccess(ssb.getSearchItem(), false);
            }
        }
        responseCache.addIncludePage("/WEB-INF/fragments/search/field.jsp");
        responseCache.addIncludePage("/WEB-INF/fragments/search/results.jsp");

        // News from Wiki
        if (ssb.getSearchItem() == null || (ssb.getSearchItem().equals(""))) {
            PageBean pb = new PageBean();
            Page page = Page.getPage("News");
            if ((page != null) && (page.getContent() != null)) {
                pb.setPage(page);
                request.setAttribute("newsContent", pb.getPageAsHtml());
                responseCache.addIncludePage("/WEB-INF/fragments/news.jsp");
            }
        }
    }

    /**
     * Generate a SearchSettingsBean with default values for dict
     * @return default dict-SearchSettingsBean
     */
    static SearchSettingsBean getDefaultSearchSettingsBean() {
        SearchSettingsBean ssb = new SearchSettingsBean();
        ssb.setDictDeNoSearch(true);
        ssb.setDictNoDeSearch(true);
        ssb.setDictExactSearch(true);
        ssb.setDictPhoneticSearch(true);
        ssb.setDictBokmaalSearch(true);
        ssb.setDictNynorskSearch(true);
        ssb.setForumKeywordSearch(true);
        ssb.setForumSearch(false);
        ssb.setWikiSearch(true);
        ssb.setSuggestion(true);
        ssb.setSearchResultOrder(SearchSettingsBean.SearchResultPage.Dict, SearchSettingsBean.SearchResultPage.Wiki, SearchSettingsBean.SearchResultPage.Forum);
        return ssb;
    }
    
    @Override
    protected CardBean.Card getCard() {
        return CardBean.Card.Dict;
    }

    /** Returns a short description of the servlet.
     */
    @Override
    public String getServletInfo() {
        return "Heinzelnisse Dict";
    }
}