/*
 * DictAdmin.java
 *
 * Created on September 6, 2006, 8:48 PM
 */

package info.heinzelnisse.www;

import info.heinzelnisse.dictionary.DbTranslationStore;
import info.heinzelnisse.dictionary.Dict4MidCSVTranslationStore;
import info.heinzelnisse.dictionary.DictTables;
import info.heinzelnisse.dictionary.DictUtils;
import info.heinzelnisse.dictionary.HeinzellisteJSONTranslationStore;
import info.heinzelnisse.dictionary.HeinzellisteTranslationStore;
import info.heinzelnisse.dictionary.OpenOfficeTableLoader;
import info.heinzelnisse.dictionary.dicttables.DictTablesStorer;
import info.heinzelnisse.forum.User;
import java.io.*;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.servlet.*;
import javax.servlet.http.*;
import org.apache.commons.fileupload.DefaultFileItemFactory;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.FileUpload;
import org.apache.commons.fileupload.FileUploadException;

/**
 * * Created on September 6, 2006, 8:48 PM
 *
 * @author Heiko Klein
 * @version
 */
public class DictAdmin extends BaseServlet {
    private static final Logger logger = Logger.getLogger(DictAdmin.class.getName());
    enum Task {ShowPage, UploadList, ReadAndDeleteWords, RevertToUnchangedText};
    private Map<String, Task> tasks = new HashMap<String, Task>();
    
    /** Body of the page
     * @param request servlet request
     * @param response servlet response
     */
    @Override
    protected void processRequestBody(HttpServletRequest request, ResponseCache responseCache) {
        User user = getUserSettings(request).getUser();
        logger.log(Level.FINE, "User: {0} Adim?{1}", new Object[]{user.getDisplayName(), user.isAdmin()});
        if (user.isSuperAdmin()) {
            Task toDo = tasks.get(request.getParameter(".State"));
            if (toDo == null) toDo = Task.ShowPage;
            logger.log(Level.FINE, "State:{0}", request.getParameter(".State"));
            if (FileUpload.isMultipartContent(request)) {
                toDo = Task.UploadList;
            }
            logger.log(Level.FINE, "Task: {0}", toDo);
            switch (toDo) {
                case ShowPage: showPage(request, responseCache); break;
                case UploadList: uploadList(request, responseCache); break;
                case ReadAndDeleteWords: readAndDeleteWords(request, responseCache); break;
                default: showPage(request, responseCache);
            }
            
        } else {
            showPage(request, responseCache);
        }
    }
    
    @Override
    protected CardBean.Card getCard() {
        return null;
    }
    
    /** Returns a short description of the servlet.
     */
    @Override
    public String getServletInfo() {
        return "Heinzelnisse DictAdmin";
    }
    @Override
    public void init() throws ServletException {
        tasks.put("show", Task.ShowPage);
        tasks.put("upload", Task.UploadList);
        tasks.put("readAndDelete", Task.ReadAndDeleteWords);
        tasks.put("revertToUnchangedText", Task.RevertToUnchangedText);
    }
    
    private void showPage(HttpServletRequest request, ResponseCache responseCache) {
        responseCache.addIncludePage("/WEB-INF/fragments/dictAdminShow.jsp");
    }
    
    private void uploadList(HttpServletRequest request, ResponseCache responseCache) {
        // Check that we have a file upload request
        boolean isMultipart = FileUpload.isMultipartContent(request);
        if (isMultipart) {
            // Create a factory for disk-based file items
            FileItemFactory factory = new DefaultFileItemFactory();
            
            // Create a new file upload handler
            FileUpload upload = new FileUpload(factory);
            upload.setSizeMax(4000000);
            try {
                // Parse the request
                List<FileItem> items = upload.parseRequest(request);
                for (FileItem item : items) {
                    String paramName = item.getFieldName();
                    logger.log(Level.FINE, "got file for param: {0}", paramName);
                    if (paramName != null && paramName.equals("heinzelliste")) {
                        String contentType = item.getContentType();
                        logger.log(Level.FINE, "has contentType: {0}", contentType);
                        if (contentType != null && contentType.equals("application/vnd.oasis.opendocument.spreadsheet")) {
                            try {
                                DictUtils.dbStoreOOTable(item.getInputStream(), (int) item.getSize());
                                logger.log(Level.FINE, "stored file 'heinzeltable' in db");
                            } catch (IOException ex) {
                                Logger.getLogger(DictAdmin.class.getName()).log(Level.SEVERE, null, ex);
                            }
                            OpenOfficeTableLoader ootl = new OpenOfficeTableLoader();
                            //ootl.addTranslationStore(DbTranslationStore.getInstance());
                            ootl.addTranslationStore(new HeinzellisteTranslationStore());
                            ootl.addTranslationStore(new HeinzellisteJSONTranslationStore());
                            ootl.addTranslationStore(new Dict4MidCSVTranslationStore());
                            long startTime = System.currentTimeMillis();
                            DictUtils.dbReadOOTable(ootl);
                            DictTables.reset();
                            request.setAttribute("uploadTime", (int) ((System.currentTimeMillis() - startTime)/1000));
                            request.setAttribute("uploadedRows", ootl.getRowNumber());
                        }
                    }
                    item.delete();
                }
            } catch (FileUploadException ex) {
                ex.printStackTrace();
            }
        }
        responseCache.addIncludePage("/WEB-INF/fragments/dictAdminUpload.jsp");
        showPage(request, responseCache);
    }
    
    private void readAndDeleteWords(HttpServletRequest request, ResponseCache responseCache) {
        //TODO:
        responseCache.addIncludePage("/WEB-INF/fragments/dictAdminReadDelete.jsp");
    }
}
