/*
 * Forum.java
 *
 * Created on November 10, 2005, 2:44 PM
 */

package info.heinzelnisse.www;

import info.heinzelnisse.forum.Answer;
import info.heinzelnisse.forum.Question;
import info.heinzelnisse.forum.User;
import java.io.*;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.*;
import javax.servlet.http.*;

/**
 *
 * @author Heiko Klein
 * @version
 */
public class Forum extends BaseServlet {
    enum Task {ReadForum, WriteMessage, AddMessage, RemoveMessage};
    private Map<String, Task> tasks = new HashMap<String, Task>();
    private static DateFormat dateFormat = DateFormat.getDateInstance(DateFormat.SHORT, Locale.GERMANY);
    private static Logger logger = Logger.getLogger(Forum.class.getName());
    
    protected CardBean.Card getCard() {
        return CardBean.Card.Forum;
    }
    
    
    protected void processRequestBody(HttpServletRequest request, ResponseCache responseCache) {
        Task toDo = tasks.get(request.getParameter(".State"));
        if (toDo == null) toDo = Task.ReadForum;
        switch (toDo) {
            case ReadForum: readForum(request, responseCache); break;
            case WriteMessage: writeMessage(request, responseCache); break;
            case AddMessage: addMessage(request, responseCache); readForum(request, responseCache); break;
            case RemoveMessage: removeMessage(request, responseCache); readForum(request, responseCache); break;
            default: readForum(request, responseCache);
        }
    }
    
    public void init()
    throws ServletException {
        tasks.put("read", Task.ReadForum);
        tasks.put(">", Task.ReadForum);
        tasks.put("search", Task.ReadForum);
        tasks.put("Suche", Task.ReadForum);
        tasks.put("S�k", Task.ReadForum);
        tasks.put("new", Task.WriteMessage);
        tasks.put("Neue Frage", Task.WriteMessage);
        tasks.put("Nytt sp�rsm�l", Task.WriteMessage);
        tasks.put("Antwort", Task.WriteMessage);
        tasks.put("Svar", Task.WriteMessage);
        tasks.put("insert", Task.AddMessage);
        tasks.put("Abschicken", Task.AddMessage);
        tasks.put("Sende", Task.AddMessage);
        tasks.put("delete", Task.RemoveMessage);
    }
    
    private void readForum(HttpServletRequest request, ResponseCache responseCache) {
        ForumReadBean frb = new ForumReadBean();
        String searchWord = request.getParameter("searchword");
        if (searchWord != null) {
            searchWord = searchWord.trim();
            logger.fine("searchword:" + searchWord);
            List<Question> questions = Question.searchByText(searchWord);
            // show max 30
            if (questions.size() > 30) questions.subList(30, questions.size()).clear();
            // mark searchWord in red
            for (Question q : questions) {
                String text = q.getText();
                text = text.replaceAll("\\Q"+searchWord+"\\E", "<font color=\"red\">"+searchWord+"</font>");
                q.setText(text);
                for (Answer a : q.getAnswers()) {
                    text = a.getText();
                    text = text.replaceAll("\\Q"+searchWord+"\\E", "<font color=\"red\">"+searchWord+"</font>");
                    a.setText(text);
                }
            }
            frb.setQuestions(questions);
            frb.setDate(new Date());
        } else {
            String dateString = request.getParameter("date");
            Date date = new Date();
            if (dateString != null) {
                try {
                    date = dateFormat.parse(dateString);
                } catch (ParseException pe) {
                    pe.printStackTrace();
                }
            }
            frb.setDate(date);
            List<Question> questions = Question.ofDate(new java.sql.Date(date.getTime()), 7);
            // max 50 questions
            if (questions.size() > 50) questions.subList(50, questions.size()).clear();
            frb.setQuestions(questions);
        }
        request.setAttribute("forumInfo", frb);
        responseCache.addIncludePage("/WEB-INF/fragments/forum/readCurrent.jsp");
    }
    private void writeMessage(HttpServletRequest request, ResponseCache responseCache) {
        ForumWriteBean fwb = new ForumWriteBean();
        String qId = request.getParameter("ID");
        if (qId != null) {
            try {
                int id = Integer.parseInt(qId);
                fwb.setQuestion(Question.searchById(id));
            } catch (NumberFormatException nfe) {
                nfe.printStackTrace();
            }
        }
        request.setAttribute("forumWriter", fwb);
        responseCache.addIncludePage("/WEB-INF/fragments/forum/writeEntry.jsp");
    }
    
    private void addMessage(HttpServletRequest request, ResponseCache responseCache) {
        String text = request.getParameter("text");
        User user = getUserSettings(request).getUser();
        if (text != null) {
            text = text.trim();
            if (isValid(text)) {
                text = reformatText(text);
                String qId = request.getParameter("ID");
                if (qId != null) {
                    // insert new answer
                    try {
                        int id = Integer.parseInt(qId);
                        Question question = Question.searchById(id);
                        boolean answerExists = false;
                        for (Answer answer : question.getAnswers()) {
                            if (text.equals(answer.getText())) {
                                answerExists = true;
                                break;
                            }
                        }
                        if (!answerExists) {
                            Answer.create(text, request.getParameter("keywords"), new Date(), request.getRemoteAddr(), id, user.getId());
                        }
                        
                    } catch (NumberFormatException nfe) {
                        nfe.printStackTrace();
                    }
                } else {
                    // insert new question
                    Question q = Question.searchByExactText(text);
                    if (q == null) {
                        Question.create(text, request.getParameter("keywords"), new Date(), request.getRemoteAddr(), user.getId());
                    }
                }
            }
        }
    }
    
    private String reformatText(String text) {
        text = ServletUtils.escapeHtml(text);
        text = text.replaceAll("\\n","<br /> ");
        text = text.replaceAll("((http:|ftp:|mailto:)[\\w.\\-_~%?&:#!+=/;@]+)","<a href=\"$1\" target=\"_new\">$1</a>");
        text = text.replaceAll("([^/\\w:])(www\\.[\\w.\\-_~%?&:#!+=/;]+)","<a href=\"http://$2\" target=\"_new\">$1$2</a>");
        text = text.replaceAll("^(www\\.[\\w.\\-_~%?&:#!+=/;]+)","<a href=\"http://$1\" target=\"_new\">$1</a>");
        
        return text;
    }
    
    private boolean isValid(String text) {
        if (text.length() < 2) {
            return false;
        }
        if (text.length() > 20000) {
            return false;
        }
        if (text.matches("[^/\\s]{80}")) {
            // more than 80 words
            return false;
        }
        return true;
    }
    
    private void removeMessage(HttpServletRequest request, ResponseCache responseCache) {
        String flag = request.getParameter("delete");
        User user = getUserSettings(request).getUser();
        if (flag == null) {
            logger.warning("deletion without delete paramenter");
            return;
        }
        if (! user.isAdmin()) {
            logger.warning("deletion of "+flag+" by "+user.getDisplayName()+" ignored: no admin");
            return;
        }
        Pattern deleteFlagPattern = Pattern.compile("([aq])_(\\d+)");
        Matcher match = deleteFlagPattern.matcher(flag);
        if (match.matches()) {
            String qa = match.group(1);
            int id = (new Integer(match.group(2))).intValue();
            if (qa.equals("q")) {
                Question q = Question.searchById(id);
                if (q != null) q.removeFromDb();
                logger.info("removing question " + flag + " by "+user.getDisplayName());
            } else if (qa.equals("a")) {
                Answer.removeFromDb(id);
                logger.info("removing answer " + flag + " by "+user.getDisplayName());
            } else {
                logger.severe("pattern doesn't match code");
            }
        } else {
            logger.warning("unknown deleteFlag "+ flag + " send by "+user.getDisplayName());
        }
    }
    
    public static DateFormat getDateFormat() {
        return dateFormat;
    }
    
    /** Returns a short description of the servlet.
     */
    public String getServletInfo() {
        return "Forum of Heinzelnisse";
    }
}
