/*
 * LoginScreen.java
 *
 * Created on January 10, 2006, 8:14 PM
 */

package info.heinzelnisse.www;

import info.heinzelnisse.dictionary.Utils;
import info.heinzelnisse.forum.User;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.mail.MessagingException;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;

import javax.servlet.*;
import javax.servlet.http.*;

/**
 * * Created on January 10, 2006, 8:14 PM
 *
 * @author Heiko Klein
 * @version
 */
public class LoginScreen extends BaseServlet {
    enum Task {DoNothing, LoginUser, NewUser, ChangeUser, ChangePassword, ForgottenPassword};
    private Map<String, Task> tasks = new HashMap<String, Task>();
    static Logger logger = Logger.getLogger(LoginScreen.class.getName());
    
    /** Body of the page
     * @param request servlet request
     * @param response servlet response
     */
    @Override
    protected void processRequestBody(HttpServletRequest request, ResponseCache responseCache) {
        String returnUrl = request.getParameter("returnUrl");
        if (returnUrl != null) {
            LoginSettingsBean lsb = new LoginSettingsBean();
            lsb.setReturnUrl(returnUrl);
            request.getSession().setAttribute("loginSettings", lsb);
        }
        Task toDo = tasks.get(request.getParameter(".State"));
        if (toDo == null) toDo = Task.DoNothing;
        logger.fine("Task: "+ toDo);
        switch (toDo) {
            case LoginUser: loginUser(request, responseCache); break;
            case NewUser: ; newUser(request, responseCache); break;
            case ChangeUser: changeUser(request, responseCache); break;
            case ChangePassword: changePassword(request, responseCache); break;
            case ForgottenPassword: forgottenPassword(request, responseCache); break;
            default: ;
        }
        
        responseCache.addIncludePage("/WEB-INF/fragments/loginScreen.jsp");
    }
    
    private void loginUser(HttpServletRequest request, ResponseCache responseCache) {
        logger.fine("loginUser");
        User user = new User(ServletUtils.getParameter(request, "loginName"), ServletUtils.getParameter(request,"password"));
        if (user.isValid()) {
            logger.fine("usersettings: " + getUserSettings(request));
            logger.fine("user: "+ user.getId());
            getUserSettings(request).setUser(user);
            logger.fine("adding identity cookie");
            String cookieVal = user.getDisplayName() + "&" + ServletUtils.getParameter(request, "password");
            try {
                cookieVal = javax.mail.internet.MimeUtility.encodeText(cookieVal, "UTF-8", "B");
            } catch (java.io.UnsupportedEncodingException uee) {
                Logger.getLogger(LoginScreen.class.getName()).log(Level.WARNING, null, uee);
            }
            Cookie cookie = new Cookie("identity", cookieVal);
            cookie.setPath("/");
            cookie.setMaxAge(60*60*24*365);
            responseCache.addCookie(cookie);

            // no need to stay in log-in loop
            LoginSettingsBean lsb = (LoginSettingsBean) request.getSession().getAttribute("loginSettings");
            if (lsb != null) {
                if (lsb.getReturnUrl() != null) {
                    if (!"".equals(lsb.getReturnUrl())) {
                        responseCache.setRedirectPage(lsb.getReturnUrl());
                    }
                }
            }
        } else {
            request.setAttribute("wrongUser", true);
            logger.fine("wrong user: "+ServletUtils.getParameter(request, "loginName") + ServletUtils.getParameter(request,"password"));
        }
    }
    
    private void newUser(HttpServletRequest request, ResponseCache responseCache) {
        logger.finer("newUser");
        request.setAttribute("newUser", true);
        String displayName = ServletUtils.getParameter(request, "loginName");
        String email = ServletUtils.getParameter(request, "email");
        InternetAddress iAddress = null;
        try {
            iAddress = new InternetAddress(email);
            iAddress.validate();
        } catch (AddressException aex) {
            iAddress = null;
            logger.fine("couldn't parse email" + email);
        }
        int errors = 0;
        if ((iAddress == null) || (!email.contains("@"))) {
            request.setAttribute("noEmail", true);
            logger.fine("noEmail:"+email);
            errors++;
        }
        User user = new User(displayName);
        if (user.isValid()) {
            logger.fine("userExists, userid: "+ user.getId());
            request.setAttribute("userExists", true);
            errors++;
        }
        user = User.findByEmail(email);
        if (user.isValid()) {
            logger.fine("emailExists, userid: "+user.getId());
            request.setAttribute("emailExists", true);
            errors++;
        }
        if (errors > 0) {
            logger.fine("newUserErrors");
            request.setAttribute("newUserErrors", true);
        } else {
            
            Random generator = new Random();
            String password = ""+generator.nextInt(10000);
            logger.fine("adding user to database: userid: "+user.getId());
            user.setDisplayName(displayName);
            user.setEmail(email);
            user.setPassword(password);
            user.update();
            logger.fine("added userid: "+user.getId());
            if (! user.isValid()) {
                logger.warning("error adding new user");
                request.setAttribute("newUserErrors", true);
            } else {
                
                try {
                    ServletUtils.sendEmail("heinzelnisse@heinzelnisse.info","Heiko.Klein@gmx.net","Willkommen zur Heinzelnisse","Hallo "+displayName+" <"+iAddress.toString()+"> "+request.getRemoteAddr()+ ",\n\n willkommen bei der Heinzelnisse. Dein Passwort ist: " +password+"\n\nViele Grüße, \n\nHeinzelnisse\n");
                } catch (MessagingException mex) {
                    mex.printStackTrace();
                }
            }
        }
    }
    
    private void forgottenPassword(HttpServletRequest request, ResponseCache responseCache) {
        String displayName = ServletUtils.getParameter(request, "loginName");
        User user;
        if (displayName == null || displayName.length() == 0) {
            return;
        }
        if (displayName.contains("@")) {
            user = User.findByEmail(displayName);
        } else {
            user = new User(displayName);
        }
        if (user.getId() == 0) {
            request.setAttribute("unknownDisplayName", true);
        } else {
            Random generator = new Random();
            String password = ""+generator.nextInt(10000);
            logger.fine("changing password of userid: "+user.getId());
            user.setPassword(password);
            user.update();
            try {
                ServletUtils.sendEmail("heinzelnisse@heinzelnisse.info", user.getEmail(),"Willkommen zur Heinzelnisse", "Hallo "+displayName+",\n\n willkommen bei der Heinzelnisse. Dein neues Passwort ist: " +password+"\n\nViele Grüße, \n\nHeinzelnisse\n");
                request.setAttribute("sendPassword", true);
                request.setAttribute("sendEmail", Utils.obfuscateEmail(user.getEmail()));
            } catch (MessagingException mex) {
                mex.printStackTrace();
            }
        }
    }
    
    private void changePassword(HttpServletRequest request, ResponseCache responseCache) {
        logger.finer("changePassword");
        User user = getUserSettings(request).getUser();
        String oldPassword = ServletUtils.getParameter(request, "old_password");
        String newPassword1 = ServletUtils.getParameter(request, "new_password1");
        String newPassword2 = ServletUtils.getParameter(request, "new_password2");
        if (newPassword1.equals(newPassword2)) {
            User userX = new User(user.getDisplayName(), oldPassword);
            if (userX.getId() == user.getId()) {
                user.setPassword(newPassword1);
                user.update();
                return;
            } else {
                logger.info("wrong old password for changing");
            }
        } else {
            logger.fine("newPasswords:" + newPassword1 + " != " + newPassword2);
        }
        
        request.setAttribute("passwordChangeFailure", true);
    }
    
    @Override
    protected CardBean.Card getCard() {
        // no card
        return null;
    }
    
    @Override
    protected Boolean hasLogin() {
        // no login on login-page
        return false;
    }
    
    @Override
    public void init()
    throws ServletException {
        tasks.put("Login", Task.LoginUser);
        tasks.put("Passwort ändern", Task.ChangePassword);
        tasks.put("Forandre passord", Task.ChangePassword);
        tasks.put("Anmeldung", Task.NewUser);
        tasks.put("Påmelding", Task.NewUser);
        tasks.put("Einstellungen ändern", Task.ChangeUser);
        tasks.put("Forandre innstillinger", Task.ChangeUser);
        tasks.put("Passwort vergessen", Task.ForgottenPassword);
        tasks.put("Glemt passord", Task.ForgottenPassword);
    }
    
    /** Returns a short description of the servlet.
     */
    @Override
    public String getServletInfo() {
        // TODO:
        return "Heinzelnisse LoginScreen";
    }
    
    private void changeUser(HttpServletRequest request, ResponseCache responseCache) {
        logger.fine("changing user");
        User user = getUserSettings(request).getUser();
        if (user == null || (!user.isValid())) {
            loginUser(request, responseCache);
            return;
        }
        String email = ServletUtils.getParameter(request, "email");
        String info = ServletUtils.getParameter(request, "info");
        String prefLang = ServletUtils.getParameter(request, "prefLang");
        String motherTongue = ServletUtils.getParameter(request, "motherTongue");
        user.setFollowUp(info);
        user.setPreferredLanguage(prefLang);
        if (motherTongue.length() == 2) {
            user.setMotherTongue(motherTongue.toLowerCase());
        } else if ("".equals(motherTongue)) {
            user.setMotherTongue(null);
        }
        
        if (!email.toLowerCase().equals(user.getEmail())) {
            try {
                String subject;
                String message;
                if (user.getPreferredLanguage() == User.Lang.de) {
                    subject = "EMail Änderung bei der Heinzelnisse";
                    message = "Hallo "+user.getDisplayName()+"\n"+
                            "du hast deine EMail-Adresse von "+user.getEmail() + " zu " + email +" geändert. Unter beiden Addressen solltest du diese Nachricht erhalten haben. \n\n"+
                            "Viele Grüße,\n"+
                            "Heinzelnisse\n";
                } else {
                    subject = "Epost endring hos Heinzelnisse";
                    message = "Hei "+user.getDisplayName()+"\n"+
                            "du forandret epost addressen fra "+user.getEmail() + " til " + email +". Denne meldingen blir send til begge adressene.\n\n"+
                            "\n"+
                            "Heinzelnisse\n";
                }
                ServletUtils.sendEmail("heinzelnisse@heinzelnisse.info", user.getEmail(), subject, message);
                user.setEmail(email);
            } catch (MessagingException mex) {
                mex.printStackTrace();
            }
        }
        user.update();
    }
    
}
