/*
 * ResponseCache.java
 *
 * Created on January 6, 2007, 8:21 PM
 *
 */

package info.heinzelnisse.www;

import java.util.ArrayList;
import java.util.List;
import javax.servlet.http.Cookie;

/**
 * caches the response-object, such that flushes are not required
 * and redirects will allways work (no flush before redirect)
 *
 * @author Heiko Klein
 */
public class ResponseCache {
    
    private List<Cookie> cookies;
    private List<String> includePages;
    private String redirectPage;
    private int errCode = javax.servlet.http.HttpServletResponse.SC_OK;
    private String errMsg = "";
    
    /** Creates a new instance of ResponseCache */
    public ResponseCache() {
        cookies = new ArrayList<Cookie>();
        includePages = new ArrayList<String>();
        setRedirectPage(null);
    }

    public List<Cookie> getCookies() {
        return cookies;
    }

    public void addCookie(Cookie cookie) {
        if (cookie != null) cookies.add(cookie);
    }

    public List<String> getIncludePages() {
        return includePages;
    }

    public void addIncludePage(String str) {
        if (str != null) includePages.add(str);
    } 

    public String getRedirectPage() {
        return redirectPage;
    }

    /**
     * allows to set the redirectPage unless set earlier
     */
    public void setRedirectPage(String redirectPage) {
        if (this.redirectPage == null) {
            this.redirectPage = redirectPage;
        }
    }
    
    /**
     * allows to set error-codes of type javax.servlet.http.HttpServletResponse.SC_*
     */
     public void sendError(int errCode) {
         this.errCode = errCode;
     }
     public void sendError(int errCode, String errMsg) {
         this.errCode = errCode;
         this.errMsg = errMsg;
     }
     public int getErrorCode() {
         return errCode;
     }
     public String getErrorMessage() {
         return errMsg;
     }
}

