/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package info.heinzelnisse.www;

import info.heinzelnisse.dictionary.DictStatisticCache;
import info.heinzelnisse.dictionary.DictUtils;
import info.heinzelnisse.forum.Question;
import info.heinzelnisse.www.forum.SimpleQuestion;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.json.JSONException;
import org.json.JSONObject;

/**
 *
 * @author hklein
 */
public class SearchResults extends HttpServlet {
    private static final Logger logger = Logger.getLogger(SearchResults.class.getName());

    /** 
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        SearchSettingsBean ssb = Dict.getDefaultSearchSettingsBean();
        ssb.addRequestParameters(request);
        SearchBean sb = new SearchBean();
        sb.setSsb(ssb);
        if (sb.getSearchItem() == null) {
            response.sendError(HttpServletResponse.SC_BAD_REQUEST, "searchItem needed");
            return;
        }

        String type = request.getParameter("type");
        if ("json".equals(type)) {
            response.setContentType("application/json;charset=UTF-8");
            List<SimpleQuestion> questions = new ArrayList<SimpleQuestion>();
            for (Question q : sb.getForumQuestions()) {
                questions.add(new SimpleQuestion(q));
            }
            JSONObject json = new JSONObject();
            try {
                json.put("searchItem", sb.getSearchItem());
                json.put("deTrans", sb.getDeTranslations());
                json.put("noTrans", sb.getNoTranslations());
                json.put("forumQuestions", questions);
                json.put("wikiPageNames", sb.getWikiPageNames());
                json.put("bookmaalWords", sb.getBokmaalWords());
                json.put("nynorskWords", sb.getNynorskWords());
                json.put("dePhonetics", sb.getDePhonetics());
                json.put("noPhonetics", sb.getNoPhonetics());
                json.write(response.getWriter());
            } catch (JSONException ex) {
                logger.log(Level.SEVERE, null, ex);
            }
            // enter statistics
            if (ssb.getSearchItem() != null && (!ssb.getSearchItem().equals(""))) {
                logger.fine("adding statistics");
                DictStatisticCache dsc = DictStatisticCache.getInstance();
                dsc.incrementUsageCounter(DictUtils.DictType.dict);
                if ((sb.getDeTranslations().size() + sb.getNoTranslations().size()) > 0) {
                    dsc.addWordAccess(ssb.getSearchItem(), true);
                } else {
                    dsc.addWordAccess(ssb.getSearchItem(), false);
                }
            }
        }
    }

    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /** 
     * Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
     * Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>
}
