/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package info.heinzelnisse.www;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Logger;
import javax.servlet.http.HttpServletRequest;

/**
 *
 * @author hklein
 */
public class SearchSettingsBean {

    /**
     * @return the suggestion
     */
    public boolean isSuggestion() {
        return suggestion;
    }

    /**
     * @param suggestion the suggestion to set
     */
    public void setSuggestion(boolean suggestion) {
        this.suggestion = suggestion;
    }

    public enum SearchResultPage {

        Wiki("/WEB-INF/fragments/search/wikiResult.jsp"),
        Forum("/WEB-INF/fragments/search/forumResult.jsp"),
        Dict("/WEB-INF/fragments/search/dictResult.jsp");
        private String page;

        private SearchResultPage(String page) {
            this.page = page;
        }

        public String getPage() {
            return page;
        }
    }

    public boolean isWikiSearch() {
        return wikiSearch;
    }

    public void setWikiSearch(boolean wikiSearch) {
        this.wikiSearch = wikiSearch;
    }

    public boolean isForumSearch() {
        return forumSearch;
    }

    public void setForumSearch(boolean forumSearch) {
        this.forumSearch = forumSearch;
    }

    public boolean isForumKeywordSearch() {
        return forumKeywordSearch;
    }

    public void setForumKeywordSearch(boolean forumKeywordSearch) {
        this.forumKeywordSearch = forumKeywordSearch;
    }

    public boolean isDictExactSearch() {
        return dictExactSearch;
    }

    public void setDictExactSearch(boolean dictExactSearch) {
        this.dictExactSearch = dictExactSearch;
    }

    public boolean isDictPhoneticSearch() {
        return dictPhoneticSearch;
    }

    public void setDictPhoneticSearch(boolean dictPhoneticSearch) {
        this.dictPhoneticSearch = dictPhoneticSearch;
    }

    public boolean isDictBokmaalSearch() {
        return dictBokmaalSearch;
    }

    public void setDictBokmaalSearch(boolean dictBokmaalSearch) {
        this.dictBokmaalSearch = dictBokmaalSearch;
    }

    public boolean isDictNynorskSearch() {
        return dictNynorskSearch;
    }

    public void setDictNynorskSearch(boolean dictNynorskSearch) {
        this.dictNynorskSearch = dictNynorskSearch;
    }

    public List<SearchResultPage> getSearchResultOrder() {
        return searchResultOrder;
    }

    /**
     * enter the output-order of jsp pages for this search
     */
    public void setSearchResultOrder(SearchResultPage... pages) {
        this.searchResultOrder = Arrays.asList(pages);
    }

    /**
     * @return location of jsp-pages to process the search-results
     */
    public List<String> getSearchResultOrderedJSPs() {
        List<String> jsps = new ArrayList<String>(searchResultOrder.size());
        for (SearchResultPage srp : searchResultOrder) {
            if (srp == SearchResultPage.Dict && (isDictDeNoSearch() || isDictNoDeSearch())) {
                jsps.add(srp.getPage());
            } else if (srp == SearchResultPage.Wiki && (isWikiSearch())) {
                jsps.add(srp.getPage());
            } else if (srp == SearchResultPage.Forum && (isForumSearch() || isForumKeywordSearch())) {
                jsps.add(srp.getPage());
            }
        }
        return jsps;
    }

    public String getSearchItem() {
        return searchItem;
    }

    public void setSearchItem(String searchItem) {
        this.searchItem = searchItem;
    }

    public boolean isDictDeNoSearch() {
        return dictDeNoSearch;
    }

    public void setDictDeNoSearch(boolean dictDeNoSearch) {
        this.dictDeNoSearch = dictDeNoSearch;
    }

    public boolean isDictNoDeSearch() {
        return dictNoDeSearch;
    }

    public void setDictNoDeSearch(boolean dictNoDeSearch) {
        this.dictNoDeSearch = dictNoDeSearch;
    }

    public void addRequestParameters(HttpServletRequest request) {
        setSearchItem(ServletUtils.getParameter(request, "searchItem").trim());
        if (searchItem.length() > 80) {
            Logger.getLogger(SearchSettingsBean.class.getName()).warning("ignoring searchItem > 80 char: " + searchItem);
            searchItem = null;
            return;
        }
        if (!ServletUtils.getParameter(request, "setOptions").equals("true")) {
            return;
        }

        // retrieve all boolean flags via reflection (boolean var, setVar(boolean))
        for (Field f : SearchSettingsBean.class.getDeclaredFields()) {
            if (f.getType().equals(boolean.class)) {
                String name = f.getName();
                String methodName = "set" + name.substring(0, 1).toUpperCase() + name.substring(1);
                try {
                    Method m = SearchSettingsBean.class.getDeclaredMethod(methodName, boolean.class);
                    if (!"".equals(ServletUtils.getParameter(request, name))) {
                        m.invoke(this, true);
                    } else {
                        m.invoke(this, false);
                    }
                } catch (SecurityException ex) {
                    ex.printStackTrace();
                } catch (NoSuchMethodException ex) {
                    ex.printStackTrace();
                } catch (IllegalArgumentException ex) {
                    ex.printStackTrace();
                } catch (InvocationTargetException ex) {
                    ex.printStackTrace();
                } catch (IllegalAccessException ex) {
                    ex.printStackTrace();
                }
            }
        }
    }

    private String getStateInfo() {
        return "Stateinfo: " + "searchItem=" + getSearchItem() + "\n" +
                "\twikiSearch: " + wikiSearch + "\n" +
                "\tforumSearch: " + forumSearch + "\n" +
                "\tdictExactSearch: " + dictExactSearch + "\n" +
                "\tdictPhoneticSearch: " + dictPhoneticSearch + "\n" +
                "\tdictDeNoSearch: " + dictDeNoSearch + "\n" +
                "\tdictNoDeSearch: " + dictNoDeSearch + "\n" +
                "\tdictBokmaalSearch" + dictBokmaalSearch + "\n" +
                "\tdictNynorskSearch" + dictNynorskSearch + "\n" +
                "\tsuggestion" + suggestion + "\n";
    }
    
    @Override
    public int hashCode() {
        int hash = 5;
        hash = 67 * hash + (this.wikiSearch ? 1 : 0);
        hash = 67 * hash + (this.forumSearch ? 1 : 0);
        hash = 67 * hash + (this.forumKeywordSearch ? 1 : 0);
        hash = 67 * hash + (this.dictExactSearch ? 1 : 0);
        hash = 67 * hash + (this.dictPhoneticSearch ? 1 : 0);
        hash = 67 * hash + (this.dictBokmaalSearch ? 1 : 0);
        hash = 67 * hash + (this.dictNynorskSearch ? 1 : 0);
        hash = 67 * hash + (this.dictDeNoSearch ? 1 : 0);
        hash = 67 * hash + (this.dictNoDeSearch ? 1 : 0);
        hash = 67 * hash + (this.searchResultOrder != null ? this.searchResultOrder.hashCode() : 0);
        hash = 67 * hash + (this.suggestion ? 1 : 0);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final SearchSettingsBean other = (SearchSettingsBean) obj;
        if (this.wikiSearch != other.wikiSearch) {
            return false;
        }
        if (this.forumSearch != other.forumSearch) {
            return false;
        }
        if (this.forumKeywordSearch != other.forumKeywordSearch) {
            return false;
        }
        if (this.dictExactSearch != other.dictExactSearch) {
            return false;
        }
        if (this.dictPhoneticSearch != other.dictPhoneticSearch) {
            return false;
        }
        if (this.dictBokmaalSearch != other.dictBokmaalSearch) {
            return false;
        }
        if (this.dictNynorskSearch != other.dictNynorskSearch) {
            return false;
        }
        if (this.dictDeNoSearch != other.dictDeNoSearch) {
            return false;
        }
        if (this.dictNoDeSearch != other.dictNoDeSearch) {
            return false;
        }
        if (this.searchResultOrder != other.searchResultOrder && (this.searchResultOrder == null || !this.searchResultOrder.equals(other.searchResultOrder))) {
            return false;
        }
        if (this.suggestion != other.suggestion) {
            return false;
        }
        return true;
    }
    
    private String searchItem;
    private boolean wikiSearch;
    private boolean forumSearch;
    private boolean forumKeywordSearch;
    private boolean dictExactSearch;
    private boolean dictPhoneticSearch;
    private boolean dictBokmaalSearch;
    private boolean dictNynorskSearch;
    private boolean dictDeNoSearch;
    private boolean dictNoDeSearch;
    private boolean suggestion;
    private List<SearchResultPage> searchResultOrder = new ArrayList<SearchResultPage>();
}
