/*
 * SitemapGenerator.java
 *
 * Created on June 12, 2007, 8:25 PM
 */

package info.heinzelnisse.www;

import info.heinzelnisse.forum.Question;
import info.heinzelnisse.wiki.Page;
import java.io.*;
import java.net.*;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import javax.servlet.*;
import javax.servlet.http.*;

/**
 *
 * @author Heiko Klein
 * @version
 */
public class SitemapGenerator extends HttpServlet {
    
    /** Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
     * @param request servlet request
     * @param response servlet response
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        response.setContentType("text/xml;charset=UTF-8");
        PrintWriter out = response.getWriter();
        out.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        out.println("<urlset xmlns=\"http://www.sitemaps.org/schemas/sitemap/0.9\">");
        println(out, formatNoDeUrl(request, "", null, "monthly", 0.9f));
        println(out, formatNoDeUrl(request, "dict", null, "monthly", 0.8f));
        println(out, formatUrl("http://" + request.getServerName() + "/app", null, "monthly", 0.8f));
        println(out, formatNoDeUrl(request, "trainer", null, "monthly", 0.7f));
        for (Page page : Page.getChangedPages(new Date(0), new Date())) {
            float priority = 0.5f;
            if ("HeinzelWiki".equals(page.getName())) {
                priority = 0.7f;
            }
            println(out, formatNoDeUrl(request, "wiki/"+page.getName(), page.getTimestamp(), "monthly", priority));                
        }
        println(out, formatNoDeUrl(request, "forum", null, "hourly", 0.7f));
        println(out, formatNoDeUrl(request, "forum/archive", null, "monthly", 0.7f));
        List<Date> monthList = new ArrayList<Date>(Question.getArchivedMonths().keySet());
        for (Date date : monthList) {
            Calendar cal = GregorianCalendar.getInstance();
            cal.setTime(date);
            int year = cal.get(Calendar.YEAR);
            int month = cal.get(Calendar.MONTH) + 1;
            println(out,formatNoDeUrl(request, "forum/archive/"+year+"-"+month, date, "monthly", 0.5f));
        }
        Calendar cal = GregorianCalendar.getInstance();
        Date now = cal.getTime();
        cal.add(Calendar.WEEK_OF_YEAR, -1);
        Date lastWeek = cal.getTime();
        String topAddr = "http://" + request.getServerName() + "/";
        List<Integer> forumIds = Question.idOfDate(new Date(0), lastWeek);
        for (int id : forumIds) {
            println(out, formatUrl(topAddr+"forum/permalink/"+id, null, "yearly", 0.3f));
        }
        forumIds = Question.idOfDate(lastWeek, now);
        for (int id : forumIds) {
            println(out, formatUrl(topAddr+"forum/permalink/"+id, null, "daily", 0.3f));
        }
        println(out, formatNoDeUrl(request, "download", null, "monthly", 0.7f));
        println(out, formatNoDeUrl(request, "help", null, "monthly", 0.7f));
        out.println("</urlset>");
        out.close();
    }
    
    private void println(PrintWriter out, List<String> strs) {
        for(String line : strs) {
            out.println(line);
        }
    }
    
    private List<String> formatNoDeUrl(HttpServletRequest request, String contextPath, Date lastmod, String changefreq, float priority) {
        List<String> strs;
        String topAddr = "http://" + request.getServerName() + "/";
        strs = formatUrl(topAddr + contextPath, lastmod, changefreq, priority);
        strs.addAll(formatUrl(topAddr + "no/" + contextPath, lastmod, changefreq, priority));
        return strs;
    }
    
    private List<String> formatUrl(String loc, Date lastmod, String changefreq, float priority) {
        List<String> strs = new ArrayList<String>(10);
        strs.add("<url>");
        strs.add("<loc>"+loc+"</loc>");
        if (lastmod != null) {
            Calendar cal = GregorianCalendar.getInstance();
            cal.setTime(lastmod);
            String month = "" + (cal.get(Calendar.MONTH)+1);
            month = month.length() < 2 ? "0" + month : month;
            String day = "" + cal.get(Calendar.DAY_OF_MONTH);
            day = day.length() < 2 ? "0" + day : day;
            strs.add("<lastmod>"+
                    cal.get(Calendar.YEAR)+"-"+
                    month+"-"+
                    day +
                    "</lastmod>");
        }
        if (changefreq != null) {
            strs.add("<changefreq>"+changefreq+"</changefreq>");
        }
        if ((priority > 0) && (priority <= 1.0)) {
            strs.add("<priority>"+priority+"</priority>");
        }
        strs.add("</url>");
        return strs;
    }
    
    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /** Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     */
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        processRequest(request, response);
    }
    
    /** Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        processRequest(request, response);
    }
    
    /** Returns a short description of the servlet.
     */
    public String getServletInfo() {
        return "Short description";
    }
    // </editor-fold>
}
