/*
 * Suggestions: on i5 2410: ~ 2600/sec with mysql
 *                            2700/sec with mem
 */

package info.heinzelnisse.www;

import info.heinzelnisse.dictionary.CanooBokmaalLinkDictUtils;
import info.heinzelnisse.dictionary.DictStatisticCache;
import info.heinzelnisse.dictionary.DictUtils;
import info.heinzelnisse.dictionary.Suggestion;
import info.heinzelnisse.he.HeinzelTable;
import java.io.IOException;
import java.text.Collator;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.json.JSONArray;
import org.json.JSONException;

/**
 *
 * @author hklein
 */
public class Suggest extends HttpServlet {
   
    private static final Logger logger = Logger.getLogger(Suggest.class.getName());
    /** 
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        String type = request.getParameter("type");
        String searchItem = request.getParameter("searchItem");
        if (searchItem == null) {
            response.sendError(HttpServletResponse.SC_BAD_REQUEST, "searchItem needed");
            return;
        }
        searchItem = searchItem.trim();
        logger.log(Level.FINE, "suggestion of {0}", searchItem);
        // serverpath
        String path = "http://" + request.getServerName();
        if (request.getServerPort() != 80) {
            path += ":" + request.getServerPort();
        }
        path += request.getContextPath();



        DictUtils du = CanooBokmaalLinkDictUtils.getInstance();
        Map<String, Suggestion> suggestions = du.dbSearchSuggestions(searchItem);

        if ("json".equals(type)) {
            response.setContentType("application/x-suggestions+json;charset=UTF-8");
            Collator deNoCol = HeinzelTable.getDeNoCollator();
            SortedSet<String> directCompletions = new TreeSet<String>(deNoCol);
            SortedSet<String> indirectCompletions = new TreeSet<String>(deNoCol);
            for (String compl : suggestions.keySet()) {
                Suggestion sugs = suggestions.get(compl);
                if (sugs.word.toLowerCase().startsWith(searchItem.toLowerCase())) {
                    directCompletions.add(sugs.word);
                } else {
                    indirectCompletions.add(sugs.word);
                }
            }
            List<String> completions = new ArrayList<String>(directCompletions);
            completions.addAll(indirectCompletions);
            JSONArray answer = new JSONArray();
            answer.put(searchItem);
            answer.put(completions);
            answer.put(new ArrayList<String>()); // empty item
            answer.put(new ArrayList<String>()); // empty item
            try {
                answer.write(response.getWriter());
            } catch (JSONException ex) {
                Logger.getLogger(Suggest.class.getName()).log(Level.SEVERE, null, ex);
            }
            logger.fine("adding statistics");
            DictStatisticCache.getInstance().incrementUsageCounter(DictUtils.DictType.suggest);
        } else if ("html".equals(type)) {
            response.setContentType("text/html;charset=UTF-8");
            StringBuilder sb = new StringBuilder(128 * suggestions.size());
            for (String compl : suggestions.keySet()) {
                Suggestion sugs = suggestions.get(compl);
                sb.append("<a class=\"suggestItem\" href=\"");
                sb.append(path);
                sb.append("/dict?searchItem=");
                sb.append(sugs.word);
                sb.append("\">");
                sb.append(compl);
                sb.append("</a>");
            }
            response.getWriter().print(sb.toString());
            logger.fine("adding statistics");
            DictStatisticCache.getInstance().incrementUsageCounter(DictUtils.DictType.suggest);
        } else {
            response.sendError(HttpServletResponse.SC_BAD_REQUEST, "type parameter needed");
        }


    }

    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /** 
     * Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        processRequest(request, response);
    } 

    /** 
     * Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>

}
