/*
 * Trainer.java
 *
 * Created on August 15, 2006, 2:49 PM
 */

package info.heinzelnisse.www;

import info.heinzelnisse.dictionary.DictStatisticCache;
import info.heinzelnisse.dictionary.DictUtils;
import java.util.logging.Logger;

import javax.servlet.*;
import javax.servlet.http.*;

/**
 * * Created on August 15, 2006, 2:49 PM
 *
 * @author Heiko Klein
 * @version
 */
public class Trainer extends BaseServlet {
    static Logger logger = Logger.getLogger(Trainer.class.getName());
    
    /** Body of the page
     * @param request servlet request
     * @param response servlet response
     */
    protected void processRequestBody(HttpServletRequest request, ResponseCache responseCache) {
        logger.fine("entering Trainer");

        SettingsBean sb = new SettingsBean();
        request.setAttribute("trainer", sb);
        String lastWord = ServletUtils.getParameter(request,"lastWord");
        sb.setLastWord(lastWord);
        sb.setLastTranslation(ServletUtils.getParameter(request, "input"));
        sb.setLanglang(ServletUtils.getParameter(request,"langlang"));
        String currentWord = DictUtils.getRandomWord(sb.getLanglang());
        sb.setCurrentWord(currentWord);

        SearchSettingsBean searchBean = new SearchSettingsBean();
        searchBean.setSearchItem(lastWord);
        searchBean.setDictExactSearch(true);
        searchBean.setDictPhoneticSearch(false);
        searchBean.setForumSearch(false);
        searchBean.setWikiSearch(false);
        searchBean.setSearchResultOrder(SearchSettingsBean.SearchResultPage.Dict);
        if (sb.getLanglang().equals("de")) {
            searchBean.setDictDeNoSearch(true);
            searchBean.setDictNoDeSearch(false);
        } else {
            searchBean.setDictDeNoSearch(false);
            searchBean.setDictNoDeSearch(true);
        }
        SearchBean searchData = new SearchBean();
        searchData.setSsb(searchBean);
        request.setAttribute("searchSettings", searchData);

        if (!lastWord.equals("")) {
            DictStatisticCache.getInstance().incrementUsageCounter(DictUtils.DictType.trainer);
        }
        
        responseCache.addIncludePage("/WEB-INF/fragments/trainer.jsp");
    }
    
    @Override
    protected CardBean.Card getCard() {
        return CardBean.Card.Trainer;
    }
    
        @Override
    protected String getTitle(HttpServletRequest request) {
        UserSettingsBean sb = getUserSettings(request);
        if ("de".equals(sb.getLang()))
            return "Heinzelnisse - Vokabelspiele";
        else
            return "Heinzelnisse - Gloseleker";
    }
    
    /** Returns a short description of the servlet.
     */
    @Override
    public String getServletInfo() {
        return "Heinzelnisse Trainer";
    }
    
    static public class SettingsBean {
        private String currentWord = "";
        private String lastWord = "";
        private String lastTranslation = "";
        private String langlang = "de";

        public String getCurrentWord() {
            return currentWord;
        }

        public void setCurrentWord(String currentWord) {
            this.currentWord = currentWord;
        }

        public String getLastWord() {
            return lastWord;
        }

        public void setLastWord(String lastWord) {
            this.lastWord = lastWord;
        }

        public String getLanglang() {
            return langlang;
        }

        public void setLanglang(String langlang) {
            if (langlang != null && (langlang.equals("de") || langlang.equals("no"))) {
                this.langlang = langlang;
            }
        }

        public String getLastTranslation() {
            return lastTranslation;
        }

        public void setLastTranslation(String lastTranslation) {
            this.lastTranslation = lastTranslation;
        }
    }
}
