/*
 * ForumArchive.java
 *
 * Created on January 7, 2007, 1:47 PM
 */

package info.heinzelnisse.www.forum;

import info.heinzelnisse.www.*;
import info.heinzelnisse.forum.Question;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import javax.servlet.*;
import javax.servlet.http.*;

/**
 * * Created on January 7, 2007, 1:47 PM
 *
 * @author Heiko Klein
 * @version
 */
public class Archive extends BaseServlet {
    static Logger logger = Logger.getLogger(Archive.class.getName());
    static Pattern overviewPattern = Pattern.compile(".*/archive");
    static Pattern datePattern = Pattern.compile(".*/archive/(\\d{4})-(\\d{1,2})");
    
    /** Body of the page
     * @param request servlet request
     * @param response servlet response
     */
    @Override
    protected void processRequestBody(HttpServletRequest request, ResponseCache responseCache) {
        SearchSettingsBean ssb = (SearchSettingsBean) request.getSession().getAttribute("forumSearchSettings");
        if (ssb == null) {
            logger.fine("initialize new SearchSettingsBean");
            ssb = new SearchSettingsBean();
            // set defaults for dict
            ssb.setDictDeNoSearch(true);
            ssb.setDictNoDeSearch(true);
            ssb.setDictExactSearch(true);
            ssb.setDictPhoneticSearch(true);
            ssb.setForumSearch(true);
            ssb.setWikiSearch(true);
            ssb.setSuggestion(true);
            ssb.setSearchResultOrder(SearchSettingsBean.SearchResultPage.Dict, SearchSettingsBean.SearchResultPage.Wiki, SearchSettingsBean.SearchResultPage.Forum);
            request.getSession().setAttribute("forumSearchSettings", ssb);
        }
        ssb.addRequestParameters(request);
        SearchBean sb = new SearchBean();
        sb.setSsb(ssb);
        request.setAttribute("searchSettings", sb);

        responseCache.addIncludePage("/WEB-INF/fragments/search/field.jsp");
        responseCache.addIncludePage("/WEB-INF/fragments/search/results.jsp");

        
        if (overviewPattern.matcher(request.getRequestURI()).matches()) {
            // show the overview
            Map<Date, int[]> monthQuestions = Question.getArchivedMonths();
            List<Date> reverseMonthList = new ArrayList<Date>(monthQuestions.keySet());
            Collections.sort(reverseMonthList, Collections.reverseOrder());
            request.setAttribute("reverseMonthList", reverseMonthList);
            request.setAttribute("monthlyQuestions",  monthQuestions);
            responseCache.addIncludePage("/WEB-INF/fragments/forum/archiveOverview.jsp");
        } else {
            Matcher matcher = datePattern.matcher(request.getRequestURI());
            if (matcher.matches()) {
                int year = Integer.parseInt(matcher.group(1));
                int month = Integer.parseInt(matcher.group(2));
                logger.fine("looking for: " + month + "." + year);
                ReadBean frb = new ReadBean();
                request.setAttribute("forumInfo", frb);
                List<Question> questions = Question.ofMonth(month, year);
                Forum.reformatQuestions(questions);
                frb.setQuestions(questions);
                if (month < 12) {
                    request.setAttribute("forumArchiveNext", year+"-"+(month+1));
                } else {
                    request.setAttribute("forumArchiveNext", (year+1)+"-01");
                }
                if (month > 1) {
                    request.setAttribute("forumArchivePrevious", year+"-"+(month-1));
                } else {
                    request.setAttribute("forumArchivePrevious", (year-1)+"-"+12);
                }
                Calendar cal = GregorianCalendar.getInstance();
                cal.set(year, month-1, 1);
                Date date = cal.getTime();
                request.setAttribute("forumArchiveCurrentDate", date);
                responseCache.addIncludePage("/WEB-INF/fragments/forum/archivePreviousNext.jsp");
                responseCache.addIncludePage("/WEB-INF/fragments/forum/result.jsp");
                responseCache.addIncludePage("/WEB-INF/fragments/forum/archivePreviousNext.jsp");
            } else {
                logger.warning("illegal request: " + request.getContextPath());
            }
        }
    }
    @Override
    protected CardBean.Card getCard() {
        return CardBean.Card.Forum;
    }
    
    /** Returns a short description of the servlet.
     */
    @Override
    public String getServletInfo() {
        return "Heinzelnisse ForumArchive";
    }
}
