/*
 * Forum.java
 *
 * Created on November 10, 2005, 2:44 PM
 */

package info.heinzelnisse.www.forum;

import info.heinzelnisse.www.*;
import info.heinzelnisse.forum.Answer;
import info.heinzelnisse.forum.Question;
import info.heinzelnisse.forum.User;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.*;
import javax.servlet.http.*;

/**
 *
 * @author Heiko Klein
 * @version
 */
public class Forum extends BaseServlet {
    enum Task {ReadForum, WriteMessage, AddMessage, RemoveMessage, PreviewMessage};
    private Map<String, Task> tasks = new HashMap<String, Task>();
    private static DateFormat dateFormat = DateFormat.getDateInstance(DateFormat.SHORT, Locale.GERMANY);
    private static Logger logger = Logger.getLogger(Forum.class.getName());
    
    @Override
    protected CardBean.Card getCard() {
        return CardBean.Card.Forum;
    }
    
    @Override
    protected String getTitle(HttpServletRequest request) {
        UserSettingsBean sb = getUserSettings(request);
        if ("de".equals(sb.getLang()))
            return "Heinzelnisse - Forum Norwegisch Deutsch";
        else
            return "Heinzelnisse - Forum Norsk Tysk";
    }
    
    @Override
    protected void processRequestBody(HttpServletRequest request, ResponseCache responseCache) {
        String state = request.getParameter(".State");
        Task toDo = tasks.get(state);
        if (toDo == null) {
            if (state != null && (!state.equals(""))) {
                logger.warning("unknown state: " + state);
            }
            toDo = Task.ReadForum;
        }
        switch (toDo) {
            case ReadForum: readForum(request, responseCache); break;
            case WriteMessage: writeMessage(request, responseCache); break;
            case PreviewMessage: writeMessage(request, responseCache); break;
            case AddMessage: addMessage(request, responseCache); readForum(request, responseCache); break;
            case RemoveMessage: removeMessage(request, responseCache); readForum(request, responseCache); break;
            default: readForum(request, responseCache);
        }
    }
    
    @Override
    public void init()
    throws ServletException {
        tasks.put("read", Task.ReadForum);
        tasks.put(">", Task.ReadForum);
        tasks.put("search", Task.ReadForum);
        tasks.put("Suche", Task.ReadForum);
        tasks.put("Søk", Task.ReadForum);
        tasks.put("new", Task.WriteMessage);
        tasks.put("Neue Frage", Task.WriteMessage);
        tasks.put("Nytt spørsmål", Task.WriteMessage);
        tasks.put("Antwort", Task.WriteMessage);
        tasks.put("Svar", Task.WriteMessage);
        tasks.put("insert", Task.AddMessage);
        tasks.put("Abschicken", Task.AddMessage);
        tasks.put("Sende", Task.AddMessage);
        tasks.put("delete", Task.RemoveMessage);
        tasks.put("Voransicht", Task.PreviewMessage);
        tasks.put("Preview", Task.PreviewMessage);
    }
    
    private void readForum(HttpServletRequest request, ResponseCache responseCache) {
        ReadBean frb = new ReadBean();
        frb.setStatus("current");
        frb.setDate(new Date());
        String searchWord = request.getParameter("searchword");
        String byUser = request.getParameter("byuser");
        if (searchWord != null || byUser != null) {
            List<Question> questions = null;
            if (searchWord != null) {
                searchWord = searchWord.trim();
                logger.fine("searchword:" + searchWord);
                questions = Question.searchByText(searchWord);
            } else {
                logger.fine("searchByUser:" + byUser);
                int userId = Integer.parseInt(byUser);
                searchWord = "set to some nonesense";
                questions = Question.searchByForumUser(userId);
            }
            // show max 30
            if (questions.size() > 30) questions.subList(30, questions.size()).clear();
            // mark searchWord in red
            for (Question q : questions) {
                String text = reformatText(q.getOriginalText());
                text = text.replaceAll("\\Q"+searchWord+"\\E", "<font color=\"red\">"+searchWord+"</font>");
                q.setText(text);
                for (Answer a : q.getAnswers()) {
                    text = reformatText(a.getOriginalText());
                    text = text.replaceAll("\\Q"+searchWord+"\\E", "<font color=\"red\">"+searchWord+"</font>");
                    a.setText(text);
                }
            }
            frb.setQuestions(questions);
        } else {
            List<Question> questions = Question.ofDate(new java.sql.Date(frb.getDate().getTime()), 7, 10);
            // max 150 questions, or reader crash
            if (questions.size() > 150) questions.subList(150, questions.size()).clear();
            reformatQuestions(questions);
            frb.setQuestions(questions);
        }
        request.setAttribute("forumInfo", frb);
        responseCache.addIncludePage("/WEB-INF/fragments/forum/readCurrent.jsp");
    }
    private void writeMessage(HttpServletRequest request, ResponseCache responseCache) {
        User user = getUserSettings(request).getUser();
        WriteBean fwb = new WriteBean();
        ReadBean frb = new ReadBean();
        String qId = request.getParameter("ID");
        int id = 0;
        Question q = null;
        if (qId != null) {
            try {
                id = Integer.parseInt(qId);
                q = Question.searchById(id);
                fwb.setQuestion(reformatQuestion(q));
            } catch (NumberFormatException nfe) {
                nfe.printStackTrace();
            }
        }
        // information for preview from earlier request
        String originalText = request.getParameter("text");
        if (originalText != null && (!"".equals(originalText))) {
            fwb.setText(originalText);
            fwb.setKeywords(request.getParameter("keywords"));
            if (q != null) {
                q.getAnswers().add(new Answer(0, originalText, originalText, new Date(), request.getRemoteAddr(), id, user.getId()));
            } else {
                q = new Question(0, originalText, originalText, new Date(), request.getRemoteAddr(), user.getId());
            }
            q.getKeywords().addAll(Arrays.asList(fwb.getKeywords().split(",")));
        }
        if (q != null) {
            ArrayList<Question> questions = new ArrayList();
            questions.add(reformatQuestion(q));
            frb.setQuestions(questions);
        }
        request.setAttribute("forumInfo", frb);
        request.setAttribute("forumWriter", fwb);
        responseCache.addIncludePage("/WEB-INF/fragments/forum/writeEntry.jsp");
    }
    
    private void addMessage(HttpServletRequest request, ResponseCache responseCache) {
        String originalText = request.getParameter("text");
        User user = getUserSettings(request).getUser();
        
        if (originalText != null) {
            originalText = originalText.trim();
            if (isValid(originalText)) {
                String text = reformatText(originalText);
                String qId = request.getParameter("ID");
                if (qId != null) {
                    // insert new answer
                    try {
                        int id = Integer.parseInt(qId);
                        Question question = Question.searchById(id);
                        boolean answerExists = false;
                        for (Answer answer : question.getAnswers()) {
                            if (originalText.equals(answer.getOriginalText())) {
                                answerExists = true;
                                break;
                            }
                        }
                        if (!answerExists) {
                            Answer.create(text, originalText, request.getParameter("keywords"), new Date(), request.getRemoteAddr(), id, user.getId());
                        }
                        
                    } catch (NumberFormatException nfe) {
                        nfe.printStackTrace();
                    }
                } else {
                    // insert new question
                    Question q = Question.searchByExactText(originalText);
                    if (q == null) {
                        Question.create(text, originalText, request.getParameter("keywords"), new Date(), request.getRemoteAddr(), user.getId());
                    }
                }
            }
        }
        
        // allow admins to change all keywords even without message
        // (make sure admins get allways all keywords as editable items in writeEntry.jsp)
        if (user.isAdmin()) {
            String qId = request.getParameter("ID");
            if (qId != null) {
                try {
                    int id = Integer.parseInt(qId);
                    String keywords = request.getParameter("keywords");
                    Question.replaceKeywords(id, keywords);
                } catch (NumberFormatException nfe) {
                    nfe.printStackTrace();
                }
            }
        }
        
    }
    
    public static void reformatQuestions(List<Question> questions) {
        for (Question q : questions) {
            reformatQuestion(q);
        }
    }
    
    public static Question reformatQuestion(Question question) {
        question.setText(reformatText(question.getOriginalText()));
        for (Answer a : question.getAnswers()) {
            a.setText(reformatText(a.getOriginalText()));
        }
        return question;
    }

    private static String reformatText(String text) {
        return ForumFormatter.getInstance().format(text);
    }
    
    private boolean isValid(String text) {
        if (text.length() < 2) {
            return false;
        }
        if (text.length() > 20000) {
            return false;
        }
        if (text.matches("[^/\\s]{80}")) {
            // more than 80 words
            return false;
        }
        return true;
    }
    
    private void removeMessage(HttpServletRequest request, ResponseCache responseCache) {
        String flag = request.getParameter("delete");
        User user = getUserSettings(request).getUser();
        if (flag == null) {
            logger.warning("deletion without delete paramenter");
            return;
        }
        if (! user.isAdmin()) {
            logger.warning("deletion of "+flag+" by "+user.getDisplayName()+" ignored: no admin");
            return;
        }
        Pattern deleteFlagPattern = Pattern.compile("([aq])_(\\d+)");
        Matcher match = deleteFlagPattern.matcher(flag);
        if (match.matches()) {
            String qa = match.group(1);
            int id = (new Integer(match.group(2))).intValue();
            if (qa.equals("q")) {
                Question q = Question.searchById(id);
                if (q != null) q.removeFromDb();
                logger.info("removing question " + flag + " by "+user.getDisplayName());
            } else if (qa.equals("a")) {
                Answer.removeFromDb(id);
                logger.info("removing answer " + flag + " by "+user.getDisplayName());
            } else {
                logger.severe("pattern doesn't match code");
            }
        } else {
            logger.warning("unknown deleteFlag "+ flag + " send by "+user.getDisplayName());
        }
    }
    
    public static DateFormat getDateFormat() {
        return dateFormat;
    }
    
    /** Returns a short description of the servlet.
     */
    @Override
    public String getServletInfo() {
        return "Forum of Heinzelnisse";
    }
}
