/*
 * Statistic.java
 *
 * Created on September 18, 2006, 8:37 PM
 */

package info.heinzelnisse.www.statistics;

import info.heinzelnisse.dictionary.CanooBokmaalLinkDictUtils;
import info.heinzelnisse.dictionary.DictUtils;
import info.heinzelnisse.dictionary.Utils;
import info.heinzelnisse.www.*;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.logging.Logger;

import javax.servlet.http.*;

/**
 * * Created on September 18, 2006, 8:37 PM
 *
 * @author Heiko Klein
 * @version
 */
public class Statistic extends BaseServlet {
    static final Logger logger = Logger.getLogger(Statistic.class.getName());
    
    /** Body of the page
     * @param request servlet request
     * @param response servlet response
     */
    @Override
    protected void processRequestBody(HttpServletRequest request, ResponseCache responseCache) {
        if (getUserSettings(request).getUser().isAdmin()) {
            StatisticBean statB = new StatisticBean();
            request.setAttribute("statistics", statB);
            CanooBokmaalLinkDictUtils du = CanooBokmaalLinkDictUtils.getInstance();
            statB.setItems(du.countItems());
            statB.setTranslations(du.countTranslations());
            statB.setRatio(du.goodTotalRatio());
            
            Calendar cal = Calendar.getInstance(Utils.langLocale(DictUtils.DictLang.DE));
            cal.set(Calendar.HOUR_OF_DAY, 0);
            cal.set(Calendar.MINUTE, 0);
            cal.set(Calendar.SECOND, 0);
            cal.set(Calendar.MILLISECOND, 0); // this will calculate everything up to yesterday
            cal.getTime(); // calculate
            
            Date startDate = cal.getTime();
            
            {
                // weeks
                List<StatFields> weekList = new ArrayList<StatFields>();
                statB.setWeekStats(weekList);
                int maxWeekCount = 0;
                Date endWeek = startDate;
                cal.setTime(startDate);
                cal.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
                
                for (int i = 0; i < 54; i++) {
                    StatFields sf = new StatFields();
                    Date startWeek = cal.getTime();
                    sf.setId(cal.get(Calendar.WEEK_OF_YEAR));
                    int[] counts = CanooBokmaalLinkDictUtils.getInstance().countAccessBetween(startWeek, endWeek);
                    int total = counts[0] + counts[1];
                    sf.setTotal(total);
                    if (total > maxWeekCount) {
                        maxWeekCount = total;
                    }
                    // missusing ratios for temporary storage, remember to devide by total!
                    sf.setDictOfMax(counts[0]);
                    sf.setTrainerOfMax(counts[1]);
                    weekList.add(sf);
                    endWeek = startWeek;
                    cal.add(Calendar.WEEK_OF_YEAR, -1);
                }
                for (StatFields sf : weekList) {
                    sf.setDictOfMax(100 * sf.getDictOfMax() / maxWeekCount);
                    sf.setTrainerOfMax(100 * sf.getTrainerOfMax() / maxWeekCount);
                }
            }
            
            
            
            {
                // month
                List<StatFields> monthList = new ArrayList<StatFields>();
                statB.setMonthStats(monthList);
                int maxMonthCount = 0;
                cal.setTime(startDate);
                Date endMonth = startDate;
                cal.set(Calendar.DAY_OF_MONTH, 1);
                
                for (int i = 0; i < 14; i++) {
                    StatFields sf = new StatFields();
                    Date startMonth = cal.getTime();
                    sf.setId(cal.get(Calendar.MONTH) + 1);
                    int[] counts = CanooBokmaalLinkDictUtils.getInstance().countAccessBetween(startMonth, endMonth);
                    int total = counts[0] + counts[1];
                    sf.setTotal(total);
                    if (total > maxMonthCount) {
                        maxMonthCount = total;
                    }
                    // missusing ratios for temporary storage, remember to devide by total!
                    sf.setDictOfMax(counts[0]);
                    sf.setTrainerOfMax(counts[1]);
                    monthList.add(sf);
                    endMonth = startMonth;
                    cal.add(Calendar.MONTH, -1);
                }
                for (StatFields sf : monthList) {
                    sf.setDictOfMax(100 * sf.getDictOfMax() / maxMonthCount);
                    sf.setTrainerOfMax(100 * sf.getTrainerOfMax() / maxMonthCount);
                }
            }
            
            {
                // year
                List<StatFields> yearList = new ArrayList<StatFields>();
                statB.setYearStats(yearList);
                int maxYearCount = 0;
                cal.setTime(startDate);
                Date endYear = startDate;
                cal.set(Calendar.DAY_OF_YEAR, 1);
                
                while (cal.get(Calendar.YEAR) >= 2002) {
                    StatFields sf = new StatFields();
                    Date startYear = cal.getTime();
                    sf.setId(cal.get(Calendar.YEAR));
                    int[] counts = CanooBokmaalLinkDictUtils.getInstance().countAccessBetween(startYear, endYear);
                    int total = counts[0] + counts[1];
                    sf.setTotal(total);
                    if (total > maxYearCount) {
                        maxYearCount = total;
                    }
                    // missusing ratios for temporary storage, remember to devide by total!
                    sf.setDictOfMax(counts[0]);
                    sf.setTrainerOfMax(counts[1]);
                    yearList.add(sf);
                    endYear = startYear;
                    cal.add(Calendar.YEAR, -1);
                }
                for (StatFields sf : yearList) {
                    sf.setDictOfMax(100 * sf.getDictOfMax() / maxYearCount);
                    sf.setTrainerOfMax(100 * sf.getTrainerOfMax() / maxYearCount);
                }
            }
        }
        
        responseCache.addIncludePage("/WEB-INF/fragments/statistics.jsp");
    }
    
    @Override
    protected CardBean.Card getCard() {
        return null;
    }
    
    /** Returns a short description of the servlet.
     */
    @Override
    public String getServletInfo() {
        return "Heinzelnisse Statistic";
    }
}
