/* 
 * heinzelsearch.js
 *
 * search class for heinzelnisse offline app
 *
 * Copyright: (C) 2011 Heiko Klein
 * License: GPL
 *
 */

/**
 * localStorage elements: hnLastData hnLastSearchItem hnDB hasHnDB hasSmallLocalStorage
 */

var HeinzelSearch = {};
HeinzelSearch.Config = {
    lastSearchItem: "",
    hndb: "",
    exact: false,
    online: false,
    searchDfd: new jQuery.Deferred(),
    delayTimer: 0
};

HeinzelSearch.regQuote = function(str) {
    return (str+'').replace(/[.?*+^$[\]\\(){}|-]/g, "\\$&");
};

HeinzelSearch.getURLParameter = function(name) {
  var reg = new RegExp(name + '=' + '(.+?)(&|$)');
  var result = reg.exec(location.href); //in non ajax-world: location.search
  console.log(result);
  if (result && result[1]) {
    return unescape(result[1]);
  } else {
    return null;
  }
}

HeinzelSearch.htmlFormatWord = function(word, article, other) {
  word = "<a class=\"search\" onclick=\"HeinzelSearch.searchEvent(this);\" href=\"\">"+word+"</a>";
  if (article) {
    word += " <em>(" + article + ")</em>";
  }
  if (other) {
    word += " <em>" + other + "</em>";
  }

  return "<span>"+word+"</span>";

}

HeinzelSearch.tableAppendTranslations = function(table, trans) {
    trans.sort(function(a, b) {
        return (a.word+"__"+a.article).localeCompare(b.word+"__"+b.article);
    });
    var lastWord = "";
    var tableData = "";
    var td = "";
    var count = 0;
    $.each(trans, function() {
        if (count < 25) {
            var wordId = this.word+"__"+this.article+"__"+this.other;
            if (lastWord == wordId) {
                td += "; " + HeinzelSearch.htmlFormatWord(this.t_word, this.t_article, this.t_other);
            } else {
                count++;
                lastWord = wordId;
                // store last row
                tableData += "<tr class=\"transRow\">" + td + "</td></tr>";
                // new word, new row
                td = "<td><b>" + HeinzelSearch.htmlFormatWord(this.word, this.article, this.other) + "</b> "
                    + HeinzelSearch.htmlFormatWord(this.t_word, this.t_article, this.t_other);
            }
        } else {
            td = "<td>...</td>";
        }
    })
    if (td != "") {
        tableData += "<tr class=\"transRow\">" + td + "</td></tr>";
    }
    table.append(tableData);
}

HeinzelSearch.searchDb = function(item) {
    var results = {deTrans: new Array(),
                   noTrans: new Array()};
    var regex;
    var wordbound = "([^\\t]+(?:^|\\s|-)|)"; // better \b for non-ascii
    // norwegian -> german
    if (HeinzelSearch.Config.exact) {
        regex = new RegExp("^("+HeinzelSearch.regQuote(item)+"\\t).*$", "img");
    } else {
        regex = new RegExp("^("+wordbound+HeinzelSearch.regQuote(item)+").*$", "img");
    }
    while ((line = regex.exec(HeinzelSearch.Config.hndb)) != null) {
        var wordList = line[0].split("\t");
        var trans = {
            word: wordList[0],
            article: wordList[1],
            other: wordList[3],
            t_word: wordList[4],
            t_article: wordList[5],
            t_other: wordList[7]
        };
        results.noTrans.push(trans);
    }
    // german -> norwegian
    if (HeinzelSearch.Config.exact) {
        regex = new RegExp("^([^\\t]*\\t[^\\t]*\\t[^\\t]*\\t[^\\t]*\\t"+HeinzelSearch.regQuote(item)+"\\t).*$", "img");
    } else {
        regex = new RegExp("^([^\\t]*\\t[^\\t]*\\t[^\\t]*\\t[^\\t]*\\t"+wordbound+HeinzelSearch.regQuote(item)+").*$", "img");
    }
    while ((line = regex.exec(HeinzelSearch.Config.hndb)) != null) {
        wordList = line[0].split("\t");
        trans = {
            t_word: wordList[0],
            t_article: wordList[1],
            t_other: wordList[3],
            word: wordList[4],
            article: wordList[5],
            other: wordList[7]
        };
        results.deTrans.push(trans);
    }
    return results;
}

HeinzelSearch.searchEvent = function(element) {
    // change the new a.search elements callback
    var searchItem = $(element).text();
    HeinzelSearch.search(searchItem);
}

HeinzelSearch.displayItemData = function(item, data) {
  $("div.header").hide();
  var results = $("#results");
  var foundTrans = 0;
  results.empty();
  if (data.deTrans && data.deTrans.length > 0) {
    foundTrans += data.deTrans.length;
    var table = $(document.createElement('table'));
    HeinzelSearch.tableAppendTranslations(table, data.deTrans);
    results
        .append($( document.createElement('div') )
            .attr({'data-role': 'collapsible',
                   'data-collapsed': false})
            .append('<h3>DE &rarr; NO</h3>')
            .append(table));
  }
  if (data.noTrans && data.noTrans.length > 0) {
    foundTrans += data.noTrans.length;
    table = $(document.createElement('table'));
    HeinzelSearch.tableAppendTranslations(table, data.noTrans);
    results
        .append($( document.createElement('div') )
            .attr({'data-role': 'collapsible',
                   'data-collapsed': false})
            .append('<h3>NO &rarr; DE</h3>')
            .append(table));
  }
  if (foundTrans == 0) {
      results.append("keine Übersetzungen gefunden");
  }

  if (data.dePhonetics && data.dePhonetics.length > 0) {
    var deUl = $( document.createElement('ul') )
    $.each(data.dePhonetics, function() {
        deUl.append($(document.createElement('li')).append(HeinzelSearch.htmlFormatWord(this.toString())));
    });
    results.append($( document.createElement('div') )
        .attr({'data-role': 'collapsible'})
        .append('<h3>DE ähnlich</h3>')
        .append(deUl));
  }
  if (data.noPhonetics && data.noPhonetics.length > 0) {
    var noUl = $( document.createElement('ul') );
    $.each(data.noPhonetics, function() {
        noUl.append($(document.createElement('li')).append(HeinzelSearch.htmlFormatWord(this.toString())));
    });
    results.append($( document.createElement('div') )
        .attr({'data-role': 'collapsible'})
        .append('<h3>NO ähnlich</h3>')
        .append(noUl));
  }
  
  // WIKI
  if (data.wikiPageNames && data.wikiPageNames.length) {
    var wikiUl = $( document.createElement('ul'));
    $.each(data.wikiPageNames, function() {
        var hRef = "http://www.heinzelnisse.info/wiki/"+this;
        var link = $(document.createElement('a')).attr({rel: 'external',
                                                        href: hRef})
                                                 .append(this.toString());
        wikiUl.append($(document.createElement('li')).append(link));
    });
    results.append($( document.createElement('div') )
        .attr({'data-role': 'collapsible',
               'data-collapsed': false})
        .append('<h3>Wiki</h3>')
        .append(wikiUl));
  }
  if (data.forumQuestions && data.forumQuestions.length) {
    var forumSet = $( document.createElement('div'))
        .attr({'collapsible-set': true});
    $.each(data.forumQuestions, function() {
        var q = $( document.createElement('div') )
            .attr({'data-role': 'collapsible'})
            .addClass('question')
            .append($(document.createElement('h3') ).append(this.keywordString))
            .append($(document.createElement('div'))
                .addClass('userTime')
                .append(this.userName + ", " + this.timestamp))
            .append(this.html);
        if (this.answers && this.answers.length) {
            $.each(this.answers, function() {
                q.append($( document.createElement('div') )
                    .addClass('answer')
                    .append($(document.createElement('div'))
                        .addClass('userTime')
                        .append(this.userName + ", " + this.timestamp))
                    .append(this.html));
            });
        }
        forumSet.append(q);
    });
    results.append(forumSet);
  }

  $("div[data-role='collapsible']").attr({'data-mini': "true",
                                          'data-collapsed-icon': "arrow-r",
                                          'data-expanded-icon': "arrow-d",
                                          'data-iconpos': "right"});
  $("div[data-role='collapsible-set']").collapsibleset('refresh');
  $("div[data-role='collapsible']").collapsible({refresh: true});
  $(".transRow:odd").css("background-color", "#ffffff");

  /* and store in local storage if possilbe */
  if (typeof(Storage)!=="undefined") {
      localStorage.hnLastData = JSON.stringify(data);
      localStorage.hnLastSearchItem = item;
      console.log("defining localStorage: " + localStorage.hnLastSearchItem);
  }
}

HeinzelSearch.deferredUpdateResultTable = function(item) {
  $.Deferred(function(defer) {
    console.log("searching for item: " + item);
    var table = $(document.createElement('table'));
    var data = HeinzelSearch.searchDb(item);
    HeinzelSearch.displayItemData(item, data);
  }).promise();
}

HeinzelSearch.onlineSearch = function(item) {
  $.ajax({
    url: "/searchResults",
    data: {
        searchItem: item,
        dictExactSearch: HeinzelSearch.Config.exact ? "on" : "",
        type: "json",
        setOptions: "true",
        dictDeNoSearch: "on",
        dictNoDeSearch: "on",
        dictPhoneticSearch: "on",
        wikiSearch: "on",
        dictNynorskSearch: "on",
        dictBokmaalSearch: "checked",
        forumKeywordSearch: "on"
    },
    dataType: "json",
    cache: true
  }).done(function(data) {
    console.log(data);
    HeinzelSearch.displayItemData(item, data);
  }).fail(function() {
    alert("error while fetching item: " + item);
  });
}

HeinzelSearch.deferredMakeNice = function() {
  $.Deferred(function (defer) {
    // and make it nice
    HeinzelSearch.Config.hndb = HeinzelSearch.Config.hndb.tr("nMiEdAzYxUoNmIeDaZyXuO","ADEIMNOUXYZadeimnouxyz");
    console.log("hndb made nice: " + HeinzelSearch.Config.hndb.substr(0,20));
    HeinzelSearch.Config.isNice = true;
  }).promise();
}

HeinzelSearch.clear = function() {
  console.log("resetting temporary items");
  HeinzelSearch.Config.lastSearchItem = "";
  if (typeof(Storage)!=="undefined") {
    localStorage.clear();
    console.log("hndb localStorage resetted");
  }
  return;
}

HeinzelSearch.loadHnDB = function () {
  if (HeinzelSearch.Config.hndb.length > 0) return; //hndb defined
  if (typeof(Storage)!=="undefined") {
    if (localStorage.hasHnDB != null) {
      HeinzelSearch.Config.hndb = localStorage.getItem("hnDB");
      HeinzelSearch.Config.isNice = true;
        console.log("hndb loaded from localStorage");
    } else {
        console.log("hndb not found in localStorage");
    }
  }
  return;
}

HeinzelSearch.fetchAndStoreHnDB = function () {  
  if (typeof(Storage)!=="undefined" && !localStorage.hasSmallLocalStorage) {
     var fileref=document.createElement('script');
     fileref.setAttribute("type","text/javascript");
     fileref.setAttribute("src", "https://www.heinzelnisse.info/Downloads/heinzelliste.js");
     fileref.setAttribute("charset", "ISO-8859-1");
     fileref.onload = function() {
      console.log("fetched heinzelliste");
      if (!HeinzelSearch.Config.isNice) {
        $.when(HeinzelSearch.deferredMakeNice());
      }
      try {
        localStorage.hnDB = HeinzelSearch.Config.hndb;
        localStorage.hasHnDB = true;
        localStorage.hasSmallLocalStorage = false;
      } catch (e) {
        if (e.name == 'QUOTA_EXCEEDED_ERR') {
          localStorage.clear();
          localStorage.hasSmallLocalStorage = true;
          HeinzelSearch.disableOfflineUsage();
        }
      }
    };
    document.getElementsByTagName("head")[0].appendChild(fileref);
  } else {
    HeinzelSearch.disableOfflineUsage();
    console.log("offline work not possible");  
  }
}


HeinzelSearch.makeNiceAndSearch = function(item) {
  /* and store in local storage if possilbe */
  HeinzelSearch.loadHnDB();
  if (!HeinzelSearch.Config.isNice) {
    $.when(HeinzelSearch.deferredMakeNice()).done($.when(HeinzelSearch.deferredUpdateResultTable(item)));
  } else {
    $.when(HeinzelSearch.deferredUpdateResultTable(item));
  }
  return;
}


/**
 * @return a promise object
 */
HeinzelSearch.search = function(item) {
  HeinzelSearch.Config.searchDfd.reject();
  if (item == "" || item == HeinzelSearch.Config.lastSearchItem) {
    return HeinzelSearch.Config.searchDfd.promise();
  }
  HeinzelSearch.Config.searchDfd = new $.Deferred(function (defer) {
    HeinzelSearch.Config.lastSearchItem = item;
    //alert("search started for "+item);
    $("#logo").remove();
    $("#results").empty();
    $("#results").append("<b>suche ...</b>");
    //$("input#searchItem").val(item);
    if (HeinzelSearch.Config.online) {
      HeinzelSearch.onlineSearch(item);        
    } else {
      HeinzelSearch.makeNiceAndSearch(item);
    }
  });
  HeinzelSearch.Config.searchDfd.promise();
}

function appCacheStatus() {
  var appCache;
  if ('applicationCache' in window) {
    appCache = window.applicationCache;
  } else {
    return "unknown";
  }
  switch (appCache.status) {
    case appCache.UNCACHED: // UNCACHED == 0
      return 'UNCACHED';
      break;
    case appCache.IDLE: // IDLE == 1
      return 'IDLE';
      break;
    case appCache.CHECKING: // CHECKING == 2
      return 'CHECKING';
      break;
    case appCache.DOWNLOADING: // DOWNLOADING == 3
      return 'DOWNLOADING';
      break;
    case appCache.UPDATEREADY:  // UPDATEREADY == 5
      return 'UPDATEREADY';
      break;
    case appCache.OBSOLETE: // OBSOLETE == 5
      return 'OBSOLETE';
      break;
    default:
      return 'UKNOWN CACHE STATUS';
      break;
  }
}

String.prototype.tr = function(from, to) {
    var fromChars = from.split("");
    var toChars = to.split("");

    var mapTable = {};

    for(i = 0; i < fromChars.length; i++) {
        var c = i < toChars.length ? toChars[i] : "";
        mapTable[fromChars[i]] = c;
    }

    var str = this;

    var re = new RegExp("["+from+"]", "g");
    str = str.replace(re, function(c) {
        return mapTable[c];
    });

    return str;
}

function handlePasteEvent(e) {
  if (e.clipboardData) {
    e.preventDefault();
    var text = e.clipboardData.getData('text/plain');
    console.log('text/plain - ' + text);
    $("input#searchItem").val(text);
    HeinzelSearch.search(text);
  } else {
    console.log('no clipboardData defined');
  }
}


function handleCacheEvent(e) {
  $("#status").empty();
  $("#status").append(appCacheStatus());
}

function handleCacheError(e) {
  alert('Error: Cache failed to update! ');
}

function checkNetworkStatus() {
    if (navigator.onLine) {
        // Just because the browser says we're online doesn't mean we're online. The browser lies.
        // Check to see if we are really online by making a call for a static JSON resource on
        // the originating Web site. If we can get to it, we're online. If not, assume we're
        // offline.
        $.ajaxSetup({
            async: true,
            cache: false,
            dataType: "html",
            error: function (req, status, ex) {
                console.log("Error: " + ex);
                // We might not be technically "offline" if the error is not a timeout, but
                // otherwise we're getting some sort of error when we shouldn't, so we're
                // going to treat it as if we're offline.
                // Note: This might not be totally correct if the error is because the
                // manifest is ill-formed.
                showNetworkStatus(false);
            },
            success: function (data, status, req) {
                showNetworkStatus(true);
            },
            timeout: 5000,
            type: "GET",
            url: "test.html"
        });
        $.ajax();
    }
    else {
        showNetworkStatus(false);
    }
}


function showNetworkStatus(online) {
    if (online) {
        $("select#onlineSearch").val(1).slider('refresh');
        HeinzelSearch.Config.online = true;
    }
    else {
        $("select#onlineSearch").val(0).slider('refresh');
        HeinzelSearch.Config.online = false;
    }
    console.log("Online status: " + HeinzelSearch.Config.online);
}

HeinzelSearch.disableOfflineUsage = function () {
  // no need to show if not possible to switch
  console.log("disabling offline usage");
  $("#onlineSearchSlider").hide(); 
  $("a#manualUpdateHnDB").hide();
    
}

// HERE STARTS THE LOCAL CODE
$(document).ready(function() {

if (typeof(Storage)!=="undefined") {
    try {
      if (localStorage.hasHnDB) {
          // nothing to be done
      } else {
          HeinzelSearch.fetchAndStoreHnDB();
      }
    } catch (e) {
        console.log("problem reading database ")
    }
    try {
        console.log("localStorage supported, trying to restore: " + localStorage.hnLastSearchItem);
        $("input#searchItem").val(localStorage.hnLastSearchItem);
        HeinzelSearch.displayItemData(localStorage.hnLastSearchItem, JSON.parse(localStorage.hnLastData));
        HeinzelSearch.Config.lastSearchItem = localStorage.hnLastSearchItem;
    } catch (e) {
        console.log("problem setting previous session: " + e);
    }
} else {
    console.log("calling disableOfflineUsage();");
    HeinzelSearch.disableOfflineUsage();
}



$("select#onlineSearch").change( function() {
      var onlineSearch = $("select#onlineSearch").val();
      HeinzelSearch.Config.online = (onlineSearch == 1) ? true : false;
      console.log("input change onlineSearch for online: "+HeinzelSearch.Config.online);
    });
$("select#searchExact").change( function() {
      var searchExact = $("select#searchExact").val();
      console.log("input change exact for "+searchExact);
      HeinzelSearch.Config.exact = (searchExact === "checked") ? true : false;
      console.log("HeinzelSearch.Config.exact =  "+HeinzelSearch.Config.exact);
      // repeat the last search
      var searchItem = HeinzelSearch.Config.lastSearchItem;
      HeinzelSearch.Config.lastSearchItem = "";
      HeinzelSearch.search(searchItem);
    });
   
$("input#searchItem").bind("input keyup change", function() {
      var searchItem = $("input#searchItem").val();
      if (searchItem.length > 2) {
        clearTimeout(HeinzelSearch.Config.delayTimer);
        HeinzelSearch.Config.delayTimer = setTimeout(function() {
          console.log("input change search for "+searchItem);
          jQuery.when(HeinzelSearch.search(searchItem));
        }, 300);
      }
    });


$("a#searchButton").click(function(event){
    var searchItem = $("input#searchItem").val();
    console.log("button click search for "+searchItem);
    HeinzelSearch.search(searchItem);
  });

$("a#manualUpdateHnDB").click(function(event) {
    console.log("update offline db");
    HeinzelSearch.fetchAndStoreHnDB();
  });

$("a#clearSettings").click(function(event) {
    console.log("clearSettings");
    HeinzelSearch.clear();
  });


// trim startup functions
var heinzelTrimWordConfig = function() {
  $("input#words").attr("max", HeinzelTrim.Config.lines.length)
    .val(HeinzelTrim.Config.words).slider('refresh');
  $("input#words").bind("change", function(event, ui) {
    HeinzelTrim.Config.words = $("input#words").val();
    if (HeinzelTrim.Config.words < 10 ||
      HeinzelTrim.Config.words > HeinzelTrim.Config.lines.length) {
      HeinzelTrim.Config.words = HeinzelTrim.Config.lines.length;
    }
    console.log("words to use: " + HeinzelTrim.Config.words);
  });
  HeinzelTrim.setNewWord();    
}

$( document ).on( "pageinit", "#trim1", function( event ) {
  HeinzelTrim.loadSet("for_nordmenn_som_egentlig_ikke_kan_tysk", heinzelTrimWordConfig);
});

$("select#trimLangNO").change( function() {
      var langNo = $("select#trimLangNO").val();
      HeinzelTrim.langNo = (langNo === "checked") ? true : false;
      console.log("HeinzelTrim.langNo =  "+HeinzelTrim.langNo);
      HeinzelTrim.setNewWord();
    });


$("input#trimItem").bind("change", function() {
    if ($("input#trimItem").val() == "") return; // 
    HeinzelTrim.checkInput();
    });
$("a#test").bind("change", function() {
    HeinzelTrim.checkInput();
    });

$("a#right").click(function(event){
    HeinzelTrim.update(true);
  });

$("a#wrong").click(function(event){
    HeinzelTrim.update(false);
  });

$("a#retry").click(function(event){
    HeinzelTrim.right = 0;
    HeinzelTrim.wrong = 0;
    $("div#trimResult").empty().append("<table id=\"trimResults\"></table>");
    HeinzelTrim.setNewWord();
    $("div#wordTranslate").show();

  });

$("input[name=radio-vsets]:radio").bind( "change", function(event, ui) {
    console.log('requesting trainer-set: '+$(this).val());
    HeinzelTrim.loadSet($(this).val(), heinzelTrimWordConfig);
});

// check for updates
if ("applicationCache" in window) {
  var appCache = window.applicationCache;
  $("#manualUpdate").click(function(event) {
    event.preventDefault();
    try {
        window.applicationCache.update();
    } catch (e) {}
  });
  $("#updateButton").click(function(event) {
    event.preventDefault();
    try {
        // Ensure the browser uses the latest version of the code
        window.applicationCache.swapCache();
        // Reload the application
        window.location.reload();
    } catch (e) {}
  });
}

window.applicationCache.addEventListener('updateready', function() {
    $("#update").show();
}, false);
  // Fired after the first cache of the manifest.
appCache.addEventListener('cached', handleCacheEvent, false);

// Checking for an update. Always the first event fired in the sequence.
appCache.addEventListener('checking', handleCacheEvent, false);

// An update was found. The browser is fetching resources.
appCache.addEventListener('downloading', handleCacheEvent, false);

// The manifest returns 404 or 410, the download failed,
// or the manifest changed while the download was in progress.
// This will also happen when the app starts in offline-mode
appCache.addEventListener('error', handleCacheEvent, false);

// Fired after the first download of the manifest.
appCache.addEventListener('noupdate', handleCacheEvent, false);

// Fired if the manifest file returns a 404 or 410.
// This results in the application cache being deleted.
appCache.addEventListener('obsolete', handleCacheEvent, false);

// Fired for each resource listed in the manifest as it is being fetched.
appCache.addEventListener('progress', handleCacheEvent, false);

// Fired when the manifest resources have been newly redownloaded.
appCache.addEventListener('updateready', handleCacheEvent, false);

$(document.body).bind("online", checkNetworkStatus);
$(document.body).bind("offline", checkNetworkStatus);
checkNetworkStatus();

});



