/* 
 * heinzelsearch.js
 *
 * search class for heinzelnisse offline app
 *
 * Copyright: (C) 2013 Heiko Klein
 * License: GPL
 *
 */
if (!String.prototype.trim) {
   String.prototype.trim=function(){return this.replace(/^\s+|\s+$/g, '');};
}
if(!window.console){ window.console = {log: function(){} }; }


var HeinzelTrim = {
    langNo: true,
    tries: 10,
    right: 0,
    wrong: 0,
    currentTrans: null
};

HeinzelTrim.Config = {
    lines: [],
    words: 0  // number of words to use, must be <= lines.length
};

HeinzelTrim.loadSet = function (name, func) {
  $.ajax({
    url: "trimSets/"+name+".csv",
    dataType: "text",
    cache: true
  }).done(function(data) {
    HeinzelTrim.Config.lines = [];
    var dataLines = data.split("\n");
    for (var i = 0; i < dataLines.length; ++i) {
      if (dataLines[i].startsWith("#")) continue;
      var lineParts = dataLines[i].split("\t");
      if (lineParts < 3) continue;
      var noWord;
      var deWord;
      if (lineParts[1] == "d") {
        noWord = lineParts[2];
        deWord = lineParts[0];
      } else {
        noWord = lineParts[0];
        deWord = lineParts[2];          
      }
      if (i < 5) {
        console.log(noWord +" -> "+ deWord);
      }
      HeinzelTrim.Config.lines.push(noWord + "\t\t\t\t" + deWord + "\t\t\t");
    }
    HeinzelTrim.Config.words = (HeinzelTrim.Config.lines.length < 25) ? HeinzelTrim.Config.lines.length : 25;
    if (func) {
        // exec optional function
        func();
    }
  }).fail(function() {
    alert("error while fetching item: trimSets/" + name + ".csv");
  });

}

HeinzelTrim.htmlFormatWord = function(word, article, other) {
  if (article) {
    word += " <em>(" + article + ")</em>";
  }
  if (other) {
    word += " <em>" + other + "</em>";
  }

  return "<span>"+word+"</span>";
}

HeinzelTrim.htmlFormatTrans = function(trans, correct) {
  return "<tr><td>"+HeinzelTrim.htmlFormatWord(trans.word, trans.article, trans.other)+"</td><td>"
          +HeinzelTrim.htmlFormatWord(trans.t_word, trans.t_article, trans.t_other)+"</td><td>"
          +(correct ? "+" : "-")+"</tr>";
}

HeinzelTrim.setNewWord = function() {
    var wordList = [];
    while (wordList.length < 5) {
        // use the last entries
        row = HeinzelTrim.Config.lines.length - Math.floor(Math.random() * HeinzelTrim.Config.words) - 1;
        console.log(row + ": " + HeinzelTrim.Config.lines.length);
        wordList = HeinzelTrim.Config.lines[row].split("\t");
    }
    var currentTrans;
    if (HeinzelTrim.langNo) {
        currentTrans = {
            word: wordList[0],
            article: wordList[1],
            other: wordList[3],
            t_word: wordList[4],
            t_article: wordList[5],
            t_other: wordList[7]
        };
    } else {
        currentTrans = {
            t_word: wordList[0],
            t_article: wordList[1],
            t_other: wordList[3],
            word: wordList[4],
            article: wordList[5],
            other: wordList[7]
        };        
    }
    HeinzelTrim.currentTrans = currentTrans;
    
    $("label#word").empty().append(currentTrans.word);
    $("span#translate").empty().append(currentTrans.t_word);
    $("div#transResult").hide();
}

HeinzelTrim.update = function(correct) {
    var tableRow = HeinzelTrim.htmlFormatTrans(HeinzelTrim.currentTrans, correct);
    $("table#trimResults").append(tableRow);
    if (correct) {
        HeinzelTrim.right++;
    } else {
        HeinzelTrim.wrong++;
    }
    $("input#trimItem").val("");
    
    if ((HeinzelTrim.right + HeinzelTrim.wrong) >=  HeinzelTrim.tries) {
        $("div#trimResult").append("Du er ferdig med økten. " +
            HeinzelTrim.right + " av " + HeinzelTrim.tries + " gloser var riktig.");
        $("div#wordTranslate").hide();
    } else {
        HeinzelTrim.setNewWord();
    }
}

HeinzelTrim.checkInput = function() {
  var trimItem = $("input#trimItem").val();
  $("div#transResult").show();
  if (HeinzelTrim.currentTrans.t_word === null) {
    HeinzelTrim.setNewWord();
  }
  if (trimItem.trim().toLowerCase() == HeinzelTrim.currentTrans.t_word.trim().toLowerCase()) {
    HeinzelTrim.update(true);
  } else {
    HeinzelTrim.update(false);
  }    
}
