/* 
 * heinzelsearch.js
 *
 * search class for heinzelnisse offline app
 *
 * Copyright: (C) 2011 Heiko Klein
 * License: GPL
 *
 */


var HeinzelSearch = {};
HeinzelSearch.Config = {
    lastSearchItem: "",
    hndb: "",
    exact: false,
    online: false,
    searchDfd: new jQuery.Deferred(),
    delayTimer: 0
};

HeinzelSearch.getURLParameter = function(name) {
  var reg = new RegExp(name + '=' + '(.+?)(&|$)');
  var result = reg.exec(location.href); //in non ajax-world: location.search
  console.log(result);
  if (result && result[1]) {
    return unescape(result[1]);
  } else {
    return null;
  }
}

HeinzelSearch.htmlFormatWord = function(word, article, other) {
  word = "<a class=\"search\" onclick=\"HeinzelSearch.searchEvent(this);\" href=\"\">"+word+"</a>";
  if (article) {
    word += " <em>(" + article + ")</em>";
  }
  if (other) {
    word += " <em>" + other + "</em>";
  }

  return "<span>"+word+"</span>";

}

HeinzelSearch.tableAppendTranslations = function(table, trans) {
    trans.sort(function(a, b) {
        return (a.word+"__"+a.article).localeCompare(b.word+"__"+b.article);
    });
    var lastWord = "";
    var tableData = "";
    var td = "";
    var count = 0;
    $.each(trans, function() {
        if (count < 25) {
            var wordId = this.word+"__"+this.article+"__"+this.other;
            if (lastWord == wordId) {
                td += "; " + HeinzelSearch.htmlFormatWord(this.t_word, this.t_article, this.t_other);
            } else {
                count++;
                lastWord = wordId;
                // store last row
                tableData += "<tr class=\"transRow\">" + td + "</td></tr>";
                // new word, new row
                td = "<td><b>" + HeinzelSearch.htmlFormatWord(this.word, this.article, this.other) + "</b> "
                    + HeinzelSearch.htmlFormatWord(this.t_word, this.t_article, this.t_other);
            }
        } else {
            td = "<td>...</td>";
        }
    })
    if (td != "") {
        tableData += "<tr class=\"transRow\">" + td + "</td></tr>";
    }
    table.append(tableData);
}

HeinzelSearch.searchDb = function(item) {
    var results = {deTrans: new Array(),
                   noTrans: new Array()};
    var regex;
    // norwegian -> german
    if (HeinzelSearch.Config.exact) {
        regex = new RegExp("^("+item+"\\t).*$", "img");
    } else {
        regex = new RegExp("^("+item+").*$", "img");
    }
    while ((line = regex.exec(HeinzelSearch.Config.hndb)) != null) {
        var wordList = line[0].split("\t");
        var trans = {
            word: wordList[0],
            article: wordList[1],
            other: wordList[3],
            t_word: wordList[4],
            t_article: wordList[5],
            t_other: wordList[7]
        };
        results.noTrans.push(trans);
    }
    // german -> norwegian
    if (HeinzelSearch.Config.exact) {
        regex = new RegExp("^([^\\t]*\\t[^\\t]*\\t[^\\t]*\\t[^\\t]*\\t"+item+"\\t).*$", "img");
    } else {
        regex = new RegExp("^([^\\t]*\\t[^\\t]*\\t[^\\t]*\\t[^\\t]*\\t"+item+").*$", "img");
    }
    while ((line = regex.exec(HeinzelSearch.Config.hndb)) != null) {
        wordList = line[0].split("\t");
        trans = {
            t_word: wordList[0],
            t_article: wordList[1],
            t_other: wordList[3],
            word: wordList[4],
            article: wordList[5],
            other: wordList[7]
        };
        results.deTrans.push(trans);
    }
    return results;
}

HeinzelSearch.searchEvent = function(element) {
    // change the new a.search elements callback
    var searchItem = $(element).text();
    HeinzelSearch.search(searchItem);
}

HeinzelSearch.displayItemData = function(item, data) {
  //$("div.header").hide();
  var results = $("#results");
  var foundTrans = 0;
  results.empty();
  if (data.deTrans && data.deTrans.length > 0) {
    foundTrans += data.deTrans.length;
    var table = $(document.createElement('table'));
    HeinzelSearch.tableAppendTranslations(table, data.deTrans);
    results
        .append($( document.createElement('div') )
            .attr({'data-role': 'collapsible',
                   'data-collapsed': false})
            .append('<h3>EN &rarr; NO</h3>')
            .append(table));
  }
  if (data.noTrans && data.noTrans.length > 0) {
    foundTrans += data.noTrans.length;
    table = $(document.createElement('table'));
    HeinzelSearch.tableAppendTranslations(table, data.noTrans);
    results
        .append($( document.createElement('div') )
            .attr({'data-role': 'collapsible',
                   'data-collapsed': false})
            .append('<h3>NO &rarr; EN</h3>')
            .append(table));
  }
  if (foundTrans == 0) {
      results.append("keine Übersetzungen gefunden");
  }

  $("div[data-role='collapsible']").attr({'data-mini': "true",
                                          'data-collapsed-icon': "arrow-r",
                                          'data-expanded-icon': "arrow-d",
                                          'data-iconpos': "right"});
  $("div[data-role='collapsible-set']").collapsibleset('refresh');
  $("div[data-role='collapsible']").collapsible({refresh: true});
  $(".transRow:odd").css("background-color", "#ffffff");

}

HeinzelSearch.deferredUpdateResultTable = function(item) {
  var dfd = $.Deferred();
  console.log("searching for item: " + item);
  var table = $(document.createElement('table'));
  var data = HeinzelSearch.searchDb(item);
  HeinzelSearch.displayItemData(item, data);
  return dfd.promise();
}

HeinzelSearch.makeNice = function() {
  var dfd = $.Deferred();
  // and make it nice
  HeinzelSearch.Config.hndb = HeinzelSearch.Config.hndb.tr("nMiEdAzYxUoNmIeDaZyXuO","ADEIMNOUXYZadeimnouxyz");
  console.log("hndb made nice: " + HeinzelSearch.Config.hndb.substr(0,20));
  HeinzelSearch.Config.isNice = true;
  return dfd.promise();
}

HeinzelSearch.makeNiceAndSearch = function(item) {
  if (!HeinzelSearch.Config.isNice) {
    $.when(HeinzelSearch.deferredMakeNice(), HeinzelSearch.deferredUpdateResultTable(item));
  } else {
    $.when(HeinzelSearch.deferredUpdateResultTable(item));
  }
  return;
}


/**
 * @return a promise object
 */
HeinzelSearch.search = function(item) {
  HeinzelSearch.Config.searchDfd.reject();
  HeinzelSearch.Config.searchDfd = new jQuery.Deferred();
  if (item == "" || item == HeinzelSearch.Config.lastSearchItem) {
    return HeinzelSearch.Config.searchDfd.promise();
  } else {
    HeinzelSearch.Config.lastSearchItem = item;
  }
  //alert("search started for "+item);
  $("#logo").remove();
  $("#results").empty();
  $("#results").append("<b>suche ...</b>");
  $("input#searchItem").val(item);
  HeinzelSearch.makeNiceAndSearch(item);
  return HeinzelSearch.Config.searchDfd.promise();
}

function appCacheStatus() {
  var appCache;
  if ('applicationCache' in window) {
    appCache = window.applicationCache;
  } else {
    return "unknown";
  }
  switch (appCache.status) {
    case appCache.UNCACHED: // UNCACHED == 0
      return 'UNCACHED';
      break;
    case appCache.IDLE: // IDLE == 1
      return 'IDLE';
      break;
    case appCache.CHECKING: // CHECKING == 2
      return 'CHECKING';
      break;
    case appCache.DOWNLOADING: // DOWNLOADING == 3
      return 'DOWNLOADING';
      break;
    case appCache.UPDATEREADY:  // UPDATEREADY == 5
      return 'UPDATEREADY';
      break;
    case appCache.OBSOLETE: // OBSOLETE == 5
      return 'OBSOLETE';
      break;
    default:
      return 'UKNOWN CACHE STATUS';
      break;
  }
}

String.prototype.tr = function(from, to) {
    var fromChars = from.split("");
    var toChars = to.split("");

    var mapTable = {};

    for(i = 0; i < fromChars.length; i++) {
        var c = i < toChars.length ? toChars[i] : "";
        mapTable[fromChars[i]] = c;
    }

    var str = this;

    var re = new RegExp("["+from+"]", "g");
    str = str.replace(re, function(c) {
        return mapTable[c];
    });

    return str;
}

function handlePasteEvent(e) {
  if (e.clipboardData) {
    e.preventDefault();
    var text = e.clipboardData.getData('text/plain');
    console.log('text/plain - ' + text);
    $("input#searchItem").val(text);
    HeinzelSearch.search(text);
  } else {
    console.log('no clipboardData defined');
  }
}


function handleCacheEvent(e) {
  $("#status").empty();
  $("#status").append(appCacheStatus());
}

function handleCacheError(e) {
  alert('Error: Cache failed to update! ');
}


function showNetworkStatus(online) {
    if (online) {
        $("select#onlineSearch").val(1).slider('refresh');
        HeinzelSearch.Config.online = true;
    }
    else {
        $("select#onlineSearch").val(0).slider('refresh');
        HeinzelSearch.Config.online = false;
    }

    console.log("Online status: " + HeinzelSearch.Config.online);
}


// HERE STARTS THE LOCAL CODE
$(document).ready(function() {

$("select#searchExact").change( function() {
      var searchExact = $("select#searchExact").val();
      console.log("input change exact for "+searchExact);
      HeinzelSearch.Config.exact = (searchExact === "checked") ? true : false;
      console.log("HeinzelSearch.Config.exact =  "+HeinzelSearch.Config.exact);
      // repeat the last search
      var searchItem = HeinzelSearch.Config.lastSearchItem;
      HeinzelSearch.Config.lastSearchItem = "";
      HeinzelSearch.search(searchItem);
    });
$("input#searchItem").bind("input change", function() {
      var searchItem = $("input#searchItem").val();
      if (searchItem.length > 2) {
        clearTimeout(HeinzelSearch.Config.delayTimer);
        HeinzelSearch.Config.delayTimer = setTimeout(function() {
          console.log("input change search for "+searchItem);
          jQuery.when(HeinzelSearch.search(searchItem));
        }, 300);
      }
    });
   
$("a#searchButton").click(function(event){
    var searchItem = $("input#searchItem").val();
    console.log("button click search for "+searchItem);
    if (searchItem == "" || searchItem == HeinzelSearch.Config.lastSearchItem) {
    } else {
      $("#results").empty();
      $("#results").append("<b>suche nach "+searchItem + "<b>");
      HeinzelSearch.search(searchItem);
    }
  });

// trim startup functions
HeinzelTrim.Config.lines = HeinzelSearch.Config.hndb.split("\n");
HeinzelTrim.Config.words = (HeinzelTrim.Config.lines.length < 15) ? HeinzelTrim.Config.lines.length : 15;
$( document ).on( "pageinit", "#trim1", function( event ) {
  $("input#words").attr("max", HeinzelTrim.Config.lines.length)
    .val(HeinzelTrim.Config.words).slider('refresh');
  $("input#words").bind("change", function(event, ui) {
    HeinzelTrim.Config.words = $("input#words").val();
    if (HeinzelTrim.Config.words < 10 ||
        HeinzelTrim.Config.words > HeinzelTrim.Config.lines.length) {
        HeinzelTrim.Config.words = HeinzelTrim.Config.lines.length;
    }
    console.log("words to use: " + HeinzelTrim.Config.words);
    });

});
HeinzelTrim.setNewWord();

$("select#trimLangNO").change( function() {
      var langNo = $("select#trimLangNO").val();
      HeinzelTrim.langNo = (langNo === "checked") ? true : false;
      console.log("HeinzelTrim.langNo =  "+HeinzelTrim.langNo);
      HeinzelTrim.setNewWord();
    });


$("input#trimItem").bind("change", function() {
    if ($("input#trimItem").val() == "") return; // 
    HeinzelTrim.checkInput();
    });
$("a#test").bind("change", function() {
    HeinzelTrim.checkInput();
    });

$("a#right").click(function(event){
    HeinzelTrim.update(true);
  });

$("a#wrong").click(function(event){
    HeinzelTrim.update(false);
  });

$("a#retry").click(function(event){
    HeinzelTrim.right = 0;
    HeinzelTrim.wrong = 0;
    $("div#trimResult").empty().append("<table id=\"trimResults\"></table>");
    HeinzelTrim.setNewWord();
    $("div#wordTranslate").show();

  });



// check for updates
if ("applicationCache" in window) {
  var appCache = window.applicationCache;
  $("#manualUpdate").click(function(event) {
    event.preventDefault();
    try {
        window.applicationCache.update();
    } catch (e) {}
  });
  $("#updateButton").click(function(event) {
    event.preventDefault();
    try {
        // Ensure the browser uses the latest version of the code
        window.applicationCache.swapCache();
        // Reload the application
        window.location.reload();
    } catch (e) {}
  });
}

window.applicationCache.addEventListener('updateready', function() {
    $("#update").show();
}, false);
  // Fired after the first cache of the manifest.
appCache.addEventListener('cached', handleCacheEvent, false);

// Checking for an update. Always the first event fired in the sequence.
appCache.addEventListener('checking', handleCacheEvent, false);

// An update was found. The browser is fetching resources.
appCache.addEventListener('downloading', handleCacheEvent, false);

// The manifest returns 404 or 410, the download failed,
// or the manifest changed while the download was in progress.
// This will also happen when the app starts in offline-mode
appCache.addEventListener('error', handleCacheEvent, false);

// Fired after the first download of the manifest.
appCache.addEventListener('noupdate', handleCacheEvent, false);

// Fired if the manifest file returns a 404 or 410.
// This results in the application cache being deleted.
appCache.addEventListener('obsolete', handleCacheEvent, false);

// Fired for each resource listed in the manifest as it is being fetched.
appCache.addEventListener('progress', handleCacheEvent, false);

// Fired when the manifest resources have been newly redownloaded.
appCache.addEventListener('updateready', handleCacheEvent, false);

});



